@@ -1,5 +1,5 @@
 /* ngettext - retrieve plural form string from message catalog and print it.
-   Copyright (C) 1995-2024 Free Software Foundation, Inc.
+   Copyright (C) 1995-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -14,19 +14,20 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+/* Written by Ulrich Drepper and Bruno Haible.  */
+
+#include <config.h>
 
-#include <getopt.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <locale.h>
+#include <ctype.h>
 #include <errno.h>
 
 #include <error.h>
+#include "options.h"
 #include "attribute.h"
 #include "noreturn.h"
 #include "closeout.h"
@@ -44,36 +45,12 @@
    message catalog.  */
 static int do_expand;
 
-/* Long options.  */
-static const struct option long_options[] =
-{
-  { "context", required_argument, NULL, 'c' },
-  { "domain", required_argument, NULL, 'd' },
-  { "help", no_argument, NULL, 'h' },
-  { "version", no_argument, NULL, 'V' },
-  { NULL, 0, NULL, 0 }
-};
-
 /* Forward declaration of local functions.  */
 _GL_NORETURN_FUNC static void usage (int status);
 
 int
 main (int argc, char *argv[])
 {
-  int optchar;
-  const char *msgid;
-  const char *msgid_plural;
-  const char *count;
-  unsigned long n;
-
-  /* Default values for command line options.  */
-  bool do_help = false;
-  bool do_version = false;
-  const char *domain = getenv ("TEXTDOMAIN");
-  const char *domaindir = getenv ("TEXTDOMAINDIR");
-  const char *context = NULL;
-  do_expand = false;
-
   /* Set program name for message texts.  */
   set_program_name (argv[0]);
 
@@ -88,34 +65,56 @@ main (int argc, char *argv[])
   /* Ensure that write errors on stdout are detected.  */
   atexit (close_stdout);
 
+  /* Default values for command line options.  */
+  bool do_help = false;
+  bool do_version = false;
+  const char *domain = getenv ("TEXTDOMAIN");
+  const char *domaindir = getenv ("TEXTDOMAINDIR");
+  const char *context = NULL;
+  do_expand = false;
+
   /* Parse command line options.  */
-  while ((optchar = getopt_long (argc, argv, "+c:d:eEhV", long_options, NULL))
-         != EOF)
-    switch (optchar)
-    {
-    case '\0':          /* Long option.  */
-      break;
-    case 'c':
-      context = optarg;
-      break;
-    case 'd':
-      domain = optarg;
-      break;
-    case 'e':
-      do_expand = true;
-      break;
-    case 'E':
-      /* Ignore.  Just for compatibility.  */
-      break;
-    case 'h':
-      do_help = true;
-      break;
-    case 'V':
-      do_version = true;
-      break;
-    default:
-      usage (EXIT_FAILURE);
-    }
+  BEGIN_ALLOW_OMITTING_FIELD_INITIALIZERS
+  static const struct program_option options[] =
+  {
+    { "context", 'c', required_argument },
+    { "domain",  'd', required_argument },
+    { "help",    'h', no_argument       },
+    { "version", 'V', no_argument       },
+    { NULL,      'e', no_argument       },
+    { NULL,      'E', no_argument       },
+  };
+  END_ALLOW_OMITTING_FIELD_INITIALIZERS
+  start_options (argc, argv, options, NON_OPTION_TERMINATES_OPTIONS, 0);
+  {
+    int optchar;
+    while ((optchar = get_next_option ()) != -1)
+      switch (optchar)
+        {
+        case '\0':          /* Long option with key == 0.  */
+          break;
+        case 'c':
+          context = optarg;
+          break;
+        case 'd':
+          domain = optarg;
+          break;
+        case 'e':
+          do_expand = true;
+          break;
+        case 'E':
+          /* Ignore.  Just for compatibility.  */
+          break;
+        case 'h':
+          do_help = true;
+          break;
+        case 'V':
+          do_version = true;
+          break;
+        default:
+          usage (EXIT_FAILURE);
+        }
+  }
 
   /* Version information is requested.  */
   if (do_version)
@@ -128,8 +127,9 @@ License GPLv3+: GNU GPL version 3 or later <%s>\n\
 This is free software: you are free to change and redistribute it.\n\
 There is NO WARRANTY, to the extent permitted by law.\n\
 "),
-              "1995-2024", "https://gnu.org/licenses/gpl.html");
-      printf (_("Written by %s.\n"), proper_name ("Ulrich Drepper"));
+              "1995-2026", "https://gnu.org/licenses/gpl.html");
+      printf (_("Written by %s and %s.\n"),
+              proper_name ("Ulrich Drepper"), proper_name ("Bruno Haible"));
       exit (EXIT_SUCCESS);
     }
 
@@ -157,20 +157,22 @@ There is NO WARRANTY, to the extent permitted by law.\n\
     }
 
   /* Now the mandatory command line options.  */
-  msgid = argv[optind++];
-  msgid_plural = argv[optind++];
-  count = argv[optind++];
+  const char *msgid = argv[optind++];
+  const char *msgid_plural = argv[optind++];
+  const char *count = argv[optind++];
 
   if (optind != argc)
     abort ();
 
+  unsigned long n;
   {
     char *endp;
     unsigned long tmp_val;
 
-    errno = 0;
-    tmp_val = strtoul (count, &endp, 10);
-    if (errno == 0 && count[0] != '\0' && endp[0] == '\0')
+    if (isdigit ((unsigned char) count[0])
+        && (errno = 0,
+            tmp_val = strtoul (count, &endp, 10),
+            errno == 0 && endp[0] == '\0'))
       n = tmp_val;
     else
       /* When COUNT is not valid, use plural.  */
@@ -224,6 +226,8 @@ Usage: %s [OPTION] [TEXTDOMAIN] MSGID MSGID-PLURAL COUNT\n\
 Display native language translation of a textual message whose grammatical\n\
 form depends on a number.\n"));
       printf ("\n");
+      printf (_("\
+Options and arguments:\n"));
       /* xgettext: no-wrap */
       printf (_("\
   -d, --domain=TEXTDOMAIN   retrieve translated message from TEXTDOMAIN\n"));
@@ -254,18 +258,18 @@ environment variable TEXTDOMAIN.  If the message catalog is not found in the\n\
 regular directory, another location can be specified with the environment\n\
 variable TEXTDOMAINDIR.\n\
 Standard search directory: %s\n"),
-              getenv ("IN_HELP2MAN") == NULL ? LOCALEDIR : "@localedir@");
+              getenv ("IN_HELP2MAN") == NULL ? relocate (LOCALEDIR) : "@localedir@");
       printf ("\n");
       /* TRANSLATORS: The first placeholder is the web address of the Savannah
          project of this package.  The second placeholder is the bug-reporting
          email address for this package.  Please add _another line_ saying
          "Report translation bugs to <...>\n" with the address for translation
          bugs (typically your translation team's web or email address).  */
-      printf(_("\
+      printf (_("\
 Report bugs in the bug tracker at <%s>\n\
 or by email to <%s>.\n"),
-             "https://savannah.gnu.org/projects/gettext",
-             "bug-gettext@gnu.org");
+              "https://savannah.gnu.org/projects/gettext",
+              "bug-gettext@gnu.org");
     }
 
   exit (status);
