@@ -1,6 +1,6 @@
 /* hash - hashing table processing.
 
-   Copyright (C) 1998-2004, 2006-2007, 2009-2025 Free Software Foundation, Inc.
+   Copyright (C) 1998-2004, 2006-2007, 2009-2026 Free Software Foundation, Inc.
 
    Written by Jim Meyering, 1992.
 
@@ -49,6 +49,8 @@ struct hash_entry
   {
     void *data;
     struct hash_entry *next;
+    /* Invariant: If DATA is NULL, this entry is an unused bucket head,
+       and therefore NEXT is NULL as well.  */
   };
 
 struct hash_table
@@ -161,10 +163,11 @@ hash_get_n_entries (const Hash_table *table)
 size_t
 hash_get_max_bucket_length (const Hash_table *table)
 {
-  struct hash_entry const *bucket;
   size_t max_bucket_length = 0;
 
-  for (bucket = table->bucket; bucket < table->bucket_limit; bucket++)
+  for (struct hash_entry const *bucket = table->bucket;
+       bucket < table->bucket_limit;
+       bucket++)
     {
       if (bucket->data)
         {
@@ -185,11 +188,12 @@ hash_get_max_bucket_length (const Hash_table *table)
 bool
 hash_table_ok (const Hash_table *table)
 {
-  struct hash_entry const *bucket;
   size_t n_buckets_used = 0;
   size_t n_entries = 0;
 
-  for (bucket = table->bucket; bucket < table->bucket_limit; bucket++)
+  for (struct hash_entry const *bucket = table->bucket;
+       bucket < table->bucket_limit;
+       bucket++)
     {
       if (bucket->data)
         {
@@ -243,12 +247,11 @@ void *
 hash_lookup (const Hash_table *table, const void *entry)
 {
   struct hash_entry const *bucket = safe_hasher (table, entry);
-  struct hash_entry const *cursor;
 
   if (bucket->data == NULL)
     return NULL;
 
-  for (cursor = bucket; cursor; cursor = cursor->next)
+  for (struct hash_entry const *cursor = bucket; cursor; cursor = cursor->next)
     if (entry == cursor->data || table->comparator (entry, cursor->data))
       return cursor->data;
 
@@ -260,12 +263,10 @@ hash_lookup (const Hash_table *table, const void *entry)
 void *
 hash_get_first (const Hash_table *table)
 {
-  struct hash_entry const *bucket;
-
   if (table->n_entries == 0)
     return NULL;
 
-  for (bucket = table->bucket; ; bucket++)
+  for (struct hash_entry const *bucket = table->bucket; ; bucket++)
     if (! (bucket < table->bucket_limit))
       abort ();
     else if (bucket->data)
@@ -276,17 +277,18 @@ void *
 hash_get_next (const Hash_table *table, const void *entry)
 {
   struct hash_entry const *bucket = safe_hasher (table, entry);
-  struct hash_entry const *cursor;
 
   /* Find next entry in the same bucket.  */
-  cursor = bucket;
-  do
-    {
-      if (cursor->data == entry && cursor->next)
-        return cursor->next->data;
-      cursor = cursor->next;
-    }
-  while (cursor != NULL);
+  {
+    struct hash_entry const *cursor = bucket;
+    do
+      {
+        if (cursor->data == entry && cursor->next)
+          return cursor->next->data;
+        cursor = cursor->next;
+      }
+    while (cursor != NULL);
+  }
 
   /* Find first entry in any subsequent bucket.  */
   while (++bucket < table->bucket_limit)
@@ -302,14 +304,14 @@ hash_get_entries (const Hash_table *table, void **buffer,
                   size_t buffer_size)
 {
   size_t counter = 0;
-  struct hash_entry const *bucket;
-  struct hash_entry const *cursor;
 
-  for (bucket = table->bucket; bucket < table->bucket_limit; bucket++)
+  for (struct hash_entry const *bucket = table->bucket;
+       bucket < table->bucket_limit;
+       bucket++)
     {
       if (bucket->data)
         {
-          for (cursor = bucket; cursor; cursor = cursor->next)
+          for (struct hash_entry const *cursor = bucket; cursor; cursor = cursor->next)
             {
               if (counter >= buffer_size)
                 return counter;
@@ -326,14 +328,14 @@ hash_do_for_each (const Hash_table *table, Hash_processor processor,
                   void *processor_data)
 {
   size_t counter = 0;
-  struct hash_entry const *bucket;
-  struct hash_entry const *cursor;
 
-  for (bucket = table->bucket; bucket < table->bucket_limit; bucket++)
+  for (struct hash_entry const *bucket = table->bucket;
+       bucket < table->bucket_limit;
+       bucket++)
     {
       if (bucket->data)
         {
-          for (cursor = bucket; cursor; cursor = cursor->next)
+          for (struct hash_entry const *cursor = bucket; cursor; cursor = cursor->next)
             {
               if (! processor (cursor->data, processor_data))
                 return counter;
@@ -385,7 +387,6 @@ static bool
 check_tuning (Hash_table *table)
 {
   const Hash_tuning *tuning = table->tuning;
-  float epsilon;
   if (tuning == &default_tuning)
     return true;
 
@@ -394,7 +395,7 @@ check_tuning (Hash_table *table)
      fail to grow or shrink as they should.  The smallest allocation
      is 11 (due to next_prime's algorithm), so an epsilon of 0.1
      should be good enough.  */
-  epsilon = 0.1f;
+  float epsilon = 0.1f;
 
   if (epsilon < tuning->growth_threshold
       && tuning->growth_threshold < 1 - epsilon
@@ -438,14 +439,12 @@ hash_initialize (size_t candidate, const Hash_tuning *tuning,
                  Hash_hasher hasher, Hash_comparator comparator,
                  Hash_data_freer data_freer)
 {
-  Hash_table *table;
-
   if (hasher == NULL)
     hasher = raw_hasher;
   if (comparator == NULL)
     comparator = raw_comparator;
 
-  table = malloc (sizeof *table);
+  Hash_table *table = malloc (sizeof *table);
   if (table == NULL)
     return NULL;
 
@@ -492,27 +491,26 @@ hash_initialize (size_t candidate, const Hash_tuning *tuning,
 void
 hash_clear (Hash_table *table)
 {
-  struct hash_entry *bucket;
-
-  for (bucket = table->bucket; bucket < table->bucket_limit; bucket++)
+  for (struct hash_entry *bucket = table->bucket;
+       bucket < table->bucket_limit;
+       bucket++)
     {
       if (bucket->data)
         {
-          struct hash_entry *cursor;
-          struct hash_entry *next;
-
           /* Free the bucket overflow.  */
-          for (cursor = bucket->next; cursor; cursor = next)
+          for (struct hash_entry *cursor = bucket->next; cursor; )
             {
               if (table->data_freer)
                 table->data_freer (cursor->data);
               cursor->data = NULL;
 
-              next = cursor->next;
+              struct hash_entry *next = cursor->next;
               /* Relinking is done one entry at a time, as it is to be expected
                  that overflows are either rare or short.  */
               cursor->next = table->free_entry_list;
               table->free_entry_list = cursor;
+
+              cursor = next;
             }
 
           /* Free the bucket head.  */
@@ -530,19 +528,18 @@ hash_clear (Hash_table *table)
 void
 hash_free (Hash_table *table)
 {
-  struct hash_entry *bucket;
-  struct hash_entry *cursor;
-  struct hash_entry *next;
-  int err = errno;
+  int saved_errno = errno;
 
   /* Call the user data_freer function.  */
   if (table->data_freer && table->n_entries)
     {
-      for (bucket = table->bucket; bucket < table->bucket_limit; bucket++)
+      for (struct hash_entry *bucket = table->bucket;
+           bucket < table->bucket_limit;
+           bucket++)
         {
           if (bucket->data)
             {
-              for (cursor = bucket; cursor; cursor = cursor->next)
+              for (struct hash_entry *cursor = bucket; cursor; cursor = cursor->next)
                 table->data_freer (cursor->data);
             }
         }
@@ -555,20 +552,24 @@ hash_free (Hash_table *table)
 #else
 
   /* Free all bucket overflowed entries.  */
-  for (bucket = table->bucket; bucket < table->bucket_limit; bucket++)
+  for (struct hash_entry *bucket = table->bucket;
+       bucket < table->bucket_limit;
+       bucket++)
     {
-      for (cursor = bucket->next; cursor; cursor = next)
+      for (struct hash_entry *cursor = bucket->next; cursor; )
         {
-          next = cursor->next;
+          struct hash_entry *next = cursor->next;
           free (cursor);
+          cursor = next;
         }
     }
 
   /* Also reclaim the internal list of previously freed entries.  */
-  for (cursor = table->free_entry_list; cursor; cursor = next)
+  for (struct hash_entry *cursor = table->free_entry_list; cursor; )
     {
-      next = cursor->next;
+      struct hash_entry *next = cursor->next;
       free (cursor);
+      cursor = next;
     }
 
 #endif
@@ -577,7 +578,7 @@ hash_free (Hash_table *table)
   free (table->bucket);
   free (table);
 
-  errno = err;
+  errno = saved_errno;
 }
 
 /* Insertion and deletion.  */
@@ -629,7 +630,6 @@ find_entry (Hash_table *table, const void *entry,
             struct hash_entry **bucket_head, bool delete)
 {
   struct hash_entry *bucket = safe_hasher (table, entry);
-  struct hash_entry *cursor;
 
   *bucket_head = bucket;
 
@@ -663,7 +663,7 @@ find_entry (Hash_table *table, const void *entry,
     }
 
   /* Scan the bucket overflow.  */
-  for (cursor = bucket; cursor->next; cursor = cursor->next)
+  for (struct hash_entry *cursor = bucket; cursor->next; cursor = cursor->next)
     {
       if (entry == cursor->next->data
           || table->comparator (entry, cursor->next->data))
@@ -697,10 +697,7 @@ find_entry (Hash_table *table, const void *entry,
 static bool
 transfer_entries (Hash_table *dst, Hash_table *src, bool safe)
 {
-  struct hash_entry *bucket;
-  struct hash_entry *cursor;
-  struct hash_entry *next;
-  for (bucket = src->bucket; bucket < src->bucket_limit; bucket++)
+  for (struct hash_entry *bucket = src->bucket; bucket < src->bucket_limit; bucket++)
     if (bucket->data)
       {
         void *data;
@@ -712,12 +709,12 @@ transfer_entries (Hash_table *dst, Hash_table *src, bool safe)
            bucket head, but moving overflow entries first may create
            free entries that can be recycled by the time we finally
            get to the bucket head.  */
-        for (cursor = bucket->next; cursor; cursor = next)
+        for (struct hash_entry *cursor = bucket->next; cursor; )
           {
             data = cursor->data;
             new_bucket = safe_hasher (dst, data);
 
-            next = cursor->next;
+            struct hash_entry *next = cursor->next;
 
             if (new_bucket->data)
               {
@@ -734,37 +731,40 @@ transfer_entries (Hash_table *dst, Hash_table *src, bool safe)
                 dst->n_buckets_used++;
                 free_entry (dst, cursor);
               }
+
+            cursor = next;
           }
         /* Now move the bucket head.  Be sure that if we fail due to
            allocation failure that the src table is in a consistent
            state.  */
         data = bucket->data;
         bucket->next = NULL;
-        if (safe)
-          continue;
-        new_bucket = safe_hasher (dst, data);
-
-        if (new_bucket->data)
+        if (!safe)
           {
-            /* Allocate or recycle an entry, when moving from a bucket
-               header into a bucket overflow.  */
-            struct hash_entry *new_entry = allocate_entry (dst);
+            new_bucket = safe_hasher (dst, data);
 
-            if (new_entry == NULL)
-              return false;
+            if (new_bucket->data)
+              {
+                /* Allocate or recycle an entry, when moving from a bucket
+                   header into a bucket overflow.  */
+                struct hash_entry *new_entry = allocate_entry (dst);
 
-            new_entry->data = data;
-            new_entry->next = new_bucket->next;
-            new_bucket->next = new_entry;
+                if (new_entry == NULL)
+                  return false;
+
+                new_entry->data = data;
+                new_entry->next = new_bucket->next;
+                new_bucket->next = new_entry;
+              }
+            else
+              {
+                /* Move from one bucket header to another.  */
+                new_bucket->data = data;
+                dst->n_buckets_used++;
+              }
+            bucket->data = NULL;
+            src->n_buckets_used--;
           }
-        else
-          {
-            /* Move from one bucket header to another.  */
-            new_bucket->data = data;
-            dst->n_buckets_used++;
-          }
-        bucket->data = NULL;
-        src->n_buckets_used--;
       }
   return true;
 }
@@ -772,15 +772,15 @@ transfer_entries (Hash_table *dst, Hash_table *src, bool safe)
 bool
 hash_rehash (Hash_table *table, size_t candidate)
 {
-  Hash_table storage;
-  Hash_table *new_table;
   size_t new_size = compute_bucket_size (candidate, table->tuning);
 
   if (!new_size)
     return false;
   if (new_size == table->n_buckets)
     return true;
-  new_table = &storage;
+
+  Hash_table storage;
+  Hash_table *new_table = &storage;
   new_table->bucket = calloc (new_size, sizeof *new_table->bucket);
   if (new_table->bucket == NULL)
     return false;
@@ -839,14 +839,14 @@ hash_rehash (Hash_table *table, size_t candidate)
      passes.  Two passes give worse cache performance and takes
      longer, but at this point, we're already out of memory, so slow
      and safe is better than failure.  */
-  int err = errno;
+  int saved_errno = errno;
   table->free_entry_list = new_table->free_entry_list;
   if (! (transfer_entries (table, new_table, true)
          && transfer_entries (table, new_table, false)))
     abort ();
   /* table->n_entries already holds its value.  */
   free (new_table->bucket);
-  errno = err;
+  errno = saved_errno;
   return false;
 }
 
@@ -854,8 +854,6 @@ int
 hash_insert_if_absent (Hash_table *table, void const *entry,
                        void const **matched_ent)
 {
-  void *data;
-  struct hash_entry *bucket;
 
   /* The caller cannot insert a NULL entry, since hash_lookup returns NULL
      to indicate "not found", and find_entry uses "bucket->data == NULL"
@@ -864,7 +862,9 @@ hash_insert_if_absent (Hash_table *table, void const *entry,
     abort ();
 
   /* If there's a matching entry already in the table, return that.  */
-  if ((data = find_entry (table, entry, &bucket, false)) != NULL)
+  struct hash_entry *bucket;
+  void *data = find_entry (table, entry, &bucket, false);
+  if (data != NULL)
     {
       if (matched_ent)
         *matched_ent = data;
@@ -948,10 +948,8 @@ hash_insert (Hash_table *table, void const *entry)
 void *
 hash_remove (Hash_table *table, const void *entry)
 {
-  void *data;
   struct hash_entry *bucket;
-
-  data = find_entry (table, entry, &bucket, true);
+  void *data = find_entry (table, entry, &bucket, true);
   if (!data)
     return NULL;
 
@@ -1012,16 +1010,14 @@ hash_remove (Hash_table *table, const void *entry)
 void
 hash_print (const Hash_table *table)
 {
-  struct hash_entry *bucket = (struct hash_entry *) table->bucket;
-
-  for ( ; bucket < table->bucket_limit; bucket++)
+  for (struct hash_entry *bucket = (struct hash_entry *) table->bucket;
+       bucket < table->bucket_limit;
+       bucket++)
     {
-      struct hash_entry *cursor;
-
       if (bucket)
         printf ("%lu:\n", (unsigned long int) (bucket - table->bucket));
 
-      for (cursor = bucket; cursor; cursor = cursor->next)
+      for (struct hash_entry *cursor = bucket; cursor; cursor = cursor->next)
         {
           char const *s = cursor->data;
           /* FIXME */
