@@ -1,5 +1,5 @@
 /* Test of dirfd() function.
-   Copyright (C) 2023 Free Software Foundation, Inc.
+   Copyright (C) 2023-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -42,6 +42,6 @@ main ()
   int fd = dirfd (d);
   ASSERT (fd >= 0);
 
-  return 0;
+  return test_exit_status;
 #endif
 }
