@@ -1,5 +1,5 @@
 /* Convert wide character to multibyte character.
-   Copyright (C) 2011-2025 Free Software Foundation, Inc.
+   Copyright (C) 2011-2026 Free Software Foundation, Inc.
    Written by Bruno Haible <bruno@clisp.org>, 2011.
 
    This file is free software: you can redistribute it and/or modify
@@ -23,10 +23,9 @@ wctomb (char *s, wchar_t wc)
   else
     {
       mbstate_t state;
-      size_t result;
-
       mbszero (&state);
-      result = wcrtomb (s, wc, &state);
+
+      size_t result = wcrtomb (s, wc, &state);
       if (result == (size_t)-1)
         return -1;
       return result;
