@@ -1,8 +1,8 @@
-/* A Bison parser, made by GNU Bison 3.6.4.  */
+/* A Bison parser, made by GNU Bison 3.8.2.  */
 
 /* Bison implementation for Yacc-like parsers in C
 
-   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2020 Free Software Foundation,
+   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2021 Free Software Foundation,
    Inc.
 
    This program is free software: you can redistribute it and/or modify
@@ -16,7 +16,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 /* As a special exception, you may create a larger work that contains
    part or all of the Bison parser skeleton and distribute that work
@@ -45,11 +45,11 @@
    define necessary library symbols; they are noted "INFRINGES ON
    USER NAME SPACE" below.  */
 
-/* Identify Bison output.  */
-#define YYBISON 1
+/* Identify Bison output, and Bison version.  */
+#define YYBISON 30802
 
-/* Bison version.  */
-#define YYBISON_VERSION "3.6.4"
+/* Bison version string.  */
+#define YYBISON_VERSION "3.8.2"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -69,23 +69,18 @@
 /* First part of user prologue.  */
 #line 19 "cldr-plural.y"
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
+
+#define SB_NO_APPENDF
 #include "unistr.h"
 #include "xalloc.h"
+#include "string-buffer.h"
 
 #include "cldr-plural-exp.h"
-#include "cldr-plural.h"
-
-/* Prototypes for local functions.  */
-static int yylex (YYSTYPE *lval, struct cldr_plural_parse_args *arg);
-static void yyerror (struct cldr_plural_parse_args *arg, const char *str);
 
 /* Allocation of expressions.  */
 
@@ -171,7 +166,26 @@ new_range (struct cldr_plural_operand_ty *start,
   return result;
 }
 
-#line 175 "cldr-plural.tab.c"
+/* Internal state of the Bison-generated parser.  */
+
+struct cldr_plural_parse_args
+{
+  /* The lifetime of cp, cp_end is limited to the cldr_plural_parse
+     invocation.  */
+  const char *cp;
+  const char *cp_end;
+
+  struct cldr_plural_rule_list_ty *result;
+};
+
+#include "cldr-plural.h"
+
+/* Prototypes for local functions, that must come after the rules.  */
+static int yylex (YYSTYPE *lval, struct cldr_plural_parse_args *arg);
+static void yyerror (struct cldr_plural_parse_args *arg, const char *str);
+
+
+#line 189 "cldr-plural.c"
 
 # ifndef YY_CAST
 #  ifdef __cplusplus
@@ -194,70 +208,7 @@ new_range (struct cldr_plural_operand_ty *start,
 #  endif
 # endif
 
-/* Use api.header.include to #include this header
-   instead of duplicating it here.  */
-#ifndef YY_YY_CLDR_PLURAL_TAB_H_INCLUDED
-# define YY_YY_CLDR_PLURAL_TAB_H_INCLUDED
-/* Debug traces.  */
-#ifndef YYDEBUG
-# define YYDEBUG 0
-#endif
-#if YYDEBUG
-extern int yydebug;
-#endif
-
-/* Token kinds.  */
-#ifndef YYTOKENTYPE
-# define YYTOKENTYPE
-  enum yytokentype
-  {
-    YYEMPTY = -2,
-    YYEOF = 0,                     /* "end of file"  */
-    YYerror = 256,                 /* error  */
-    YYUNDEF = 257,                 /* "invalid token"  */
-    AND = 258,                     /* AND  */
-    OR = 259,                      /* OR  */
-    RANGE = 260,                   /* RANGE  */
-    ELLIPSIS = 261,                /* ELLIPSIS  */
-    OTHER = 262,                   /* OTHER  */
-    AT_INTEGER = 263,              /* AT_INTEGER  */
-    AT_DECIMAL = 264,              /* AT_DECIMAL  */
-    KEYWORD = 265,                 /* KEYWORD  */
-    INTEGER = 266,                 /* INTEGER  */
-    DECIMAL = 267,                 /* DECIMAL  */
-    OPERAND = 268                  /* OPERAND  */
-  };
-  typedef enum yytokentype yytoken_kind_t;
-#endif
-
-/* Value type.  */
-#if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
-union YYSTYPE
-{
-#line 129 "cldr-plural.y"
-
-  char *sval;
-  struct cldr_plural_condition_ty *cval;
-  struct cldr_plural_relation_ty *lval;
-  struct cldr_plural_expression_ty *eval;
-  struct cldr_plural_range_ty *gval;
-  struct cldr_plural_operand_ty *oval;
-  struct cldr_plural_range_list_ty *rval;
-  int ival;
-
-#line 249 "cldr-plural.tab.c"
-
-};
-typedef union YYSTYPE YYSTYPE;
-# define YYSTYPE_IS_TRIVIAL 1
-# define YYSTYPE_IS_DECLARED 1
-#endif
-
-
-
-int yyparse (struct cldr_plural_parse_args *arg);
-
-#endif /* !YY_YY_CLDR_PLURAL_TAB_H_INCLUDED  */
+#include "cldr-plural.h"
 /* Symbol kind.  */
 enum yysymbol_kind_t
 {
@@ -343,6 +294,18 @@ typedef int_least16_t yytype_int16;
 typedef short yytype_int16;
 #endif
 
+/* Work around bug in HP-UX 11.23, which defines these macros
+   incorrectly for preprocessor constants.  This workaround can likely
+   be removed in 2023, as HPE has promised support for HP-UX 11.23
+   (aka HP-UX 11i v2) only through the end of 2022; see Table 2 of
+   <https://h20195.www2.hpe.com/V2/getpdf.aspx/4AA4-7673ENW.pdf>.  */
+#ifdef __hpux
+# undef UINT_LEAST8_MAX
+# undef UINT_LEAST16_MAX
+# define UINT_LEAST8_MAX 255
+# define UINT_LEAST16_MAX 65535
+#endif
+
 #if defined __UINT_LEAST8_MAX__ && __UINT_LEAST8_MAX__ <= __INT_MAX__
 typedef __UINT_LEAST8_TYPE__ yytype_uint8;
 #elif (!defined __UINT_LEAST8_MAX__ && defined YY_STDINT_H \
@@ -440,17 +403,23 @@ typedef int yy_state_fast_t;
 
 /* Suppress unused-variable warnings by "using" E.  */
 #if ! defined lint || defined __GNUC__
-# define YYUSE(E) ((void) (E))
+# define YY_USE(E) ((void) (E))
 #else
-# define YYUSE(E) /* empty */
+# define YY_USE(E) /* empty */
 #endif
 
-#if defined __GNUC__ && ! defined __ICC && 407 <= __GNUC__ * 100 + __GNUC_MINOR__
 /* Suppress an incorrect diagnostic about yylval being uninitialized.  */
-# define YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN                            \
+#if defined __GNUC__ && ! defined __ICC && 406 <= __GNUC__ * 100 + __GNUC_MINOR__
+# if __GNUC__ * 100 + __GNUC_MINOR__ < 407
+#  define YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN                           \
+    _Pragma ("GCC diagnostic push")                                     \
+    _Pragma ("GCC diagnostic ignored \"-Wuninitialized\"")
+# else
+#  define YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN                           \
     _Pragma ("GCC diagnostic push")                                     \
     _Pragma ("GCC diagnostic ignored \"-Wuninitialized\"")              \
     _Pragma ("GCC diagnostic ignored \"-Wmaybe-uninitialized\"")
+# endif
 # define YY_IGNORE_MAYBE_UNINITIALIZED_END      \
     _Pragma ("GCC diagnostic pop")
 #else
@@ -620,6 +589,7 @@ union yyalloc
 /* YYNSTATES -- Number of states.  */
 #define YYNSTATES  52
 
+/* YYMAXUTOK -- Last valid token kind.  */
 #define YYMAXUTOK   268
 
 
@@ -664,13 +634,13 @@ static const yytype_int8 yytranslate[] =
 };
 
 #if YYDEBUG
-  /* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
+/* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
 static const yytype_int16 yyrline[] =
 {
-       0,   161,   161,   162,   165,   178,   181,   185,   191,   195,
-     203,   207,   213,   217,   223,   230,   236,   240,   246,   253,
-     256,   257,   260,   261,   264,   266,   267,   269,   270,   273,
-     275,   277,   279
+       0,   175,   175,   176,   179,   192,   195,   199,   205,   209,
+     217,   221,   227,   231,   237,   244,   250,   254,   260,   267,
+     270,   271,   274,   275,   278,   280,   281,   283,   284,   287,
+     289,   291,   293
 };
 #endif
 
@@ -702,17 +672,6 @@ yysymbol_name (yysymbol_kind_t yysymbol)
 }
 #endif
 
-#ifdef YYPRINT
-/* YYTOKNUM[NUM] -- (External) token number corresponding to the
-   (internal) symbol number NUM (which must be that of a token).  */
-static const yytype_int16 yytoknum[] =
-{
-       0,   256,   257,   258,   259,   260,   261,   262,   263,   264,
-     265,   266,   267,   268,    59,    58,    61,    33,    37,    44,
-     126
-};
-#endif
-
 #define YYPACT_NINF (-20)
 
 #define yypact_value_is_default(Yyn) \
@@ -723,8 +682,8 @@ static const yytype_int16 yytoknum[] =
 #define yytable_value_is_error(Yyn) \
   0
 
-  /* YYPACT[STATE-NUM] -- Index in YYTABLE of the portion describing
-     STATE-NUM.  */
+/* YYPACT[STATE-NUM] -- Index in YYTABLE of the portion describing
+   STATE-NUM.  */
 static const yytype_int8 yypact[] =
 {
        3,   -11,    -7,     0,   -20,     4,    -2,   -20,     3,    -9,
@@ -735,9 +694,9 @@ static const yytype_int8 yypact[] =
      -20,   -20
 };
 
-  /* YYDEFACT[STATE-NUM] -- Default reduction number in state STATE-NUM.
-     Performed when YYTABLE does not specify something else to do.  Zero
-     means the default is an error.  */
+/* YYDEFACT[STATE-NUM] -- Default reduction number in state STATE-NUM.
+   Performed when YYTABLE does not specify something else to do.  Zero
+   means the default is an error.  */
 static const yytype_int8 yydefact[] =
 {
        0,     0,     0,     0,     2,    20,     0,     1,     0,     0,
@@ -748,23 +707,23 @@ static const yytype_int8 yydefact[] =
       18,    15
 };
 
-  /* YYPGOTO[NTERM-NUM].  */
+/* YYPGOTO[NTERM-NUM].  */
 static const yytype_int8 yypgoto[] =
 {
      -20,   -20,    21,   -20,    14,    17,   -20,    18,   -19,   -20,
       20,   -20,   -20,    23,   -20,   -20,    22
 };
 
-  /* YYDEFGOTO[NTERM-NUM].  */
+/* YYDEFGOTO[NTERM-NUM].  */
 static const yytype_int8 yydefgoto[] =
 {
-      -1,     3,     4,    13,    14,    15,    16,    40,    41,    42,
+       0,     3,     4,    13,    14,    15,    16,    40,    41,    42,
       10,    11,    24,    20,    21,    34,    22
 };
 
-  /* YYTABLE[YYPACT[STATE-NUM]] -- What to do in state STATE-NUM.  If
-     positive, shift that token.  If negative, reduce the rule whose
-     number is the opposite.  If YYTABLE_NINF, syntax error.  */
+/* YYTABLE[YYPACT[STATE-NUM]] -- What to do in state STATE-NUM.  If
+   positive, shift that token.  If negative, reduce the rule whose
+   number is the opposite.  If YYTABLE_NINF, syntax error.  */
 static const yytype_int8 yytable[] =
 {
        7,    46,    18,    19,     5,    26,    18,    19,     6,     9,
@@ -785,8 +744,8 @@ static const yytype_int8 yycheck[] =
       -1,    -1,    -1,    -1,    -1,    33
 };
 
-  /* YYSTOS[STATE-NUM] -- The (internal number of the) accessing
-     symbol of state STATE-NUM.  */
+/* YYSTOS[STATE-NUM] -- The symbol kind of the accessing symbol of
+   state STATE-NUM.  */
 static const yytype_int8 yystos[] =
 {
        0,     7,    10,    22,    23,    15,    15,     0,    14,     8,
@@ -797,7 +756,7 @@ static const yytype_int8 yystos[] =
       11,    29
 };
 
-  /* YYR1[YYN] -- Symbol number of symbol that rule YYN derives.  */
+/* YYR1[RULE-NUM] -- Symbol kind of the left-hand side of rule RULE-NUM.  */
 static const yytype_int8 yyr1[] =
 {
        0,    21,    22,    22,    23,    23,    24,    24,    25,    25,
@@ -806,7 +765,7 @@ static const yytype_int8 yyr1[] =
       37,    37,    37
 };
 
-  /* YYR2[YYN] -- Number of symbols on the right hand side of rule YYN.  */
+/* YYR2[RULE-NUM] -- Number of symbols on the right-hand side of rule RULE-NUM.  */
 static const yytype_int8 yyr2[] =
 {
        0,     2,     1,     3,     4,     3,     1,     3,     1,     3,
@@ -824,6 +783,7 @@ enum { YYENOMEM = -2 };
 #define YYACCEPT        goto yyacceptlab
 #define YYABORT         goto yyabortlab
 #define YYERROR         goto yyerrorlab
+#define YYNOMEM         goto yyexhaustedlab
 
 
 #define YYRECOVERING()  (!!yyerrstatus)
@@ -864,10 +824,7 @@ do {                                            \
     YYFPRINTF Args;                             \
 } while (0)
 
-/* This macro is provided for backward compatibility. */
-# ifndef YY_LOCATION_PRINT
-#  define YY_LOCATION_PRINT(File, Loc) ((void) 0)
-# endif
+
 
 
 # define YY_SYMBOL_PRINT(Title, Kind, Value, Location)                    \
@@ -891,16 +848,12 @@ yy_symbol_value_print (FILE *yyo,
                        yysymbol_kind_t yykind, YYSTYPE const * const yyvaluep, struct cldr_plural_parse_args *arg)
 {
   FILE *yyoutput = yyo;
-  YYUSE (yyoutput);
-  YYUSE (arg);
+  YY_USE (yyoutput);
+  YY_USE (arg);
   if (!yyvaluep)
     return;
-# ifdef YYPRINT
-  if (yykind < YYNTOKENS)
-    YYPRINT (yyo, yytoknum[yykind], *yyvaluep);
-# endif
   YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
-  YYUSE (yykind);
+  YY_USE (yykind);
   YY_IGNORE_MAYBE_UNINITIALIZED_END
 }
 
@@ -1014,8 +967,8 @@ static void
 yydestruct (const char *yymsg,
             yysymbol_kind_t yykind, YYSTYPE *yyvaluep, struct cldr_plural_parse_args *arg)
 {
-  YYUSE (yyvaluep);
-  YYUSE (arg);
+  YY_USE (yyvaluep);
+  YY_USE (arg);
   if (!yymsg)
     yymsg = "Deleting";
   YY_SYMBOL_PRINT (yymsg, yykind, yyvaluep, yylocationp);
@@ -1023,70 +976,70 @@ yydestruct (const char *yymsg,
   YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
   switch (yykind)
     {
-    case 10: /* KEYWORD  */
-#line 140 "cldr-plural.y"
+    case YYSYMBOL_KEYWORD: /* KEYWORD  */
+#line 154 "cldr-plural.y"
             { free (((*yyvaluep).sval)); }
-#line 1030 "cldr-plural.tab.c"
+#line 983 "cldr-plural.c"
         break;
 
-    case 11: /* INTEGER  */
-#line 145 "cldr-plural.y"
+    case YYSYMBOL_INTEGER: /* INTEGER  */
+#line 159 "cldr-plural.y"
             { free (((*yyvaluep).oval)); }
-#line 1036 "cldr-plural.tab.c"
+#line 989 "cldr-plural.c"
         break;
 
-    case 12: /* DECIMAL  */
-#line 145 "cldr-plural.y"
+    case YYSYMBOL_DECIMAL: /* DECIMAL  */
+#line 159 "cldr-plural.y"
             { free (((*yyvaluep).oval)); }
-#line 1042 "cldr-plural.tab.c"
+#line 995 "cldr-plural.c"
         break;
 
-    case 13: /* OPERAND  */
-#line 147 "cldr-plural.y"
+    case YYSYMBOL_OPERAND: /* OPERAND  */
+#line 161 "cldr-plural.y"
             { }
-#line 1048 "cldr-plural.tab.c"
+#line 1001 "cldr-plural.c"
         break;
 
-    case 24: /* condition  */
-#line 141 "cldr-plural.y"
+    case YYSYMBOL_condition: /* condition  */
+#line 155 "cldr-plural.y"
             { cldr_plural_condition_free (((*yyvaluep).cval)); }
-#line 1054 "cldr-plural.tab.c"
+#line 1007 "cldr-plural.c"
         break;
 
-    case 25: /* and_condition  */
-#line 141 "cldr-plural.y"
+    case YYSYMBOL_and_condition: /* and_condition  */
+#line 155 "cldr-plural.y"
             { cldr_plural_condition_free (((*yyvaluep).cval)); }
-#line 1060 "cldr-plural.tab.c"
+#line 1013 "cldr-plural.c"
         break;
 
-    case 26: /* relation  */
-#line 142 "cldr-plural.y"
+    case YYSYMBOL_relation: /* relation  */
+#line 156 "cldr-plural.y"
             { cldr_plural_relation_free (((*yyvaluep).lval)); }
-#line 1066 "cldr-plural.tab.c"
+#line 1019 "cldr-plural.c"
         break;
 
-    case 27: /* expression  */
-#line 143 "cldr-plural.y"
+    case YYSYMBOL_expression: /* expression  */
+#line 157 "cldr-plural.y"
             { free (((*yyvaluep).eval)); }
-#line 1072 "cldr-plural.tab.c"
+#line 1025 "cldr-plural.c"
         break;
 
-    case 28: /* range_list  */
-#line 146 "cldr-plural.y"
+    case YYSYMBOL_range_list: /* range_list  */
+#line 160 "cldr-plural.y"
             { cldr_plural_range_list_free (((*yyvaluep).rval)); }
-#line 1078 "cldr-plural.tab.c"
+#line 1031 "cldr-plural.c"
         break;
 
-    case 29: /* range_or_integer  */
-#line 144 "cldr-plural.y"
+    case YYSYMBOL_range_or_integer: /* range_or_integer  */
+#line 158 "cldr-plural.y"
             { cldr_plural_range_free (((*yyvaluep).gval)); }
-#line 1084 "cldr-plural.tab.c"
+#line 1037 "cldr-plural.c"
         break;
 
-    case 30: /* range  */
-#line 144 "cldr-plural.y"
+    case YYSYMBOL_range: /* range  */
+#line 158 "cldr-plural.y"
             { cldr_plural_range_free (((*yyvaluep).gval)); }
-#line 1090 "cldr-plural.tab.c"
+#line 1043 "cldr-plural.c"
         break;
 
       default:
@@ -1107,7 +1060,7 @@ yydestruct (const char *yymsg,
 int
 yyparse (struct cldr_plural_parse_args *arg)
 {
-/* The lookahead symbol.  */
+/* Lookahead token kind.  */
 int yychar;
 
 
@@ -1118,36 +1071,32 @@ YY_INITIAL_VALUE (static YYSTYPE yyval_default;)
 YYSTYPE yylval YY_INITIAL_VALUE (= yyval_default);
 
     /* Number of syntax errors so far.  */
-    int yynerrs;
+    int yynerrs = 0;
 
-    yy_state_fast_t yystate;
+    yy_state_fast_t yystate = 0;
     /* Number of tokens to shift before error messages enabled.  */
-    int yyerrstatus;
+    int yyerrstatus = 0;
 
-    /* The stacks and their tools:
-       'yyss': related to states.
-       'yyvs': related to semantic values.
-
-       Refer to the stacks through separate pointers, to allow yyoverflow
+    /* Refer to the stacks through separate pointers, to allow yyoverflow
        to reallocate them elsewhere.  */
 
     /* Their size.  */
-    YYPTRDIFF_T yystacksize;
+    YYPTRDIFF_T yystacksize = YYINITDEPTH;
 
-    /* The state stack.  */
+    /* The state stack: array, bottom, top.  */
     yy_state_t yyssa[YYINITDEPTH];
-    yy_state_t *yyss;
-    yy_state_t *yyssp;
+    yy_state_t *yyss = yyssa;
+    yy_state_t *yyssp = yyss;
 
-    /* The semantic value stack.  */
+    /* The semantic value stack: array, bottom, top.  */
     YYSTYPE yyvsa[YYINITDEPTH];
-    YYSTYPE *yyvs;
-    YYSTYPE *yyvsp;
+    YYSTYPE *yyvs = yyvsa;
+    YYSTYPE *yyvsp = yyvs;
 
   int yyn;
   /* The return value of yyparse.  */
   int yyresult;
-  /* Lookahead token as an internal (translated) token number.  */
+  /* Lookahead symbol kind.  */
   yysymbol_kind_t yytoken = YYSYMBOL_YYEMPTY;
   /* The variables used to return semantic value and location from the
      action routines.  */
@@ -1161,18 +1110,10 @@ YYSTYPE yylval YY_INITIAL_VALUE (= yyval_default);
      Keep to zero when no symbol should be popped.  */
   int yylen = 0;
 
-  yynerrs = 0;
-  yystate = 0;
-  yyerrstatus = 0;
-
-  yystacksize = YYINITDEPTH;
-  yyssp = yyss = yyssa;
-  yyvsp = yyvs = yyvsa;
-
-
   YYDPRINTF ((stderr, "Starting parse\n"));
 
   yychar = YYEMPTY; /* Cause a token to be read.  */
+
   goto yysetstate;
 
 
@@ -1198,7 +1139,7 @@ yysetstate:
 
   if (yyss + yystacksize - 1 <= yyssp)
 #if !defined yyoverflow && !defined YYSTACK_RELOCATE
-    goto yyexhaustedlab;
+    YYNOMEM;
 #else
     {
       /* Get the current used size of the three stacks, in elements.  */
@@ -1226,7 +1167,7 @@ yysetstate:
 # else /* defined YYSTACK_RELOCATE */
       /* Extend the stack our own way.  */
       if (YYMAXDEPTH <= yystacksize)
-        goto yyexhaustedlab;
+        YYNOMEM;
       yystacksize *= 2;
       if (YYMAXDEPTH < yystacksize)
         yystacksize = YYMAXDEPTH;
@@ -1237,7 +1178,7 @@ yysetstate:
           YY_CAST (union yyalloc *,
                    YYSTACK_ALLOC (YY_CAST (YYSIZE_T, YYSTACK_BYTES (yystacksize))));
         if (! yyptr)
-          goto yyexhaustedlab;
+          YYNOMEM;
         YYSTACK_RELOCATE (yyss_alloc, yyss);
         YYSTACK_RELOCATE (yyvs_alloc, yyvs);
 #  undef YYSTACK_RELOCATE
@@ -1259,6 +1200,7 @@ yysetstate:
     }
 #endif /* !defined yyoverflow && !defined YYSTACK_RELOCATE */
 
+
   if (yystate == YYFINAL)
     YYACCEPT;
 
@@ -1370,8 +1312,8 @@ yyreduce:
   YY_REDUCE_PRINT (yyn);
   switch (yyn)
     {
-  case 4:
-#line 166 "cldr-plural.y"
+  case 4: /* rule: KEYWORD ':' condition samples  */
+#line 180 "cldr-plural.y"
         {
           struct cldr_plural_rule_ty *rule = new_rule ((yyvsp[-3].sval), (yyvsp[-1].cval));
           struct cldr_plural_rule_list_ty *result = arg->result;
@@ -1384,144 +1326,144 @@ yyreduce:
             }
           result->items[result->nitems++] = rule;
         }
-#line 1388 "cldr-plural.tab.c"
+#line 1330 "cldr-plural.c"
     break;
 
-  case 6:
-#line 182 "cldr-plural.y"
+  case 6: /* condition: and_condition  */
+#line 196 "cldr-plural.y"
         {
           (yyval.cval) = (yyvsp[0].cval);
         }
-#line 1396 "cldr-plural.tab.c"
+#line 1338 "cldr-plural.c"
     break;
 
-  case 7:
-#line 186 "cldr-plural.y"
+  case 7: /* condition: condition OR and_condition  */
+#line 200 "cldr-plural.y"
         {
           (yyval.cval) = new_branch_condition (CLDR_PLURAL_CONDITION_OR, (yyvsp[-2].cval), (yyvsp[0].cval));
         }
-#line 1404 "cldr-plural.tab.c"
+#line 1346 "cldr-plural.c"
     break;
 
-  case 8:
-#line 192 "cldr-plural.y"
+  case 8: /* and_condition: relation  */
+#line 206 "cldr-plural.y"
         {
           (yyval.cval) = new_leaf_condition ((yyvsp[0].lval));
         }
-#line 1412 "cldr-plural.tab.c"
+#line 1354 "cldr-plural.c"
     break;
 
-  case 9:
-#line 196 "cldr-plural.y"
+  case 9: /* and_condition: and_condition AND relation  */
+#line 210 "cldr-plural.y"
         {
           (yyval.cval) = new_branch_condition (CLDR_PLURAL_CONDITION_AND,
                                      (yyvsp[-2].cval),
                                      new_leaf_condition ((yyvsp[0].lval)));
         }
-#line 1422 "cldr-plural.tab.c"
+#line 1364 "cldr-plural.c"
     break;
 
-  case 10:
-#line 204 "cldr-plural.y"
+  case 10: /* relation: expression '=' range_list  */
+#line 218 "cldr-plural.y"
         {
           (yyval.lval) = new_relation ((yyvsp[-2].eval), CLDR_PLURAL_RELATION_EQUAL, (yyvsp[0].rval));
         }
-#line 1430 "cldr-plural.tab.c"
+#line 1372 "cldr-plural.c"
     break;
 
-  case 11:
-#line 208 "cldr-plural.y"
+  case 11: /* relation: expression '!' range_list  */
+#line 222 "cldr-plural.y"
         {
           (yyval.lval) = new_relation ((yyvsp[-2].eval), CLDR_PLURAL_RELATION_NOT_EQUAL, (yyvsp[0].rval));
         }
-#line 1438 "cldr-plural.tab.c"
+#line 1380 "cldr-plural.c"
     break;
 
-  case 12:
-#line 214 "cldr-plural.y"
+  case 12: /* expression: OPERAND  */
+#line 228 "cldr-plural.y"
         {
           (yyval.eval) = new_expression ((yyvsp[0].ival), 0);
         }
-#line 1446 "cldr-plural.tab.c"
+#line 1388 "cldr-plural.c"
     break;
 
-  case 13:
-#line 218 "cldr-plural.y"
+  case 13: /* expression: OPERAND '%' INTEGER  */
+#line 232 "cldr-plural.y"
         {
           (yyval.eval) = new_expression ((yyvsp[-2].ival), (yyvsp[0].oval)->value.ival);
         }
-#line 1454 "cldr-plural.tab.c"
+#line 1396 "cldr-plural.c"
     break;
 
-  case 14:
-#line 224 "cldr-plural.y"
+  case 14: /* range_list: range_or_integer  */
+#line 238 "cldr-plural.y"
         {
           struct cldr_plural_range_list_ty *ranges =
             XMALLOC (struct cldr_plural_range_list_ty);
           memset (ranges, 0, sizeof (struct cldr_plural_range_list_ty));
           (yyval.rval) = add_range (ranges, (yyvsp[0].gval));
         }
-#line 1465 "cldr-plural.tab.c"
+#line 1407 "cldr-plural.c"
     break;
 
-  case 15:
-#line 231 "cldr-plural.y"
+  case 15: /* range_list: range_list ',' range_or_integer  */
+#line 245 "cldr-plural.y"
         {
           (yyval.rval) = add_range ((yyvsp[-2].rval), (yyvsp[0].gval));
         }
-#line 1473 "cldr-plural.tab.c"
+#line 1415 "cldr-plural.c"
     break;
 
-  case 16:
-#line 237 "cldr-plural.y"
+  case 16: /* range_or_integer: range  */
+#line 251 "cldr-plural.y"
         {
           (yyval.gval) = (yyvsp[0].gval);
         }
-#line 1481 "cldr-plural.tab.c"
+#line 1423 "cldr-plural.c"
     break;
 
-  case 17:
-#line 241 "cldr-plural.y"
+  case 17: /* range_or_integer: INTEGER  */
+#line 255 "cldr-plural.y"
         {
           (yyval.gval) = new_range ((yyvsp[0].oval), (yyvsp[0].oval));
         }
-#line 1489 "cldr-plural.tab.c"
+#line 1431 "cldr-plural.c"
     break;
 
-  case 18:
-#line 247 "cldr-plural.y"
+  case 18: /* range: INTEGER RANGE INTEGER  */
+#line 261 "cldr-plural.y"
         {
           (yyval.gval) = new_range ((yyvsp[-2].oval), (yyvsp[0].oval));
         }
-#line 1497 "cldr-plural.tab.c"
+#line 1439 "cldr-plural.c"
     break;
 
-  case 29:
-#line 274 "cldr-plural.y"
+  case 29: /* sample_range: DECIMAL  */
+#line 288 "cldr-plural.y"
         { free ((yyvsp[0].oval)); }
-#line 1503 "cldr-plural.tab.c"
+#line 1445 "cldr-plural.c"
     break;
 
-  case 30:
-#line 276 "cldr-plural.y"
+  case 30: /* sample_range: DECIMAL '~' DECIMAL  */
+#line 290 "cldr-plural.y"
         { free ((yyvsp[-2].oval)); free ((yyvsp[0].oval)); }
-#line 1509 "cldr-plural.tab.c"
+#line 1451 "cldr-plural.c"
     break;
 
-  case 31:
-#line 278 "cldr-plural.y"
+  case 31: /* sample_range: INTEGER  */
+#line 292 "cldr-plural.y"
         { free ((yyvsp[0].oval)); }
-#line 1515 "cldr-plural.tab.c"
+#line 1457 "cldr-plural.c"
     break;
 
-  case 32:
-#line 280 "cldr-plural.y"
+  case 32: /* sample_range: INTEGER '~' INTEGER  */
+#line 294 "cldr-plural.y"
         { free ((yyvsp[-2].oval)); free ((yyvsp[0].oval)); }
-#line 1521 "cldr-plural.tab.c"
+#line 1463 "cldr-plural.c"
     break;
 
 
-#line 1525 "cldr-plural.tab.c"
+#line 1467 "cldr-plural.c"
 
       default: break;
     }
@@ -1603,6 +1545,7 @@ yyerrorlab:
      label yyerrorlab therefore never appears in user code.  */
   if (0)
     YYERROR;
+  ++yynerrs;
 
   /* Do not reclaim the symbols of the rule whose action triggered
      this YYERROR.  */
@@ -1663,7 +1606,7 @@ yyerrlab1:
 `-------------------------------------*/
 yyacceptlab:
   yyresult = 0;
-  goto yyreturn;
+  goto yyreturnlab;
 
 
 /*-----------------------------------.
@@ -1671,24 +1614,22 @@ yyacceptlab:
 `-----------------------------------*/
 yyabortlab:
   yyresult = 1;
-  goto yyreturn;
+  goto yyreturnlab;
 
 
-#if !defined yyoverflow
-/*-------------------------------------------------.
-| yyexhaustedlab -- memory exhaustion comes here.  |
-`-------------------------------------------------*/
+/*-----------------------------------------------------------.
+| yyexhaustedlab -- YYNOMEM (memory exhaustion) comes here.  |
+`-----------------------------------------------------------*/
 yyexhaustedlab:
   yyerror (arg, YY_("memory exhausted"));
   yyresult = 2;
-  /* Fall through.  */
-#endif
+  goto yyreturnlab;
 
 
-/*-----------------------------------------------------.
-| yyreturn -- parsing is finished, return the result.  |
-`-----------------------------------------------------*/
-yyreturn:
+/*----------------------------------------------------------.
+| yyreturnlab -- parsing is finished, clean up and return.  |
+`----------------------------------------------------------*/
+yyreturnlab:
   if (yychar != YYEMPTY)
     {
       /* Make sure we have latest lookahead translation.  See comments at
@@ -1715,9 +1656,11 @@ yyreturn:
   return yyresult;
 }
 
-#line 283 "cldr-plural.y"
+#line 297 "cldr-plural.y"
 
 
+/* Functions invoked by the Bison-generated parser.  */
+
 static int
 yylex (YYSTYPE *lval, struct cldr_plural_parse_args *arg)
 {
@@ -1725,9 +1668,6 @@ yylex (YYSTYPE *lval, struct cldr_plural_parse_args *arg)
   ucs4_t uc;
   int length;
   int result;
-  static char *buffer;
-  static size_t bufmax;
-  size_t bufpos;
 
   while (1)
     {
@@ -1749,27 +1689,27 @@ yylex (YYSTYPE *lval, struct cldr_plural_parse_args *arg)
       arg->cp = exp + length;
       return ELLIPSIS;
     }
-  else if (strncmp ("...", exp, 3) == 0)
+  else if (str_startswith (exp, "..."))
     {
       arg->cp = exp + 3;
       return ELLIPSIS;
     }
-  else if (strncmp ("..", exp, 2) == 0)
+  else if (str_startswith (exp, ".."))
     {
       arg->cp = exp + 2;
       return RANGE;
     }
-  else if (strncmp ("other", exp, 5) == 0)
+  else if (str_startswith (exp, "other"))
     {
       arg->cp = exp + 5;
       return OTHER;
     }
-  else if (strncmp ("@integer", exp, 8) == 0)
+  else if (str_startswith (exp, "@integer"))
     {
       arg->cp = exp + 8;
       return AT_INTEGER;
     }
-  else if (strncmp ("@decimal", exp, 8) == 0)
+  else if (str_startswith (exp, "@decimal"))
     {
       arg->cp = exp + 8;
       return AT_DECIMAL;
@@ -1820,67 +1760,65 @@ yylex (YYSTYPE *lval, struct cldr_plural_parse_args *arg)
     case 'h': case 'i': case 'j': case 'k': case 'l': case 'm': case 'n':
     case 'o': case 'p': case 'q': case 'r': case 's': case 't': case 'u':
     case 'v': case 'w': case 'x': case 'y': case 'z':
-      bufpos = 0;
-      for (;;)
-        {
-          if (bufpos >= bufmax)
-            {
-              bufmax = 2 * bufmax + 10;
-              buffer = xrealloc (buffer, bufmax);
-            }
-          buffer[bufpos++] = result;
-          result = *exp;
-          switch (result)
-            {
-            case 'a': case 'b': case 'c': case 'd': case 'e':
-            case 'f': case 'g': case 'h': case 'i': case 'j':
-            case 'k': case 'l': case 'm': case 'n': case 'o':
-            case 'p': case 'q': case 'r': case 's': case 't':
-            case 'u': case 'v': case 'w': case 'x': case 'y':
-            case 'z':
-              ++exp;
-              continue;
-            default:
-              break;
-            }
-          break;
-        }
+      {
+        struct string_buffer buffer;
 
-      if (bufpos >= bufmax)
-        {
-          bufmax = 2 * bufmax + 10;
-          buffer = xrealloc (buffer, bufmax);
-        }
-      buffer[bufpos] = '\0';
+        sb_init (&buffer);
+        for (;;)
+          {
+            sb_xappend1 (&buffer, result);
+            result = *exp;
+            switch (result)
+              {
+              case 'a': case 'b': case 'c': case 'd': case 'e':
+              case 'f': case 'g': case 'h': case 'i': case 'j':
+              case 'k': case 'l': case 'm': case 'n': case 'o':
+              case 'p': case 'q': case 'r': case 's': case 't':
+              case 'u': case 'v': case 'w': case 'x': case 'y':
+              case 'z':
+                ++exp;
+                continue;
+              default:
+                break;
+              }
+            break;
+          }
 
-      /* Operands.  */
-      if (bufpos == 1)
-        {
-          switch (buffer[0])
-            {
-            case 'n': case 'i': case 'f': case 't': case 'v': case 'w':
-              arg->cp = exp;
-              lval->ival = buffer[0];
-              return OPERAND;
-            default:
-              break;
-            }
-        }
+        const char *ident = sb_xcontents_c (&buffer);
 
-      /* Keywords.  */
-      if (strcmp (buffer, "and") == 0)
-        {
-          arg->cp = exp;
-          return AND;
-        }
-      else if (strcmp (buffer, "or") == 0)
-        {
-          arg->cp = exp;
-          return OR;
-        }
+        /* Operands.  */
+        if (strlen (ident) == 1)
+          {
+            switch (ident[0])
+              {
+              // See https://unicode.org/reports/tr35/tr35-numbers.html#table-plural-operand-meanings
+              case 'n': case 'i': case 'f': case 't': case 'v': case 'w': case 'c': case 'e':
+                arg->cp = exp;
+                lval->ival = ident[0];
+                sb_free (&buffer);
+                return OPERAND;
+              default:
+                break;
+              }
+          }
 
-      lval->sval = xstrdup (buffer);
-      result = KEYWORD;
+        /* Keywords.  */
+        if (strcmp (ident, "and") == 0)
+          {
+            arg->cp = exp;
+            sb_free (&buffer);
+            return AND;
+          }
+        else if (strcmp (ident, "or") == 0)
+          {
+            arg->cp = exp;
+            sb_free (&buffer);
+            return OR;
+          }
+
+        lval->sval = sb_xdupfree_c (&buffer);
+        result = KEYWORD;
+      }
       break;
     case '!':
       if (exp[0] == '=')
@@ -1905,3 +1843,22 @@ yyerror (struct cldr_plural_parse_args *arg, char const *s)
 {
   fprintf (stderr, "%s\n", s);
 }
+
+/* Entry point to the parser.  */
+
+struct cldr_plural_rule_list_ty *
+cldr_plural_parse (const char *input)
+{
+  struct cldr_plural_parse_args arg;
+
+  memset (&arg, 0, sizeof (struct cldr_plural_parse_args));
+  arg.cp = input;
+  arg.cp_end = input + strlen (input);
+  arg.result = XMALLOC (struct cldr_plural_rule_list_ty);
+  memset (arg.result, 0, sizeof (struct cldr_plural_rule_list_ty));
+
+  if (yyparse (&arg) != 0)
+    return NULL;
+
+  return arg.result;
+}
