@@ -1,6 +1,5 @@
 /* Boost format strings.
-   Copyright (C) 2001-2025 Free Software Foundation, Inc.
-   Written by Bruno Haible <haible@clisp.cons.org>, 2006.
+   Copyright (C) 2001-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,9 +14,9 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+/* Written by Bruno Haible.  */
+
+#include <config.h>
 
 #include <stdbool.h>
 #include <stdlib.h>
@@ -49,7 +48,7 @@
            - optional: a width specification: '*' (reads an argument) or '*m$'
              or a nonempty digit sequence,
            - optional: a '.' and a precision specification: '*' (reads an
-             argument) or '*m$' or a nonempty digit sequence,
+             argument) or '*m$' or an optional nonempty digit sequence,
            - optional: any of the characters 'h', 'l', 'L',
            - if the directive started with '%|':
                an optional specifier and a final '|',
@@ -85,14 +84,19 @@ enum format_arg_type
 
 struct numbered_arg
 {
-  unsigned int number;
+  size_t number;
   enum format_arg_type type;
 };
 
 struct spec
 {
-  unsigned int directives;
-  unsigned int numbered_arg_count;
+  size_t directives;
+  /* We consider a directive as "likely intentional" if it does not contain a
+     space.  This prevents xgettext from flagging strings like "100% complete"
+     as 'boost-format' if they don't occur in a context that requires a format
+     string.  */
+  size_t likely_intentional_directives;
+  size_t numbered_arg_count;
   struct numbered_arg *numbered;
 };
 
@@ -100,8 +104,8 @@ struct spec
 static int
 numbered_arg_compare (const void *p1, const void *p2)
 {
-  unsigned int n1 = ((const struct numbered_arg *) p1)->number;
-  unsigned int n2 = ((const struct numbered_arg *) p2)->number;
+  size_t n1 = ((const struct numbered_arg *) p1)->number;
+  size_t n2 = ((const struct numbered_arg *) p2)->number;
 
   return (n1 > n2 ? 1 : n1 < n2 ? -1 : 0);
 }
@@ -111,16 +115,14 @@ format_parse (const char *format, bool translated, char *fdi,
               char **invalid_reason)
 {
   const char *const format_start = format;
-  struct spec spec;
-  unsigned int numbered_allocated;
-  unsigned int unnumbered_arg_count;
-  struct spec *result;
 
+  struct spec spec;
   spec.directives = 0;
+  spec.likely_intentional_directives = 0;
   spec.numbered_arg_count = 0;
   spec.numbered = NULL;
-  numbered_allocated = 0;
-  unnumbered_arg_count = 0;
+  size_t numbered_allocated = 0;
+  size_t unnumbered_arg_count = 0;
 
   for (; *format != '\0';)
     /* Invariant: spec.numbered_arg_count == 0 || unnumbered_arg_count == 0.  */
@@ -129,26 +131,27 @@ format_parse (const char *format, bool translated, char *fdi,
         /* A directive.  */
         FDI_SET (format - 1, FMTDIR_START);
         spec.directives++;
+        bool likely_intentional = true;
 
         if (*format == '%')
           format++;
         else
           {
-            bool brackets = false;
             bool done = false;
-            unsigned int number = 0;
             enum format_arg_type type = FAT_NONE;
 
+            bool brackets = false;
             if (*format == '|')
               {
                 format++;
                 brackets = true;
               }
 
+            size_t number = 0;
             if (c_isdigit (*format) && *format != '0')
               {
                 const char *f = format;
-                unsigned int m = 0;
+                size_t m = 0;
 
                 do
                   {
@@ -184,7 +187,11 @@ format_parse (const char *format, bool translated, char *fdi,
                         || *format == '#' || *format == '0' || *format == '\''
                         || *format == '_' || *format == '=' || *format == 'h'
                         || *format == 'l')
-                      format++;
+                      {
+                        if (*format == ' ')
+                          likely_intentional = false;
+                        format++;
+                      }
                     else
                       break;
                   }
@@ -192,14 +199,14 @@ format_parse (const char *format, bool translated, char *fdi,
                 /* Parse width.  */
                 if (*format == '*')
                   {
-                    unsigned int width_number = 0;
+                    size_t width_number = 0;
 
                     format++;
 
                     if (c_isdigit (*format))
                       {
                         const char *f = format;
-                        unsigned int m = 0;
+                        size_t m = 0;
 
                         do
                           {
@@ -281,14 +288,14 @@ format_parse (const char *format, bool translated, char *fdi,
 
                     if (*format == '*')
                       {
-                        unsigned int precision_number = 0;
+                        size_t precision_number = 0;
 
                         format++;
 
                         if (c_isdigit (*format))
                           {
                             const char *f = format;
-                            unsigned int m = 0;
+                            size_t m = 0;
 
                             do
                               {
@@ -442,7 +449,7 @@ format_parse (const char *format, bool translated, char *fdi,
                         else
                           {
                             *invalid_reason =
-                              xasprintf (_("The directive number %u starts with | but does not end with |."),
+                              xasprintf (_("The directive number %zu starts with | but does not end with |."),
                                          spec.directives);
                             FDI_SET (format, FMTDIR_ERROR);
                           }
@@ -499,6 +506,8 @@ format_parse (const char *format, bool translated, char *fdi,
               }
           }
 
+        if (likely_intentional)
+          spec.likely_intentional_directives++;
         FDI_SET (format - 1, FMTDIR_END);
       }
 
@@ -508,21 +517,19 @@ format_parse (const char *format, bool translated, char *fdi,
   /* Sort the numbered argument array, and eliminate duplicates.  */
   else if (spec.numbered_arg_count > 1)
     {
-      unsigned int i, j;
-      bool err;
-
       qsort (spec.numbered, spec.numbered_arg_count,
              sizeof (struct numbered_arg), numbered_arg_compare);
 
       /* Remove duplicates: Copy from i to j, keeping 0 <= j <= i.  */
-      err = false;
+      bool err = false;
+      size_t i, j;
       for (i = j = 0; i < spec.numbered_arg_count; i++)
         if (j > 0 && spec.numbered[i].number == spec.numbered[j-1].number)
           {
             enum format_arg_type type1 = spec.numbered[i].type;
             enum format_arg_type type2 = spec.numbered[j-1].type;
-            enum format_arg_type type_both;
 
+            enum format_arg_type type_both;
             if (type1 == type2 || type2 == FAT_ANY)
               type_both = type1;
             else if (type1 == FAT_ANY)
@@ -554,7 +561,7 @@ format_parse (const char *format, bool translated, char *fdi,
         goto bad_format;
     }
 
-  result = XMALLOC (struct spec);
+  struct spec *result = XMALLOC (struct spec);
   *result = spec;
   return result;
 
@@ -582,6 +589,14 @@ format_get_number_of_directives (void *descr)
   return spec->directives;
 }
 
+static bool
+format_is_unlikely_intentional (void *descr)
+{
+  struct spec *spec = (struct spec *) descr;
+
+  return spec->likely_intentional_directives == 0;
+}
+
 static bool
 format_check (void *msgid_descr, void *msgstr_descr, bool equality,
               formatstring_error_logger_t error_logger, void *error_logger_data,
@@ -593,68 +608,73 @@ format_check (void *msgid_descr, void *msgstr_descr, bool equality,
 
   if (spec1->numbered_arg_count + spec2->numbered_arg_count > 0)
     {
-      unsigned int i, j;
-      unsigned int n1 = spec1->numbered_arg_count;
-      unsigned int n2 = spec2->numbered_arg_count;
+      size_t n1 = spec1->numbered_arg_count;
+      size_t n2 = spec2->numbered_arg_count;
 
       /* Check that the argument numbers are the same.
          Both arrays are sorted.  We search for the first difference.  */
-      for (i = 0, j = 0; i < n1 || j < n2; )
-        {
-          int cmp = (i >= n1 ? 1 :
-                     j >= n2 ? -1 :
-                     spec1->numbered[i].number > spec2->numbered[j].number ? 1 :
-                     spec1->numbered[i].number < spec2->numbered[j].number ? -1 :
-                     0);
+      {
+        size_t i, j;
+        for (i = 0, j = 0; i < n1 || j < n2; )
+          {
+            int cmp = (i >= n1 ? 1 :
+                       j >= n2 ? -1 :
+                       spec1->numbered[i].number > spec2->numbered[j].number ? 1 :
+                       spec1->numbered[i].number < spec2->numbered[j].number ? -1 :
+                       0);
 
-          if (cmp > 0)
+            if (cmp > 0)
+              {
+                if (error_logger)
+                  error_logger (error_logger_data,
+                                _("a format specification for argument %zu, as in '%s', doesn't exist in '%s'"),
+                                spec2->numbered[j].number, pretty_msgstr,
+                                pretty_msgid);
+                err = true;
+                break;
+              }
+            else if (cmp < 0)
+              {
+                if (equality)
+                  {
+                    if (error_logger)
+                      error_logger (error_logger_data,
+                                    _("a format specification for argument %zu doesn't exist in '%s'"),
+                                    spec1->numbered[i].number, pretty_msgstr);
+                    err = true;
+                    break;
+                  }
+                else
+                  i++;
+              }
+            else
+              j++, i++;
+          }
+      }
+      /* Check the argument types are the same.  */
+      if (!err)
+        {
+          size_t i, j;
+          for (i = 0, j = 0; j < n2; )
             {
-              if (error_logger)
-                error_logger (error_logger_data,
-                              _("a format specification for argument %u, as in '%s', doesn't exist in '%s'"),
-                              spec2->numbered[j].number, pretty_msgstr,
-                              pretty_msgid);
-              err = true;
-              break;
-            }
-          else if (cmp < 0)
-            {
-              if (equality)
+              if (spec1->numbered[i].number == spec2->numbered[j].number)
                 {
-                  if (error_logger)
-                    error_logger (error_logger_data,
-                                  _("a format specification for argument %u doesn't exist in '%s'"),
-                                  spec1->numbered[i].number, pretty_msgstr);
-                  err = true;
-                  break;
+                  if (spec1->numbered[i].type != spec2->numbered[j].type)
+                    {
+                      if (error_logger)
+                        error_logger (error_logger_data,
+                                      _("format specifications in '%s' and '%s' for argument %zu are not the same"),
+                                      pretty_msgid, pretty_msgstr,
+                                      spec2->numbered[j].number);
+                      err = true;
+                      break;
+                    }
+                  j++, i++;
                 }
               else
                 i++;
             }
-          else
-            j++, i++;
         }
-      /* Check the argument types are the same.  */
-      if (!err)
-        for (i = 0, j = 0; j < n2; )
-          {
-            if (spec1->numbered[i].number == spec2->numbered[j].number)
-              {
-                if (spec1->numbered[i].type != spec2->numbered[j].type)
-                  {
-                    if (error_logger)
-                      error_logger (error_logger_data,
-                                    _("format specifications in '%s' and '%s' for argument %u are not the same"),
-                                    pretty_msgid, pretty_msgstr,
-                                    spec2->numbered[j].number);
-                    err = true;
-                    break;
-                  }
-                j++, i++;
-              }
-            else
-              i++;
-          }
     }
 
   return err;
@@ -666,7 +686,7 @@ struct formatstring_parser formatstring_boost =
   format_parse,
   format_free,
   format_get_number_of_directives,
-  NULL,
+  format_is_unlikely_intentional,
   format_check
 };
 
@@ -682,8 +702,6 @@ static void
 format_print (void *descr)
 {
   struct spec *spec = (struct spec *) descr;
-  unsigned int last;
-  unsigned int i;
 
   if (spec == NULL)
     {
@@ -692,10 +710,10 @@ format_print (void *descr)
     }
 
   printf ("(");
-  last = 1;
-  for (i = 0; i < spec->numbered_arg_count; i++)
+  size_t last = 1;
+  for (size_t i = 0; i < spec->numbered_arg_count; i++)
     {
-      unsigned int number = spec->numbered[i].number;
+      size_t number = spec->numbered[i].number;
 
       if (i > 0)
         printf (" ");
@@ -735,18 +753,14 @@ main ()
     {
       char *line = NULL;
       size_t line_size = 0;
-      int line_len;
-      char *invalid_reason;
-      void *descr;
-
-      line_len = getline (&line, &line_size, stdin);
+      int line_len = getline (&line, &line_size, stdin);
       if (line_len < 0)
         break;
       if (line_len > 0 && line[line_len - 1] == '\n')
         line[--line_len] = '\0';
 
-      invalid_reason = NULL;
-      descr = format_parse (line, false, NULL, &invalid_reason);
+      char *invalid_reason = NULL;
+      void *descr = format_parse (line, false, NULL, &invalid_reason);
 
       format_print (descr);
       printf ("\n");
@@ -763,7 +777,7 @@ main ()
 /*
  * For Emacs M-x compile
  * Local Variables:
- * compile-command: "/bin/sh ../libtool --tag=CC --mode=link gcc -o a.out -static -O -g -Wall -I.. -I../gnulib-lib -I../../gettext-runtime/intl -DHAVE_CONFIG_H -DTEST format-boost.c ../gnulib-lib/libgettextlib.la"
+ * compile-command: "/bin/sh ../libtool --tag=CC --mode=link gcc -o a.out -static -O -g -Wall -I.. -I../gnulib-lib -I../../gettext-runtime/intl -DTEST format-boost.c ../gnulib-lib/libgettextlib.la"
  * End:
  */
 
