@@ -1,6 +1,5 @@
 /* Python brace format strings.
-   Copyright (C) 2004, 2006-2007, 2013-2014, 2016, 2019, 2023 Free Software Foundation, Inc.
-   Written by Daiki Ueno <ueno@gnu.org>, 2013.
+   Copyright (C) 2004-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,9 +14,9 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+/* Written by Daiki Ueno and Bruno Haible.  */
+
+#include <config.h>
 
 #include <stdbool.h>
 #include <stdlib.h>
@@ -37,13 +36,15 @@
    Documentation:
      https://peps.python.org/pep-3101/
      https://docs.python.org/3/library/string.html#formatstrings
+   Here we assume Python >= 3.1, which allows unnamed argument specifications.
    A format string directive here consists of
      - an opening brace '{',
-     - an identifier [_A-Za-z][_0-9A-Za-z]*|[0-9]+,
-     - an optional sequence of
-         - getattr ('.' identifier) or
-         - getitem ('[' identifier ']')
-       operators,
+     - optionally:
+       - an identifier [_A-Za-z][_0-9A-Za-z]*|[0-9]+,
+       - an optional sequence of
+           - getattr ('.' identifier) or
+           - getitem ('[' identifier ']')
+         operators,
      - optionally, a ':' and a format specifier, where a format specifier is
        - either a format directive of the form '{' ... '}' without a format
          specifier, or
@@ -60,6 +61,8 @@
              - 'b', 'c', 'd', 'o', 'x', 'X', 'n' for integers,
              - 'e', 'E', 'f', 'F', 'g', 'G', 'n', '%' for floating-point values,
      - a closing brace '}'.
+   Numbered (identifier being a number) and unnamed argument specifications
+   cannot be used in the same string.
    Brace characters '{' and '}' can be escaped by doubling them: '{{' and '}}'.
 */
 
@@ -70,9 +73,9 @@ struct named_arg
 
 struct spec
 {
-  unsigned int directives;
-  unsigned int named_arg_count;
-  unsigned int allocated;
+  size_t directives;
+  size_t named_arg_count;
+  size_t allocated;
   struct named_arg *named;
 };
 
@@ -81,6 +84,15 @@ struct spec
 static void free_named_args (struct spec *spec);
 
 
+struct toplevel_counters
+{
+  /* Number of arguments seen whose identifier is a number.  */
+  size_t numbered_arg_counter;
+  /* Number of arguments with a missing identifier.  */
+  size_t unnamed_arg_counter;
+};
+
+
 /* All the parse_* functions (except parse_upto) follow the same
    calling convention.  FORMATP shall point to the beginning of a token.
    If parsing succeeds, FORMATP will point to the next character after
@@ -89,8 +101,8 @@ static void free_named_args (struct spec *spec);
 
 static bool
 parse_named_field (struct spec *spec,
-                   const char **formatp, bool translated, char *fdi,
-                   char **invalid_reason)
+                   const char **formatp,
+                   char *fdi, char **invalid_reason)
 {
   const char *format = *formatp;
   char c;
@@ -110,8 +122,8 @@ parse_named_field (struct spec *spec,
 
 static bool
 parse_numeric_field (struct spec *spec,
-                     const char **formatp, bool translated, char *fdi,
-                     char **invalid_reason)
+                     const char **formatp,
+                     char *fdi, char **invalid_reason)
 {
   const char *format = *formatp;
   char c;
@@ -136,12 +148,12 @@ parse_numeric_field (struct spec *spec,
  */
 static bool
 parse_directive (struct spec *spec,
-                 const char **formatp, bool is_toplevel,
-                 bool translated, char *fdi, char **invalid_reason)
+                 const char **formatp, struct toplevel_counters *toplevel,
+                 char *fdi, char **invalid_reason)
 {
   const char *format = *formatp;
   const char *const format_start = format;
-  const char *name_start;
+
   char c;
 
   c = *++format;
@@ -152,73 +164,106 @@ parse_directive (struct spec *spec,
       return true;
     }
 
-  name_start = format;
-  if (!parse_named_field (spec, &format, translated, fdi, invalid_reason)
-      && !parse_numeric_field (spec, &format, translated, fdi, invalid_reason))
+  const char *name_start = format;
+  if (parse_named_field (spec, &format, fdi, invalid_reason)
+      || parse_numeric_field (spec, &format, fdi, invalid_reason))
     {
-      *invalid_reason =
-        xasprintf (_("In the directive number %u, '%c' cannot start a field name."),
-                   spec->directives, *format);
-      FDI_SET (format, FMTDIR_ERROR);
-      return false;
-    }
-
-  /* Parse '.' (getattr) or '[..]' (getitem) operators followed by a
-     name.  If must not recurse, but can be specifed in a chain, such
-     as "foo.bar.baz[0]".  */
-  for (;;)
-    {
-      c = *format;
-
-      if (c == '.')
+      /* Parse '.' (getattr) or '[..]' (getitem) operators followed by a
+         name.  If must not recurse, but can be specified in a chain, such
+         as "foo.bar.baz[0]".  */
+      for (;;)
         {
-          format++;
-          if (!parse_named_field (spec, &format, translated, fdi,
-                                  invalid_reason))
-            {
-              *invalid_reason =
-                xasprintf (_("In the directive number %u, '%c' cannot start a getattr argument."),
-                           spec->directives, *format);
-              FDI_SET (format, FMTDIR_ERROR);
-              return false;
-            }
-        }
-      else if (c == '[')
-        {
-          format++;
-          if (!parse_named_field (spec, &format, translated, fdi,
-                                  invalid_reason)
-              && !parse_numeric_field (spec, &format, translated, fdi,
-                                       invalid_reason))
-            {
-              *invalid_reason =
-                xasprintf (_("In the directive number %u, '%c' cannot start a getitem argument."),
-                           spec->directives, *format);
-              FDI_SET (format, FMTDIR_ERROR);
-              return false;
-            }
+          c = *format;
 
-          if (*format != ']')
+          if (c == '.')
             {
-              *invalid_reason =
-                xasprintf (_("In the directive number %u, there is an unterminated getitem argument."),
-                           spec->directives);
-              FDI_SET (format, FMTDIR_ERROR);
-              return false;
+              format++;
+              if (!parse_named_field (spec, &format, fdi, invalid_reason))
+                {
+                  if (*format == '\0')
+                    {
+                      *invalid_reason = INVALID_UNTERMINATED_DIRECTIVE ();
+                      FDI_SET (format - 1, FMTDIR_ERROR);
+                    }
+                  else
+                    {
+                      *invalid_reason =
+                        (c_isprint (*format)
+                         ? xasprintf (_("In the directive number %zu, '%c' cannot start a getattr argument."),
+                                      spec->directives, *format)
+                         : xasprintf (_("In the directive number %zu, a getattr argument starts with a character that is not alphabetical or underscore."),
+                                      spec->directives));
+                      FDI_SET (format, FMTDIR_ERROR);
+                    }
+                  return false;
+                }
             }
-          format++;
+          else if (c == '[')
+            {
+              format++;
+              if (!parse_named_field (spec, &format, fdi, invalid_reason)
+                  && !parse_numeric_field (spec, &format, fdi, invalid_reason))
+                {
+                  if (*format == '\0')
+                    {
+                      *invalid_reason = INVALID_UNTERMINATED_DIRECTIVE ();
+                      FDI_SET (format - 1, FMTDIR_ERROR);
+                    }
+                  else
+                    {
+                      *invalid_reason =
+                        (c_isprint (*format)
+                         ? xasprintf (_("In the directive number %zu, '%c' cannot start a getitem argument."),
+                                      spec->directives, *format)
+                         : xasprintf (_("In the directive number %zu, a getitem argument starts with a character that is not alphanumerical or underscore."),
+                                      spec->directives));
+                      FDI_SET (format, FMTDIR_ERROR);
+                    }
+                  return false;
+                }
+
+              if (*format != ']')
+                {
+                  *invalid_reason =
+                    xasprintf (_("In the directive number %zu, there is an unterminated getitem argument."),
+                               spec->directives);
+                  FDI_SET (format - 1, FMTDIR_ERROR);
+                  return false;
+                }
+              format++;
+            }
+          else
+            break;
         }
-      else
-        break;
     }
-
-  /* Here c == *format.  */
-  if (c == ':')
+  else
     {
-      if (!is_toplevel)
+      if (*format == '\0')
+        {
+          *invalid_reason = INVALID_UNTERMINATED_DIRECTIVE ();
+          FDI_SET (format - 1, FMTDIR_ERROR);
+          return false;
+        }
+      if (!(toplevel != NULL && (*format == ':' || *format == '}')))
         {
           *invalid_reason =
-            xasprintf (_("In the directive number %u, no more nesting is allowed in a format specifier."),
+            (c_isprint (*format)
+             ? xasprintf (_("In the directive number %zu, '%c' cannot start a field name."),
+                          spec->directives, *format)
+             : xasprintf (_("In the directive number %zu, a field name starts with a character that is not alphanumerical or underscore."),
+                          spec->directives));
+          FDI_SET (format, FMTDIR_ERROR);
+          return false;
+        }
+    }
+  const char *name_end = format;
+
+  if (*format == ':')
+    {
+      if (toplevel == NULL)
+        {
+          *invalid_reason =
+            xasprintf (_("In the directive number %zu, no more nesting is allowed in a format specifier."),
                        spec->directives);
           FDI_SET (format, FMTDIR_ERROR);
           return false;
@@ -237,8 +282,7 @@ parse_directive (struct spec *spec,
       if (*format == '{')
         {
           /* Nested format directive.  */
-          if (!parse_directive (spec, &format, false, translated, fdi,
-                                invalid_reason))
+          if (!parse_directive (spec, &format, NULL, fdi, invalid_reason))
             {
               /* FDI and INVALID_REASON will be set by a recursive call of
                  parse_directive.  */
@@ -251,19 +295,17 @@ parse_directive (struct spec *spec,
              [[fill]align][sign][#][0][minimumwidth][.precision][type]  */
 
           /* Look ahead two characters to skip [[fill]align].  */
-          int c1, c2;
-
-          c1 = format[0];
+          int c1 = format[0];
           if (c1 == '\0')
             {
               *invalid_reason =
-                xasprintf (_("In the directive number %u, there is an unterminated format directive."),
+                xasprintf (_("The directive number %zu is unterminated."),
                            spec->directives);
-              FDI_SET (format, FMTDIR_ERROR);
+              FDI_SET (format - 1, FMTDIR_ERROR);
               return false;
             }
 
-          c2 = format[1];
+          int c2 = format[1];
 
           if (c2 == '<' || c2 == '>' || c2 == '=' || c2 == '^')
             format += 2;
@@ -310,22 +352,47 @@ parse_directive (struct spec *spec,
   if (*format != '}')
     {
       *invalid_reason =
-        xasprintf (_("In the directive number %u, there is an unterminated format directive."),
+        xasprintf (_("The directive number %zu is unterminated."),
                    spec->directives);
-      FDI_SET (format, FMTDIR_ERROR);
+      FDI_SET (format - 1, FMTDIR_ERROR);
       return false;
     }
 
-  if (is_toplevel)
+  if (toplevel != NULL)
     {
-      char *name;
-      size_t n = format - name_start;
-
       FDI_SET (name_start - 1, FMTDIR_START);
 
-      name = XNMALLOC (n + 1, char);
-      memcpy (name, name_start, n);
-      name[n] = '\0';
+      size_t n = name_end - name_start;
+      char *name;
+      if (n == 0)
+        {
+          if (toplevel->numbered_arg_counter > 0)
+            {
+              *invalid_reason =
+                xstrdup (_("The string refers to arguments both through absolute argument numbers and through unnamed argument specifications."));
+              FDI_SET (format, FMTDIR_ERROR);
+              return false;
+            }
+          name = xasprintf ("%zu", toplevel->unnamed_arg_counter);
+          toplevel->unnamed_arg_counter++;
+        }
+      else
+        {
+          name = XNMALLOC (n + 1, char);
+          memcpy (name, name_start, n);
+          name[n] = '\0';
+          if (name_start[0] >= '0' && name_start[0] <= '9')
+            {
+              if (toplevel->unnamed_arg_counter > 0)
+                {
+                  *invalid_reason =
+                    xstrdup (_("The string refers to arguments both through absolute argument numbers and through unnamed argument specifications."));
+                  FDI_SET (format, FMTDIR_ERROR);
+                  return false;
+                }
+              toplevel->numbered_arg_counter++;
+            }
+        }
 
       spec->directives++;
 
@@ -346,8 +413,8 @@ parse_directive (struct spec *spec,
 
 static bool
 parse_upto (struct spec *spec,
-            const char **formatp, bool is_toplevel, char terminator,
-            bool translated, char *fdi, char **invalid_reason)
+            const char **formatp, struct toplevel_counters *toplevel,
+            char terminator, char *fdi, char **invalid_reason)
 {
   const char *format = *formatp;
 
@@ -355,8 +422,7 @@ parse_upto (struct spec *spec,
     {
       if (*format == '{')
         {
-          if (!parse_directive (spec, &format, is_toplevel, translated, fdi,
-                                invalid_reason))
+          if (!parse_directive (spec, &format, toplevel, fdi, invalid_reason))
             return false;
         }
       else
@@ -375,18 +441,19 @@ named_arg_compare (const void *p1, const void *p2)
 }
 
 static void *
-format_parse (const char *format, bool translated, char *fdi,
-              char **invalid_reason)
+format_parse (const char *format, bool translated,
+              char *fdi, char **invalid_reason)
 {
   struct spec spec;
-  struct spec *result;
-
   spec.directives = 0;
   spec.named_arg_count = 0;
   spec.allocated = 0;
   spec.named = NULL;
 
-  if (!parse_upto (&spec, &format, true, '\0', translated, fdi, invalid_reason))
+  struct toplevel_counters toplevel;
+  toplevel.numbered_arg_counter = 0;
+  toplevel.unnamed_arg_counter = 0;
+  if (!parse_upto (&spec, &format, &toplevel, '\0', fdi, invalid_reason))
     {
       free_named_args (&spec);
       return NULL;
@@ -395,12 +462,11 @@ format_parse (const char *format, bool translated, char *fdi,
   /* Sort the named argument array, and eliminate duplicates.  */
   if (spec.named_arg_count > 1)
     {
-      unsigned int i, j;
-
       qsort (spec.named, spec.named_arg_count, sizeof (struct named_arg),
              named_arg_compare);
 
       /* Remove duplicates: Copy from i to j, keeping 0 <= j <= i.  */
+      size_t i, j;
       for (i = j = 0; i < spec.named_arg_count; i++)
         if (j > 0 && strcmp (spec.named[i].name, spec.named[j-1].name) == 0)
           free (spec.named[i].name);
@@ -413,7 +479,7 @@ format_parse (const char *format, bool translated, char *fdi,
       spec.named_arg_count = j;
     }
 
-  result = XMALLOC (struct spec);
+  struct spec *result = XMALLOC (struct spec);
   *result = spec;
   return result;
 }
@@ -423,7 +489,7 @@ free_named_args (struct spec *spec)
 {
   if (spec->named != NULL)
     {
-      unsigned int i;
+      size_t i;
       for (i = 0; i < spec->named_arg_count; i++)
         free (spec->named[i].name);
       free (spec->named);
@@ -458,12 +524,12 @@ format_check (void *msgid_descr, void *msgstr_descr, bool equality,
 
   if (spec1->named_arg_count + spec2->named_arg_count > 0)
     {
-      unsigned int i, j;
-      unsigned int n1 = spec1->named_arg_count;
-      unsigned int n2 = spec2->named_arg_count;
+      size_t n1 = spec1->named_arg_count;
+      size_t n2 = spec2->named_arg_count;
 
-      /* Check the argument names in spec1 are contained in those of spec2.
-         Both arrays are sorted.  We search for the differences.  */
+      /* Check the argument names in spec2 are contained in those of spec1.
+         Both arrays are sorted.  We search for the first difference.  */
+      size_t i, j;
       for (i = 0, j = 0; i < n1 || j < n2; )
         {
           int cmp = (i >= n1 ? 1 :
@@ -472,17 +538,13 @@ format_check (void *msgid_descr, void *msgstr_descr, bool equality,
 
           if (cmp > 0)
             {
-              if (equality)
-                {
-                  if (error_logger)
-                    error_logger (error_logger_data,
-                                  _("a format specification for argument '%s' doesn't exist in '%s'"),
-                                  spec2->named[i].name, pretty_msgid);
-                  err = true;
-                  break;
-                }
-              else
-                j++;
+              if (error_logger)
+                error_logger (error_logger_data,
+                              _("a format specification for argument '%s', as in '%s', doesn't exist in '%s'"),
+                              spec2->named[j].name, pretty_msgstr,
+                              pretty_msgid);
+              err = true;
+              break;
             }
           else if (cmp < 0)
             {
@@ -528,7 +590,6 @@ static void
 format_print (void *descr)
 {
   struct spec *spec = (struct spec *) descr;
-  unsigned int i;
 
   if (spec == NULL)
     {
@@ -537,7 +598,7 @@ format_print (void *descr)
     }
 
   printf ("{");
-  for (i = 0; i < spec->named_arg_count; i++)
+  for (size_t i = 0; i < spec->named_arg_count; i++)
     {
       if (i > 0)
         printf (", ");
@@ -553,18 +614,14 @@ main ()
     {
       char *line = NULL;
       size_t line_size = 0;
-      int line_len;
-      char *invalid_reason;
-      void *descr;
-
-      line_len = getline (&line, &line_size, stdin);
+      int line_len = getline (&line, &line_size, stdin);
       if (line_len < 0)
         break;
       if (line_len > 0 && line[line_len - 1] == '\n')
         line[--line_len] = '\0';
 
-      invalid_reason = NULL;
-      descr = format_parse (line, false, NULL, &invalid_reason);
+      char *invalid_reason = NULL;
+      void *descr = format_parse (line, false, NULL, &invalid_reason);
 
       format_print (descr);
       printf ("\n");
@@ -581,7 +638,7 @@ main ()
 /*
  * For Emacs M-x compile
  * Local Variables:
- * compile-command: "/bin/sh ../libtool --tag=CC --mode=link gcc -o a.out -static -O -g -Wall -I.. -I../gnulib-lib -I../../gettext-runtime/intl -DHAVE_CONFIG_H -DTEST format-python-brace.c ../gnulib-lib/libgettextlib.la"
+ * compile-command: "/bin/sh ../libtool --tag=CC --mode=link gcc -o a.out -static -O -g -Wall -I.. -I../gnulib-lib -I../../gettext-runtime/intl -DTEST format-python-brace.c ../gnulib-lib/libgettextlib.la"
  * End:
  */
 
