@@ -1,6 +1,5 @@
 /* Python format strings.
-   Copyright (C) 2001-2024 Free Software Foundation, Inc.
-   Written by Bruno Haible <haible@clisp.cons.org>, 2001.
+   Copyright (C) 2001-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,9 +14,9 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+/* Written by Bruno Haible.  */
+
+#include <config.h>
 
 #include <stdbool.h>
 #include <stdlib.h>
@@ -50,7 +49,7 @@
    - is optionally followed by a width specification: '*' (reads an argument)
      or a nonempty digit sequence,
    - is optionally followed by '.' and a precision specification: '*' (reads
-     an argument) or a nonempty digit sequence,
+     an argument) or an optional nonempty digit sequence,
    - is optionally followed by a size specifier, one of 'h' 'l' 'L'.
    - is finished by a specifier
        - '%', that needs no argument,
@@ -89,19 +88,18 @@ struct unnamed_arg
 
 struct spec
 {
-  unsigned int directives;
-  unsigned int named_arg_count;
-  unsigned int unnamed_arg_count;
+  size_t directives;
+  /* We consider a directive as "likely intentional" if it does not contain a
+     space.  This prevents xgettext from flagging strings like "100% complete"
+     as 'python-format' if they don't occur in a context that requires a format
+     string.  */
+  size_t likely_intentional_directives;
+  size_t named_arg_count;
+  size_t unnamed_arg_count;
   struct named_arg *named;
   struct unnamed_arg *unnamed;
 };
 
-/* Locale independent test for a decimal digit.
-   Argument can be  'char' or 'unsigned char'.  (Whereas the argument of
-   <ctype.h> isdigit must be an 'unsigned char'.)  */
-#undef isdigit
-#define isdigit(c) ((unsigned int) ((c) - '0') < 10)
-
 
 static int
 named_arg_compare (const void *p1, const void *p2)
@@ -118,58 +116,53 @@ format_parse (const char *format, bool translated, char *fdi,
               char **invalid_reason)
 {
   const char *const format_start = format;
-  struct spec spec;
-  unsigned int allocated;
-  struct spec *result;
 
+  struct spec spec;
   spec.directives = 0;
+  spec.likely_intentional_directives = 0;
   spec.named_arg_count = 0;
   spec.unnamed_arg_count = 0;
   spec.named = NULL;
   spec.unnamed = NULL;
-  allocated = 0;
+  size_t allocated = 0;
 
   for (; *format != '\0';)
     if (*format++ == '%')
       {
         /* A directive.  */
-        char *name = NULL;
-        bool zero_precision = false;
-        enum format_arg_type type;
-
         FDI_SET (format - 1, FMTDIR_START);
         spec.directives++;
+        bool likely_intentional = true;
 
+        char *name = NULL;
         if (*format == '(')
           {
-            unsigned int depth;
-            const char *name_start;
-            const char *name_end;
-            size_t n;
-
-            name_start = ++format;
-            depth = 0;
-            for (; *format != '\0'; format++)
-              {
-                if (*format == '(')
-                  depth++;
-                else if (*format == ')')
-                  {
-                    if (depth == 0)
-                      break;
-                    else
-                      depth--;
-                  }
-              }
+            const char *name_start = ++format;
+            size_t depth;
+            {
+              depth = 0;
+              for (; *format != '\0'; format++)
+                {
+                  if (*format == '(')
+                    depth++;
+                  else if (*format == ')')
+                    {
+                      if (depth == 0)
+                        break;
+                      else
+                        depth--;
+                    }
+                }
+            }
             if (*format == '\0')
               {
                 *invalid_reason = INVALID_UNTERMINATED_DIRECTIVE ();
                 FDI_SET (format - 1, FMTDIR_ERROR);
                 goto bad_format;
               }
-            name_end = format++;
+            const char *name_end = format++;
 
-            n = name_end - name_start;
+            size_t n = name_end - name_start;
             name = XNMALLOC (n + 1, char);
             memcpy (name, name_start, n);
             name[n] = '\0';
@@ -177,7 +170,11 @@ format_parse (const char *format, bool translated, char *fdi,
 
         while (*format == '-' || *format == '+' || *format == ' '
                || *format == '#' || *format == '0')
-          format++;
+          {
+            if (*format == ' ')
+              likely_intentional = false;
+            format++;
+          }
 
         if (*format == '*')
           {
@@ -186,6 +183,7 @@ format_parse (const char *format, bool translated, char *fdi,
             /* Named and unnamed specifications are exclusive.  */
             if (spec.named_arg_count > 0)
               {
+                free (name);
                 *invalid_reason = INVALID_MIXES_NAMED_UNNAMED ();
                 FDI_SET (format - 1, FMTDIR_ERROR);
                 goto bad_format;
@@ -199,11 +197,12 @@ format_parse (const char *format, bool translated, char *fdi,
             spec.unnamed[spec.unnamed_arg_count].type = FAT_INTEGER;
             spec.unnamed_arg_count++;
           }
-        else if (isdigit (*format))
+        else if (c_isdigit (*format))
           {
-            do format++; while (isdigit (*format));
+            do format++; while (c_isdigit (*format));
           }
 
+        bool zero_precision = false;
         if (*format == '.')
           {
             format++;
@@ -215,6 +214,7 @@ format_parse (const char *format, bool translated, char *fdi,
                 /* Named and unnamed specifications are exclusive.  */
                 if (spec.named_arg_count > 0)
                   {
+                    free (name);
                     *invalid_reason = INVALID_MIXES_NAMED_UNNAMED ();
                     FDI_SET (format - 1, FMTDIR_ERROR);
                     goto bad_format;
@@ -228,7 +228,7 @@ format_parse (const char *format, bool translated, char *fdi,
                 spec.unnamed[spec.unnamed_arg_count].type = FAT_INTEGER;
                 spec.unnamed_arg_count++;
               }
-            else if (isdigit (*format))
+            else if (c_isdigit (*format))
               {
                 zero_precision = true;
                 do
@@ -237,13 +237,14 @@ format_parse (const char *format, bool translated, char *fdi,
                       zero_precision = false;
                     format++;
                   }
-                while (isdigit (*format));
+                while (c_isdigit (*format));
               }
           }
 
         if (*format == 'h' || *format == 'l' || *format == 'L')
           format++;
 
+        enum format_arg_type type;
         switch (*format)
           {
           case '%':
@@ -262,6 +263,7 @@ format_parse (const char *format, bool translated, char *fdi,
             type = FAT_FLOAT;
             break;
           default:
+            free (name);
             if (*format == '\0')
               {
                 *invalid_reason = INVALID_UNTERMINATED_DIRECTIVE ();
@@ -283,6 +285,7 @@ format_parse (const char *format, bool translated, char *fdi,
             /* Named and unnamed specifications are exclusive.  */
             if (spec.unnamed_arg_count > 0)
               {
+                free (name);
                 *invalid_reason = INVALID_MIXES_NAMED_UNNAMED ();
                 FDI_SET (format, FMTDIR_ERROR);
                 goto bad_format;
@@ -304,6 +307,7 @@ format_parse (const char *format, bool translated, char *fdi,
             /* Named and unnamed specifications are exclusive.  */
             if (spec.named_arg_count > 0)
               {
+                free (name);
                 *invalid_reason = INVALID_MIXES_NAMED_UNNAMED ();
                 FDI_SET (format, FMTDIR_ERROR);
                 goto bad_format;
@@ -318,6 +322,8 @@ format_parse (const char *format, bool translated, char *fdi,
             spec.unnamed_arg_count++;
           }
 
+        if (likely_intentional)
+          spec.likely_intentional_directives++;
         FDI_SET (format, FMTDIR_END);
 
         format++;
@@ -326,21 +332,19 @@ format_parse (const char *format, bool translated, char *fdi,
   /* Sort the named argument array, and eliminate duplicates.  */
   if (spec.named_arg_count > 1)
     {
-      unsigned int i, j;
-      bool err;
-
       qsort (spec.named, spec.named_arg_count, sizeof (struct named_arg),
              named_arg_compare);
 
       /* Remove duplicates: Copy from i to j, keeping 0 <= j <= i.  */
-      err = false;
+      bool err = false;
+      size_t i, j;
       for (i = j = 0; i < spec.named_arg_count; i++)
         if (j > 0 && strcmp (spec.named[i].name, spec.named[j-1].name) == 0)
           {
             enum format_arg_type type1 = spec.named[i].type;
             enum format_arg_type type2 = spec.named[j-1].type;
-            enum format_arg_type type_both;
 
+            enum format_arg_type type_both;
             if (type1 == type2 || type2 == FAT_ANY)
               type_both = type1;
             else if (type1 == FAT_ANY)
@@ -373,15 +377,14 @@ format_parse (const char *format, bool translated, char *fdi,
         goto bad_format;
     }
 
-  result = XMALLOC (struct spec);
+  struct spec *result = XMALLOC (struct spec);
   *result = spec;
   return result;
 
  bad_format:
   if (spec.named != NULL)
     {
-      unsigned int i;
-      for (i = 0; i < spec.named_arg_count; i++)
+      for (size_t i = 0; i < spec.named_arg_count; i++)
         free (spec.named[i].name);
       free (spec.named);
     }
@@ -397,8 +400,7 @@ format_free (void *descr)
 
   if (spec->named != NULL)
     {
-      unsigned int i;
-      for (i = 0; i < spec->named_arg_count; i++)
+      for (size_t i = 0; i < spec->named_arg_count; i++)
         free (spec->named[i].name);
       free (spec->named);
     }
@@ -415,6 +417,14 @@ format_get_number_of_directives (void *descr)
   return spec->directives;
 }
 
+static bool
+format_is_unlikely_intentional (void *descr)
+{
+  struct spec *spec = (struct spec *) descr;
+
+  return spec->likely_intentional_directives == 0;
+}
+
 static bool
 format_check (void *msgid_descr, void *msgstr_descr, bool equality,
               formatstring_error_logger_t error_logger, void *error_logger_data,
@@ -444,75 +454,78 @@ format_check (void *msgid_descr, void *msgstr_descr, bool equality,
     {
       if (spec1->named_arg_count + spec2->named_arg_count > 0)
         {
-          unsigned int i, j;
-          unsigned int n1 = spec1->named_arg_count;
-          unsigned int n2 = spec2->named_arg_count;
+          size_t n1 = spec1->named_arg_count;
+          size_t n2 = spec2->named_arg_count;
 
           /* Check the argument names in spec2 are contained in those of spec1.
              Both arrays are sorted.  We search for the first difference.  */
-          for (i = 0, j = 0; i < n1 || j < n2; )
-            {
-              int cmp = (i >= n1 ? 1 :
-                         j >= n2 ? -1 :
-                         strcmp (spec1->named[i].name, spec2->named[j].name));
+          {
+            size_t i, j;
+            for (i = 0, j = 0; i < n1 || j < n2; )
+              {
+                int cmp = (i >= n1 ? 1 :
+                           j >= n2 ? -1 :
+                           strcmp (spec1->named[i].name, spec2->named[j].name));
 
-              if (cmp > 0)
+                if (cmp > 0)
+                  {
+                    if (error_logger)
+                      error_logger (error_logger_data,
+                                    _("a format specification for argument '%s', as in '%s', doesn't exist in '%s'"),
+                                    spec2->named[j].name, pretty_msgstr,
+                                    pretty_msgid);
+                    err = true;
+                    break;
+                  }
+                else if (cmp < 0)
+                  {
+                    if (equality)
+                      {
+                        if (error_logger)
+                          error_logger (error_logger_data,
+                                        _("a format specification for argument '%s' doesn't exist in '%s'"),
+                                        spec1->named[i].name, pretty_msgstr);
+                        err = true;
+                        break;
+                      }
+                    else
+                      i++;
+                  }
+                else
+                  j++, i++;
+              }
+          }
+          /* Check the argument types are the same.  */
+          if (!err)
+            {
+              size_t i, j;
+              for (i = 0, j = 0; j < n2; )
                 {
-                  if (error_logger)
-                    error_logger (error_logger_data,
-                                  _("a format specification for argument '%s', as in '%s', doesn't exist in '%s'"),
-                                  spec2->named[j].name, pretty_msgstr,
-                                  pretty_msgid);
-                  err = true;
-                  break;
-                }
-              else if (cmp < 0)
-                {
-                  if (equality)
+                  if (strcmp (spec1->named[i].name, spec2->named[j].name) == 0)
                     {
-                      if (error_logger)
-                        error_logger (error_logger_data,
-                                      _("a format specification for argument '%s' doesn't exist in '%s'"),
-                                      spec1->named[i].name, pretty_msgstr);
-                      err = true;
-                      break;
+                      if (!(spec1->named[i].type == spec2->named[j].type
+                            || (!equality
+                                && (spec1->named[i].type == FAT_ANY
+                                    || spec2->named[j].type == FAT_ANY))))
+                        {
+                          if (error_logger)
+                            error_logger (error_logger_data,
+                                          _("format specifications in '%s' and '%s' for argument '%s' are not the same"),
+                                          pretty_msgid, pretty_msgstr,
+                                          spec2->named[j].name);
+                          err = true;
+                          break;
+                        }
+                      j++, i++;
                     }
                   else
                     i++;
                 }
-              else
-                j++, i++;
             }
-          /* Check the argument types are the same.  */
-          if (!err)
-            for (i = 0, j = 0; j < n2; )
-              {
-                if (strcmp (spec1->named[i].name, spec2->named[j].name) == 0)
-                  {
-                    if (!(spec1->named[i].type == spec2->named[j].type
-                          || (!equality
-                              && (spec1->named[i].type == FAT_ANY
-                                  || spec2->named[j].type == FAT_ANY))))
-                      {
-                        if (error_logger)
-                          error_logger (error_logger_data,
-                                        _("format specifications in '%s' and '%s' for argument '%s' are not the same"),
-                                        pretty_msgid, pretty_msgstr,
-                                        spec2->named[j].name);
-                        err = true;
-                        break;
-                      }
-                    j++, i++;
-                  }
-                else
-                  i++;
-              }
         }
 
       if (spec1->unnamed_arg_count + spec2->unnamed_arg_count > 0)
         {
-          unsigned int i;
-
           /* Check the argument types are the same.  */
           if (spec1->unnamed_arg_count != spec2->unnamed_arg_count)
             {
@@ -523,7 +536,7 @@ format_check (void *msgid_descr, void *msgstr_descr, bool equality,
               err = true;
             }
           else
-            for (i = 0; i < spec2->unnamed_arg_count; i++)
+            for (size_t i = 0; i < spec2->unnamed_arg_count; i++)
               if (!(spec1->unnamed[i].type == spec2->unnamed[i].type
                     || (!equality
                         && (spec1->unnamed[i].type == FAT_ANY
@@ -531,7 +544,7 @@ format_check (void *msgid_descr, void *msgstr_descr, bool equality,
                 {
                   if (error_logger)
                     error_logger (error_logger_data,
-                                  _("format specifications in '%s' and '%s' for argument %u are not the same"),
+                                  _("format specifications in '%s' and '%s' for argument %zu are not the same"),
                                   pretty_msgid, pretty_msgstr, i + 1);
                   err = true;
                 }
@@ -547,12 +560,12 @@ struct formatstring_parser formatstring_python =
   format_parse,
   format_free,
   format_get_number_of_directives,
-  NULL,
+  format_is_unlikely_intentional,
   format_check
 };
 
 
-unsigned int
+size_t
 get_python_format_unnamed_arg_count (const char *string)
 {
   /* Parse the format string.  */
@@ -562,7 +575,7 @@ get_python_format_unnamed_arg_count (const char *string)
 
   if (descr != NULL)
     {
-      unsigned int result = descr->unnamed_arg_count;
+      size_t result = descr->unnamed_arg_count;
 
       format_free (descr);
       return result;
@@ -586,7 +599,6 @@ static void
 format_print (void *descr)
 {
   struct spec *spec = (struct spec *) descr;
-  unsigned int i;
 
   if (spec == NULL)
     {
@@ -600,7 +612,7 @@ format_print (void *descr)
         abort ();
 
       printf ("{");
-      for (i = 0; i < spec->named_arg_count; i++)
+      for (size_t i = 0; i < spec->named_arg_count; i++)
         {
           if (i > 0)
             printf (", ");
@@ -631,7 +643,7 @@ format_print (void *descr)
   else
     {
       printf ("(");
-      for (i = 0; i < spec->unnamed_arg_count; i++)
+      for (size_t i = 0; i < spec->unnamed_arg_count; i++)
         {
           if (i > 0)
             printf (" ");
@@ -667,18 +679,14 @@ main ()
     {
       char *line = NULL;
       size_t line_size = 0;
-      int line_len;
-      char *invalid_reason;
-      void *descr;
-
-      line_len = getline (&line, &line_size, stdin);
+      int line_len = getline (&line, &line_size, stdin);
       if (line_len < 0)
         break;
       if (line_len > 0 && line[line_len - 1] == '\n')
         line[--line_len] = '\0';
 
-      invalid_reason = NULL;
-      descr = format_parse (line, false, NULL, &invalid_reason);
+      char *invalid_reason = NULL;
+      void *descr = format_parse (line, false, NULL, &invalid_reason);
 
       format_print (descr);
       printf ("\n");
@@ -695,7 +703,7 @@ main ()
 /*
  * For Emacs M-x compile
  * Local Variables:
- * compile-command: "/bin/sh ../libtool --tag=CC --mode=link gcc -o a.out -static -O -g -Wall -I.. -I../gnulib-lib -I../../gettext-runtime/intl -DHAVE_CONFIG_H -DTEST format-python.c ../gnulib-lib/libgettextlib.la"
+ * compile-command: "/bin/sh ../libtool --tag=CC --mode=link gcc -o a.out -static -O -g -Wall -I.. -I../gnulib-lib -I../../gettext-runtime/intl -DTEST format-python.c ../gnulib-lib/libgettextlib.la"
  * End:
  */
 
