@@ -1,6 +1,5 @@
 /* Rust format strings.
-   Copyright (C) 2001-2025 Free Software Foundation, Inc.
-   Written by Bruno Haible <haible@clisp.cons.org>, 2025.
+   Copyright (C) 2001-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,9 +14,9 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+/* Written by Bruno Haible.  */
+
+#include <config.h>
 
 #include <limits.h>
 #include <stdbool.h>
@@ -91,25 +90,19 @@ struct named_arg
 struct numbered_arg
 {
   /* The number of the argument, 0-based.  */
-  unsigned int number;
+  size_t number;
 };
 
 struct spec
 {
-  unsigned int directives;
-  unsigned int named_arg_count;
-  unsigned int numbered_arg_count;
+  size_t directives;
+  size_t named_arg_count;
+  size_t numbered_arg_count;
   struct named_arg *named;
   struct numbered_arg *numbered;
 };
 
 
-/* All the parse_* functions (except parse_upto) follow the same
-   calling convention.  FORMATP shall point to the beginning of a token.
-   If parsing succeeds, FORMATP will point to the next character after
-   the token, and true is returned.  Otherwise, FORMATP will be
-   unchanged and false is returned.  */
-
 static int
 named_arg_compare (const void *p1, const void *p2)
 {
@@ -120,8 +113,8 @@ named_arg_compare (const void *p1, const void *p2)
 static int
 numbered_arg_compare (const void *p1, const void *p2)
 {
-  unsigned int n1 = ((const struct numbered_arg *) p1)->number;
-  unsigned int n2 = ((const struct numbered_arg *) p2)->number;
+  size_t n1 = ((const struct numbered_arg *) p1)->number;
+  size_t n2 = ((const struct numbered_arg *) p2)->number;
 
   return (n1 > n2 ? 1 : n1 < n2 ? -1 : 0);
 }
@@ -131,21 +124,15 @@ format_parse (const char *format, bool translated, char *fdi,
               char **invalid_reason)
 {
   struct spec spec;
-  unsigned int named_allocated;
-  unsigned int numbered_allocated;
-  bool seen_numbered_args;
-  unsigned int unnumbered_arg_count;
-  struct spec *result;
-
   spec.directives = 0;
   spec.named_arg_count = 0;
   spec.numbered_arg_count = 0;
   spec.named = NULL;
   spec.numbered = NULL;
-  named_allocated = 0;
-  numbered_allocated = 0;
-  seen_numbered_args = false;
-  unnumbered_arg_count = 0;
+  size_t named_allocated = 0;
+  size_t numbered_allocated = 0;
+  bool seen_numbered_args = false;
+  size_t unnumbered_arg_count = 0;
 
   for (; *format != '\0';)
     {
@@ -163,8 +150,8 @@ format_parse (const char *format, bool translated, char *fdi,
             {
               const char *const format_start = format;
               bool seen_identifier_or_keyword = false;
-              unsigned int arg_id;
 
+              unsigned int arg_id;
               if (c_isdigit (*format))
                 {
                   /* Numbered and unnumbered specifications are exclusive.  */
@@ -182,7 +169,7 @@ format_parse (const char *format, bool translated, char *fdi,
                       if (arg_id >= UINT_MAX / 10)
                         {
                           *invalid_reason =
-                            xasprintf (_("In the directive number %u, the argument number is too large."), spec.directives);
+                            xasprintf (_("In the directive number %zu, the argument number is too large."), spec.directives);
                           FDI_SET (format, FMTDIR_ERROR);
                           goto bad_format;
                         }
@@ -267,19 +254,17 @@ format_parse (const char *format, bool translated, char *fdi,
                      [[fill]align][sign][#][0][minimumwidth][.precision][type]  */
 
                   /* Look ahead two characters to skip [[fill]align].  */
-                  int c1, c2;
-
-                  c1 = format[0];
+                  int c1 = format[0];
                   if (c1 == '\0')
                     {
                       *invalid_reason =
-                        xasprintf (_("The directive number %u is unterminated."),
+                        xasprintf (_("The directive number %zu is unterminated."),
                                    spec.directives);
                       FDI_SET (format - 1, FMTDIR_ERROR);
                       goto bad_format;
                     }
 
-                  c2 = format[1];
+                  int c2 = format[1];
 
                   if (c2 == '<' || c2 == '>' || c2 == '^')
                     format += 2;
@@ -330,7 +315,7 @@ format_parse (const char *format, bool translated, char *fdi,
               if (*format != '}')
                 {
                   *invalid_reason =
-                    xasprintf (_("The directive number %u is unterminated."),
+                    xasprintf (_("The directive number %zu is unterminated."),
                                spec.directives);
                   FDI_SET (format - 1, FMTDIR_ERROR);
                   goto bad_format;
@@ -358,12 +343,11 @@ format_parse (const char *format, bool translated, char *fdi,
   /* Sort the numbered argument array, and eliminate duplicates.  */
   if (spec.numbered_arg_count > 1)
     {
-      unsigned int i, j;
-
       qsort (spec.numbered, spec.numbered_arg_count,
              sizeof (struct numbered_arg), numbered_arg_compare);
 
       /* Remove duplicates: Copy from i to j, keeping 0 <= j <= i.  */
+      size_t i, j;
       for (i = j = 0; i < spec.numbered_arg_count; i++)
         if (j > 0 && spec.numbered[i].number == spec.numbered[j-1].number)
           ;
@@ -379,12 +363,11 @@ format_parse (const char *format, bool translated, char *fdi,
   /* Sort the named argument array, and eliminate duplicates.  */
   if (spec.named_arg_count > 1)
     {
-      unsigned int i, j;
-
       qsort (spec.named, spec.named_arg_count, sizeof (struct named_arg),
              named_arg_compare);
 
       /* Remove duplicates: Copy from i to j, keeping 0 <= j <= i.  */
+      size_t i, j;
       for (i = j = 0; i < spec.named_arg_count; i++)
         if (j > 0 && strcmp (spec.named[i].name, spec.named[j-1].name) == 0)
           free (spec.named[i].name);
@@ -397,15 +380,14 @@ format_parse (const char *format, bool translated, char *fdi,
       spec.named_arg_count = j;
     }
 
-  result = XMALLOC (struct spec);
+  struct spec *result = XMALLOC (struct spec);
   *result = spec;
   return result;
 
  bad_format:
   if (spec.named != NULL)
     {
-      unsigned int i;
-      for (i = 0; i < spec.named_arg_count; i++)
+      for (size_t i = 0; i < spec.named_arg_count; i++)
         free (spec.named[i].name);
       free (spec.named);
     }
@@ -421,8 +403,7 @@ format_free (void *descr)
 
   if (spec->named != NULL)
     {
-      unsigned int i;
-      for (i = 0; i < spec->named_arg_count; i++)
+      for (size_t i = 0; i < spec->named_arg_count; i++)
         free (spec->named[i].name);
       free (spec->named);
     }
@@ -449,12 +430,12 @@ format_check (void *msgid_descr, void *msgstr_descr, bool equality,
 
   if (spec1->named_arg_count + spec2->named_arg_count > 0)
     {
-      unsigned int i, j;
-      unsigned int n1 = spec1->named_arg_count;
-      unsigned int n2 = spec2->named_arg_count;
+      size_t n1 = spec1->named_arg_count;
+      size_t n2 = spec2->named_arg_count;
 
       /* Check the argument names in spec2 are contained in those of spec1.
          Both arrays are sorted.  We search for the first difference.  */
+      size_t i, j;
       for (i = 0, j = 0; i < n1 || j < n2; )
         {
           int cmp = (i >= n1 ? 1 :
@@ -492,13 +473,13 @@ format_check (void *msgid_descr, void *msgstr_descr, bool equality,
 
   if (spec1->numbered_arg_count + spec2->numbered_arg_count > 0)
     {
-      unsigned int i, j;
-      unsigned int n1 = spec1->numbered_arg_count;
-      unsigned int n2 = spec2->numbered_arg_count;
-      unsigned int missing = 0; /* only used if !equality */
+      size_t n1 = spec1->numbered_arg_count;
+      size_t n2 = spec2->numbered_arg_count;
+      size_t missing = 0; /* only used if !equality */
 
       /* Check that the argument numbers are the same.
          Both arrays are sorted.  We search for the first difference.  */
+      size_t i, j;
       for (i = 0, j = 0; i < n1 || j < n2; )
         {
           int cmp = (i >= n1 ? 1 :
@@ -511,7 +492,7 @@ format_check (void *msgid_descr, void *msgstr_descr, bool equality,
             {
               if (error_logger)
                 error_logger (error_logger_data,
-                              _("a format specification for argument %u, as in '%s', doesn't exist in '%s'"),
+                              _("a format specification for argument %zu, as in '%s', doesn't exist in '%s'"),
                               spec2->numbered[j].number, pretty_msgstr,
                               pretty_msgid);
               err = true;
@@ -523,7 +504,7 @@ format_check (void *msgid_descr, void *msgstr_descr, bool equality,
                 {
                   if (error_logger)
                     error_logger (error_logger_data,
-                                  _("a format specification for argument %u doesn't exist in '%s'"),
+                                  _("a format specification for argument %zu doesn't exist in '%s'"),
                                   spec1->numbered[i].number, pretty_msgstr);
                   err = true;
                   break;
@@ -532,7 +513,7 @@ format_check (void *msgid_descr, void *msgstr_descr, bool equality,
                 {
                   if (error_logger)
                     error_logger (error_logger_data,
-                                  _("a format specification for arguments %u and %u doesn't exist in '%s', only one argument may be ignored"),
+                                  _("a format specification for arguments %zu and %zu doesn't exist in '%s', only one argument may be ignored"),
                                   missing, spec1->numbered[i].number,
                                   pretty_msgstr);
                   err = true;
@@ -574,8 +555,6 @@ static void
 format_print (void *descr)
 {
   struct spec *spec = (struct spec *) descr;
-  unsigned int last;
-  unsigned int i, j;
 
   if (spec == NULL)
     {
@@ -584,10 +563,11 @@ format_print (void *descr)
     }
 
   printf ("(");
-  last = 0;
+  size_t last = 0;
+  size_t i;
   for (i = 0; i < spec->numbered_arg_count; i++)
     {
-      unsigned int number = spec->numbered[i].number;
+      size_t number = spec->numbered[i].number;
 
       if (i > 0)
         printf (" ");
@@ -598,7 +578,7 @@ format_print (void *descr)
       printf ("*");
       last = number + 1;
     }
-  for (j = 0; j < spec->named_arg_count; j++)
+  for (size_t j = 0; j < spec->named_arg_count; j++)
     {
       if (i > 0 || j > 0)
         printf (" ");
@@ -614,18 +594,14 @@ main ()
     {
       char *line = NULL;
       size_t line_size = 0;
-      int line_len;
-      char *invalid_reason;
-      void *descr;
-
-      line_len = getline (&line, &line_size, stdin);
+      int line_len = getline (&line, &line_size, stdin);
       if (line_len < 0)
         break;
       if (line_len > 0 && line[line_len - 1] == '\n')
         line[--line_len] = '\0';
 
-      invalid_reason = NULL;
-      descr = format_parse (line, false, NULL, &invalid_reason);
+      char *invalid_reason = NULL;
+      void *descr = format_parse (line, false, NULL, &invalid_reason);
 
       format_print (descr);
       printf ("\n");
@@ -642,7 +618,7 @@ main ()
 /*
  * For Emacs M-x compile
  * Local Variables:
- * compile-command: "/bin/sh ../libtool --tag=CC --mode=link gcc -o a.out -static -O -g -Wall -I.. -I../gnulib-lib -I../../gettext-runtime/intl -DHAVE_CONFIG_H -DTEST format-rust.c ../gnulib-lib/libgettextlib.la"
+ * compile-command: "/bin/sh ../libtool --tag=CC --mode=link gcc -o a.out -static -O -g -Wall -I.. -I../gnulib-lib -I../../gettext-runtime/intl -DTEST format-rust.c ../gnulib-lib/libgettextlib.la"
  * End:
  */
 
