@@ -1,6 +1,5 @@
 /* Shell format strings.
-   Copyright (C) 2003-2004, 2006-2007, 2009, 2019-2020, 2023 Free Software Foundation, Inc.
-   Written by Bruno Haible <bruno@clisp.org>, 2003.
+   Copyright (C) 2003-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,9 +14,9 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+/* Written by Bruno Haible.  */
+
+#include <config.h>
 
 #include <stdbool.h>
 #include <stdlib.h>
@@ -61,8 +60,8 @@ struct named_arg
 
 struct spec
 {
-  unsigned int directives;
-  unsigned int named_arg_count;
+  size_t directives;
+  size_t named_arg_count;
   struct named_arg *named;
 };
 
@@ -88,31 +87,24 @@ format_parse (const char *format, bool translated, char *fdi,
               char **invalid_reason)
 {
   const char *const format_start = format;
-  struct spec spec;
-  unsigned int named_allocated;
-  struct spec *result;
 
+  struct spec spec;
   spec.directives = 0;
   spec.named_arg_count = 0;
   spec.named = NULL;
-  named_allocated = 0;
+  size_t named_allocated = 0;
 
   for (; *format != '\0';)
     if (*format++ == '$')
       {
         /* A variable substitution.  */
-        char *name;
-
         FDI_SET (format - 1, FMTDIR_START);
         spec.directives++;
 
+        char *name;
         if (*format == '{')
           {
-            const char *name_start;
-            const char *name_end;
-            size_t n;
-
-            name_start = ++format;
+            const char *name_start = ++format;
             for (; *format != '\0'; format++)
               {
                 if (*format == '}')
@@ -145,9 +137,9 @@ format_parse (const char *format, bool translated, char *fdi,
                 FDI_SET (format - 1, FMTDIR_ERROR);
                 goto bad_format;
               }
-            name_end = format++;
+            const char *name_end = format++;
 
-            n = name_end - name_start;
+            size_t n = name_end - name_start;
             if (n == 0)
               {
                 *invalid_reason = INVALID_EMPTY_VARIABLE ();
@@ -160,17 +152,13 @@ format_parse (const char *format, bool translated, char *fdi,
           }
         else if (c_isalpha (*format) || *format == '_')
           {
-            const char *name_start;
-            const char *name_end;
-            size_t n;
-
-            name_start = format;
+            const char *name_start = format;
             do
               format++;
             while (*format != '\0' && (c_isalnum (*format) || *format == '_'));
-            name_end = format;
+            const char *name_end = format;
 
-            n = name_end - name_start;
+            size_t n = name_end - name_start;
             name = XNMALLOC (n + 1, char);
             memcpy (name, name_start, n);
             name[n] = '\0';
@@ -212,12 +200,11 @@ format_parse (const char *format, bool translated, char *fdi,
   /* Sort the named argument array, and eliminate duplicates.  */
   if (spec.named_arg_count > 1)
     {
-      unsigned int i, j;
-
       qsort (spec.named, spec.named_arg_count, sizeof (struct named_arg),
              named_arg_compare);
 
       /* Remove duplicates: Copy from i to j, keeping 0 <= j <= i.  */
+      size_t i, j;
       for (i = j = 0; i < spec.named_arg_count; i++)
         if (j > 0 && strcmp (spec.named[i].name, spec.named[j-1].name) == 0)
           free (spec.named[i].name);
@@ -230,15 +217,14 @@ format_parse (const char *format, bool translated, char *fdi,
       spec.named_arg_count = j;
     }
 
-  result = XMALLOC (struct spec);
+  struct spec *result = XMALLOC (struct spec);
   *result = spec;
   return result;
 
  bad_format:
   if (spec.named != NULL)
     {
-      unsigned int i;
-      for (i = 0; i < spec.named_arg_count; i++)
+      for (size_t i = 0; i < spec.named_arg_count; i++)
         free (spec.named[i].name);
       free (spec.named);
     }
@@ -252,8 +238,7 @@ format_free (void *descr)
 
   if (spec->named != NULL)
     {
-      unsigned int i;
-      for (i = 0; i < spec->named_arg_count; i++)
+      for (size_t i = 0; i < spec->named_arg_count; i++)
         free (spec->named[i].name);
       free (spec->named);
     }
@@ -279,12 +264,12 @@ format_check (void *msgid_descr, void *msgstr_descr, bool equality,
 
   if (spec1->named_arg_count + spec2->named_arg_count > 0)
     {
-      unsigned int i, j;
-      unsigned int n1 = spec1->named_arg_count;
-      unsigned int n2 = spec2->named_arg_count;
+      size_t n1 = spec1->named_arg_count;
+      size_t n2 = spec2->named_arg_count;
 
-      /* Check that the argument names are the same.
+      /* Check the argument names in spec2 are contained in those of spec1.
          Both arrays are sorted.  We search for the first difference.  */
+      size_t i, j;
       for (i = 0, j = 0; i < n1 || j < n2; )
         {
           int cmp = (i >= n1 ? 1 :
@@ -345,7 +330,6 @@ static void
 format_print (void *descr)
 {
   struct spec *spec = (struct spec *) descr;
-  unsigned int i;
 
   if (spec == NULL)
     {
@@ -354,7 +338,7 @@ format_print (void *descr)
     }
 
   printf ("{");
-  for (i = 0; i < spec->named_arg_count; i++)
+  for (size_t i = 0; i < spec->named_arg_count; i++)
     {
       if (i > 0)
         printf (", ");
@@ -370,18 +354,14 @@ main ()
     {
       char *line = NULL;
       size_t line_size = 0;
-      int line_len;
-      char *invalid_reason;
-      void *descr;
-
-      line_len = getline (&line, &line_size, stdin);
+      int line_len = getline (&line, &line_size, stdin);
       if (line_len < 0)
         break;
       if (line_len > 0 && line[line_len - 1] == '\n')
         line[--line_len] = '\0';
 
-      invalid_reason = NULL;
-      descr = format_parse (line, false, NULL, &invalid_reason);
+      char *invalid_reason = NULL;
+      void *descr = format_parse (line, false, NULL, &invalid_reason);
 
       format_print (descr);
       printf ("\n");
@@ -398,7 +378,7 @@ main ()
 /*
  * For Emacs M-x compile
  * Local Variables:
- * compile-command: "/bin/sh ../libtool --tag=CC --mode=link gcc -o a.out -static -O -g -Wall -I.. -I../gnulib-lib -I../../gettext-runtime/intl -DHAVE_CONFIG_H -DTEST format-sh.c ../gnulib-lib/libgettextlib.la"
+ * compile-command: "/bin/sh ../libtool --tag=CC --mode=link gcc -o a.out -static -O -g -Wall -I.. -I../gnulib-lib -I../../gettext-runtime/intl -DTEST format-sh.c ../gnulib-lib/libgettextlib.la"
  * End:
  */
 
