@@ -1,6 +1,5 @@
 /* Table of languages.
-   Copyright (C) 2001-2007, 2015 Free Software Foundation, Inc.
-   Written by Bruno Haible <haible@clisp.cons.org>, 2005.
+   Copyright (C) 2001-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,23 +12,37 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
+
+/* Written by Bruno Haible.  */
 
 #ifndef _LANG_TABLE_H
 #define _LANG_TABLE_H
 
 #include <stddef.h>
 
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+
 struct language_table_entry
 {
   const char *code;
   const char *english;
 };
 
-extern struct language_table_entry language_table[];
-extern const size_t language_table_size;
+extern LIBGETTEXTSRC_DLL_VARIABLE struct language_table_entry language_table[];
+extern LIBGETTEXTSRC_DLL_VARIABLE const size_t language_table_size;
+
+extern LIBGETTEXTSRC_DLL_VARIABLE struct language_table_entry language_variant_table[];
+extern LIBGETTEXTSRC_DLL_VARIABLE const size_t language_variant_table_size;
+
+
+#ifdef __cplusplus
+}
+#endif
 
-extern struct language_table_entry language_variant_table[];
-extern const size_t language_variant_table_size;
 
 #endif /* _LANG_TABLE_H */
