@@ -1,7 +1,5 @@
 /* GNU gettext - internationalization aids
-   Copyright (C) 1995-2025 Free Software Foundation, Inc.
-
-   This file was written by Peter Miller <millerp@canb.auug.org.au>
+   Copyright (C) 1995-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,9 +14,9 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+/* Written by Peter Miller, Ulrich Drepper, and Bruno Haible.  */
+
+#include <config.h>
 
 /* Specification.  */
 #include "message.h"
@@ -57,6 +55,7 @@ const char *const format_language[NFORMATS] =
   /* format_pascal */           "object-pascal",
   /* format_modula2 */          "modula2",
   /* format_d */                "d",
+  /* format_ocaml */            "ocaml",
   /* format_smalltalk */        "smalltalk",
   /* format_qt */               "qt",
   /* format_qt_plursl */        "qt-plural",
@@ -97,6 +96,7 @@ const char *const format_language_pretty[NFORMATS] =
   /* format_pascal */           "Object Pascal",
   /* format_modula2 */          "Modula-2",
   /* format_d */                "D",
+  /* format_ocaml */            "OCaml",
   /* format_smalltalk */        "Smalltalk",
   /* format_qt */               "Qt",
   /* format_qt_plural */        "Qt plural",
@@ -112,6 +112,47 @@ const char *const format_language_pretty[NFORMATS] =
   /* format_ycp */              "YCP"
 };
 
+const char *const format_flag[NFORMATS] =
+{
+  /* format_c */                "no-" "c"             "-format",
+  /* format_objc */             "no-" "objc"          "-format",
+  /* format_cplusplus_brace */  "no-" "c++"           "-format",
+  /* format_python */           "no-" "python"        "-format",
+  /* format_python_brace */     "no-" "python-brace"  "-format",
+  /* format_java */             "no-" "java"          "-format",
+  /* format_java_printf */      "no-" "java-printf"   "-format",
+  /* format_csharp */           "no-" "csharp"        "-format",
+  /* format_javascript */       "no-" "javascript"    "-format",
+  /* format_scheme */           "no-" "scheme"        "-format",
+  /* format_lisp */             "no-" "lisp"          "-format",
+  /* format_elisp */            "no-" "elisp"         "-format",
+  /* format_librep */           "no-" "librep"        "-format",
+  /* format_rust */             "no-" "rust"          "-format",
+  /* format_go */               "no-" "go"            "-format",
+  /* format_ruby */             "no-" "ruby"          "-format",
+  /* format_sh */               "no-" "sh"            "-format",
+  /* format_sh_printf */        "no-" "sh-printf"     "-format",
+  /* format_awk */              "no-" "awk"           "-format",
+  /* format_lua */              "no-" "lua"           "-format",
+  /* format_pascal */           "no-" "object-pascal" "-format",
+  /* format_modula2 */          "no-" "modula2"       "-format",
+  /* format_d */                "no-" "d"             "-format",
+  /* format_ocaml */            "no-" "ocaml"         "-format",
+  /* format_smalltalk */        "no-" "smalltalk"     "-format",
+  /* format_qt */               "no-" "qt"            "-format",
+  /* format_qt_plursl */        "no-" "qt-plural"     "-format",
+  /* format_kde */              "no-" "kde"           "-format",
+  /* format_kde_kuit */         "no-" "kde-kuit"      "-format",
+  /* format_boost */            "no-" "boost"         "-format",
+  /* format_tcl */              "no-" "tcl"           "-format",
+  /* format_perl */             "no-" "perl"          "-format",
+  /* format_perl_brace */       "no-" "perl-brace"    "-format",
+  /* format_php */              "no-" "php"           "-format",
+  /* format_gcc_internal */     "no-" "gcc-internal"  "-format",
+  /* format_gfc_internal */     "no-" "gfc-internal"  "-format",
+  /* format_ycp */              "no-" "ycp"           "-format"
+};
+
 
 bool
 possible_format_p (enum is_format is_format)
@@ -122,12 +163,21 @@ possible_format_p (enum is_format is_format)
 }
 
 
+bool
+not_format_p (enum is_format is_format)
+{
+  return is_format == no;
+}
+
+
 const char *const syntax_check_name[NSYNTAXCHECKS] =
 {
   /* sc_ellipsis_unicode */     "ellipsis-unicode",
   /* sc_space_ellipsis */       "space-ellipsis",
   /* sc_quote_unicode */        "quote-unicode",
-  /* sc_bullet_unicode */       "bullet-unicode"
+  /* sc_bullet_unicode */       "bullet-unicode",
+  /* sc_url */                  "url",
+  /* sc_email */                "email"
 };
 
 
@@ -137,10 +187,7 @@ message_alloc (const char *msgctxt,
                const char *msgstr, size_t msgstr_len,
                const lex_pos_ty *pp)
 {
-  message_ty *mp;
-  size_t i;
-
-  mp = XMALLOC (message_ty);
+  message_ty *mp = XMALLOC (message_ty);
   mp->msgctxt = msgctxt;
   mp->msgid = msgid;
   mp->msgid_plural = (msgid_plural != NULL ? xstrdup (msgid_plural) : NULL);
@@ -152,18 +199,19 @@ message_alloc (const char *msgctxt,
   mp->filepos_count = 0;
   mp->filepos = NULL;
   mp->is_fuzzy = false;
-  for (i = 0; i < NFORMATS; i++)
+  for (size_t i = 0; i < NFORMATS; i++)
     mp->is_format[i] = undecided;
   mp->range.min = -1;
   mp->range.max = -1;
   mp->do_wrap = undecided;
-  for (i = 0; i < NSYNTAXCHECKS; i++)
+  for (size_t i = 0; i < NSYNTAXCHECKS; i++)
     mp->do_syntax_check[i] = undecided;
   mp->prev_msgctxt = NULL;
   mp->prev_msgid = NULL;
   mp->prev_msgid_plural = NULL;
   mp->used = 0;
   mp->obsolete = false;
+
   return mp;
 }
 
@@ -171,8 +219,6 @@ message_alloc (const char *msgctxt,
 void
 message_free (message_ty *mp)
 {
-  size_t j;
-
   free ((char *) mp->msgid);
   if (mp->msgid_plural != NULL)
     free ((char *) mp->msgid_plural);
@@ -181,7 +227,7 @@ message_free (message_ty *mp)
     string_list_free (mp->comment);
   if (mp->comment_dot != NULL)
     string_list_free (mp->comment_dot);
-  for (j = 0; j < mp->filepos_count; ++j)
+  for (size_t j = 0; j < mp->filepos_count; ++j)
     free ((char *) mp->filepos[j].file_name);
   if (mp->filepos != NULL)
     free (mp->filepos);
@@ -217,25 +263,21 @@ void
 message_comment_filepos (message_ty *mp,
                          const char *file_name, size_t line_number)
 {
-  size_t j;
-  size_t nbytes;
-  lex_pos_ty *pp;
-
   /* See if we have this position already.  */
-  for (j = 0; j < mp->filepos_count; j++)
+  for (size_t j = 0; j < mp->filepos_count; j++)
     {
-      pp = &mp->filepos[j];
+      lex_pos_ty *pp = &mp->filepos[j];
       if (strcmp (pp->file_name, file_name) == 0
           && pp->line_number == line_number)
         return;
     }
 
   /* Extend the list so that we can add a position to it.  */
-  nbytes = (mp->filepos_count + 1) * sizeof (mp->filepos[0]);
+  size_t nbytes = (mp->filepos_count + 1) * sizeof (mp->filepos[0]);
   mp->filepos = xrealloc (mp->filepos, nbytes);
 
   /* Insert the position at the end.  Don't sort the file positions here.  */
-  pp = &mp->filepos[mp->filepos_count++];
+  lex_pos_ty *pp = &mp->filepos[mp->filepos_count++];
   pp->file_name = xstrdup (file_name);
   pp->line_number = line_number;
 }
@@ -244,31 +286,29 @@ message_comment_filepos (message_ty *mp,
 message_ty *
 message_copy (message_ty *mp)
 {
-  message_ty *result;
-  size_t j, i;
-
-  result = message_alloc (mp->msgctxt != NULL ? xstrdup (mp->msgctxt) : NULL,
-                          xstrdup (mp->msgid), mp->msgid_plural,
-                          mp->msgstr, mp->msgstr_len, &mp->pos);
+  message_ty *result =
+    message_alloc (mp->msgctxt != NULL ? xstrdup (mp->msgctxt) : NULL,
+                   xstrdup (mp->msgid), mp->msgid_plural,
+                   mp->msgstr, mp->msgstr_len, &mp->pos);
 
   if (mp->comment)
     {
-      for (j = 0; j < mp->comment->nitems; ++j)
+      for (size_t j = 0; j < mp->comment->nitems; ++j)
         message_comment_append (result, mp->comment->item[j]);
     }
   if (mp->comment_dot)
     {
-      for (j = 0; j < mp->comment_dot->nitems; ++j)
+      for (size_t j = 0; j < mp->comment_dot->nitems; ++j)
         message_comment_dot_append (result, mp->comment_dot->item[j]);
     }
   result->is_fuzzy = mp->is_fuzzy;
-  for (i = 0; i < NFORMATS; i++)
+  for (size_t i = 0; i < NFORMATS; i++)
     result->is_format[i] = mp->is_format[i];
   result->range = mp->range;
   result->do_wrap = mp->do_wrap;
-  for (i = 0; i < NSYNTAXCHECKS; i++)
+  for (size_t i = 0; i < NSYNTAXCHECKS; i++)
     result->do_syntax_check[i] = mp->do_syntax_check[i];
-  for (j = 0; j < mp->filepos_count; ++j)
+  for (size_t j = 0; j < mp->filepos_count; ++j)
     {
       lex_pos_ty *pp = &mp->filepos[j];
       message_comment_filepos (result, pp->file_name, pp->line_number);
@@ -286,14 +326,13 @@ message_copy (message_ty *mp)
 message_list_ty *
 message_list_alloc (bool use_hashtable)
 {
-  message_list_ty *mlp;
-
-  mlp = XMALLOC (message_list_ty);
+  message_list_ty *mlp = XMALLOC (message_list_ty);
   mlp->nitems = 0;
   mlp->nitems_max = 0;
   mlp->item = NULL;
   if ((mlp->use_hashtable = use_hashtable))
     hash_init (&mlp->htable, 10);
+
   return mlp;
 }
 
@@ -301,10 +340,8 @@ message_list_alloc (bool use_hashtable)
 void
 message_list_free (message_list_ty *mlp, int keep_messages)
 {
-  size_t j;
-
   if (keep_messages == 0)
-    for (j = 0; j < mlp->nitems; ++j)
+    for (size_t j = 0; j < mlp->nitems; ++j)
       message_free (mlp->item[j]);
   if (mlp->item)
     free (mlp->item);
@@ -320,8 +357,6 @@ message_list_hash_insert_entry (hash_table *htable, message_ty *mp)
   char *alloced_key;
   const char *key;
   size_t keylen;
-  int found;
-
   if (mp->msgctxt != NULL)
     {
       /* Concatenate mp->msgctxt and mp->msgid, to form the hash table key.  */
@@ -341,7 +376,7 @@ message_list_hash_insert_entry (hash_table *htable, message_ty *mp)
       keylen = strlen (mp->msgid) + 1;
     }
 
-  found = (hash_insert_entry (htable, key, keylen, mp) == NULL);
+  int found = (hash_insert_entry (htable, key, keylen, mp) == NULL);
 
   if (mp->msgctxt != NULL)
     freea (alloced_key);
@@ -355,10 +390,8 @@ message_list_append (message_list_ty *mlp, message_ty *mp)
 {
   if (mlp->nitems >= mlp->nitems_max)
     {
-      size_t nbytes;
-
       mlp->nitems_max = mlp->nitems_max * 2 + 4;
-      nbytes = mlp->nitems_max * sizeof (message_ty *);
+      size_t nbytes = mlp->nitems_max * sizeof (message_ty *);
       mlp->item = xrealloc (mlp->item, nbytes);
     }
   mlp->item[mlp->nitems++] = mp;
@@ -374,17 +407,13 @@ message_list_append (message_list_ty *mlp, message_ty *mp)
 void
 message_list_prepend (message_list_ty *mlp, message_ty *mp)
 {
-  size_t j;
-
   if (mlp->nitems >= mlp->nitems_max)
     {
-      size_t nbytes;
-
       mlp->nitems_max = mlp->nitems_max * 2 + 4;
-      nbytes = mlp->nitems_max * sizeof (message_ty *);
+      size_t nbytes = mlp->nitems_max * sizeof (message_ty *);
       mlp->item = xrealloc (mlp->item, nbytes);
     }
-  for (j = mlp->nitems; j > 0; j--)
+  for (size_t j = mlp->nitems; j > 0; j--)
     mlp->item[j] = mlp->item[j - 1];
   mlp->item[0] = mp;
   mlp->nitems++;
@@ -400,19 +429,18 @@ message_list_prepend (message_list_ty *mlp, message_ty *mp)
 void
 message_list_insert_at (message_list_ty *mlp, size_t n, message_ty *mp)
 {
-  size_t j;
-
   if (mlp->nitems >= mlp->nitems_max)
     {
-      size_t nbytes;
-
       mlp->nitems_max = mlp->nitems_max * 2 + 4;
-      nbytes = mlp->nitems_max * sizeof (message_ty *);
+      size_t nbytes = mlp->nitems_max * sizeof (message_ty *);
       mlp->item = xrealloc (mlp->item, nbytes);
     }
-  for (j = mlp->nitems; j > n; j--)
-    mlp->item[j] = mlp->item[j - 1];
-  mlp->item[j] = mp;
+  {
+    size_t j;
+    for (j = mlp->nitems; j > n; j--)
+      mlp->item[j] = mlp->item[j - 1];
+    mlp->item[j] = mp;
+  }
   mlp->nitems++;
 
   if (mlp->use_hashtable)
@@ -427,12 +455,10 @@ message_list_insert_at (message_list_ty *mlp, size_t n, message_ty *mp)
 void
 message_list_delete_nth (message_list_ty *mlp, size_t n)
 {
-  size_t j;
-
   if (n >= mlp->nitems)
     return;
   message_free (mlp->item[n]);
-  for (j = n + 1; j < mlp->nitems; ++j)
+  for (size_t j = n + 1; j < mlp->nitems; ++j)
     mlp->item[j - 1] = mlp->item[j];
   mlp->nitems--;
 
@@ -450,9 +476,8 @@ void
 message_list_remove_if_not (message_list_ty *mlp,
                             message_predicate_ty *predicate)
 {
-  size_t i, j;
-
-  for (j = 0, i = 0; j < mlp->nitems; j++)
+  size_t i = 0;
+  for (size_t j = 0; j < mlp->nitems; j++)
     if (predicate (mlp->item[j]))
       mlp->item[i++] = mlp->item[j];
   if (mlp->use_hashtable && i < mlp->nitems)
@@ -471,12 +496,11 @@ message_list_msgids_changed (message_list_ty *mlp)
   if (mlp->use_hashtable)
     {
       unsigned long int size = mlp->htable.size;
-      size_t j;
 
       hash_destroy (&mlp->htable);
       hash_init (&mlp->htable, size);
 
-      for (j = 0; j < mlp->nitems; j++)
+      for (size_t j = 0; j < mlp->nitems; j++)
         {
           message_ty *mp = mlp->item[j];
 
@@ -498,11 +522,9 @@ message_list_msgids_changed (message_list_ty *mlp)
 message_list_ty *
 message_list_copy (message_list_ty *mlp, int copy_level)
 {
-  message_list_ty *result;
-  size_t j;
+  message_list_ty *result = message_list_alloc (mlp->use_hashtable);
 
-  result = message_list_alloc (mlp->use_hashtable);
-  for (j = 0; j < mlp->nitems; j++)
+  for (size_t j = 0; j < mlp->nitems; j++)
     {
       message_ty *mp = mlp->item[j];
 
@@ -522,7 +544,6 @@ message_list_search (const message_list_ty *mlp,
       char *alloced_key;
       const char *key;
       size_t keylen;
-
       if (msgctxt != NULL)
         {
           /* Concatenate the msgctxt and msgid, to form the hash table key.  */
@@ -542,28 +563,23 @@ message_list_search (const message_list_ty *mlp,
           keylen = strlen (msgid) + 1;
         }
 
-      {
-        void *htable_value;
-        int found = !hash_find_entry (&mlp->htable, key, keylen, &htable_value);
+      void *htable_value;
+      int found = !hash_find_entry (&mlp->htable, key, keylen, &htable_value);
 
-        if (msgctxt != NULL)
-          freea (alloced_key);
+      if (msgctxt != NULL)
+        freea (alloced_key);
 
-        if (found)
-          return (message_ty *) htable_value;
-        else
-          return NULL;
-      }
+      if (found)
+        return (message_ty *) htable_value;
+      else
+        return NULL;
     }
   else
     {
-      size_t j;
-
-      for (j = 0; j < mlp->nitems; ++j)
+      for (size_t j = 0; j < mlp->nitems; ++j)
         {
-          message_ty *mp;
+          message_ty *mp = mlp->item[j];
 
-          mp = mlp->item[j];
           if ((msgctxt != NULL
                ? mp->msgctxt != NULL && strcmp (msgctxt, mp->msgctxt) == 0
                : mp->msgctxt == NULL)
@@ -595,19 +611,17 @@ fuzzy_search_goal_function (const message_ty *mp,
       lower_bound -= bonus * 1.01;
     }
 
-  {
-    /* The use of 'volatile' guarantees that excess precision bits are dropped
-       before the addition and before the following comparison at the caller's
-       site.  It is necessary on x86 systems where double-floats are not IEEE
-       compliant by default, to avoid that msgmerge results become platform and
-       compiler option dependent.  'volatile' is a portable alternative to
-       gcc's -ffloat-store option.  */
-    volatile double weight = fstrcmp_bounded (msgid, mp->msgid, lower_bound);
+  /* The use of 'volatile' guarantees that excess precision bits are dropped
+     before the addition and before the following comparison at the caller's
+     site.  It is necessary on x86 systems where double-floats are not IEEE
+     compliant by default, to avoid that msgmerge results become platform and
+     compiler option dependent.  'volatile' is a portable alternative to
+     gcc's -ffloat-store option.  */
+  volatile double weight = fstrcmp_bounded (msgid, mp->msgid, lower_bound);
 
-    weight += bonus;
+  weight += bonus;
 
-    return weight;
-  }
+  return weight;
 }
 
 
@@ -616,15 +630,11 @@ message_list_search_fuzzy_inner (message_list_ty *mlp,
                                  const char *msgctxt, const char *msgid,
                                  double *best_weight_p)
 {
-  size_t j;
-  message_ty *best_mp;
+  message_ty *best_mp = NULL;
 
-  best_mp = NULL;
-  for (j = 0; j < mlp->nitems; ++j)
+  for (size_t j = 0; j < mlp->nitems; ++j)
     {
-      message_ty *mp;
-
-      mp = mlp->item[j];
+      message_ty *mp = mlp->item[j];
 
       if (mp->msgstr != NULL && mp->msgstr[0] != '\0')
         {
@@ -637,6 +647,7 @@ message_list_search_fuzzy_inner (message_list_ty *mlp,
             }
         }
     }
+
   return best_mp;
 }
 
@@ -645,9 +656,7 @@ message_ty *
 message_list_search_fuzzy (message_list_ty *mlp,
                            const char *msgctxt, const char *msgid)
 {
-  double best_weight;
-
-  best_weight = FUZZY_THRESHOLD;
+  double best_weight = FUZZY_THRESHOLD;
   return message_list_search_fuzzy_inner (mlp, msgctxt, msgid, &best_weight);
 }
 
@@ -655,12 +664,11 @@ message_list_search_fuzzy (message_list_ty *mlp,
 message_list_list_ty *
 message_list_list_alloc ()
 {
-  message_list_list_ty *mllp;
-
-  mllp = XMALLOC (message_list_list_ty);
+  message_list_list_ty *mllp = XMALLOC (message_list_list_ty);
   mllp->nitems = 0;
   mllp->nitems_max = 0;
   mllp->item = NULL;
+
   return mllp;
 }
 
@@ -668,10 +676,8 @@ message_list_list_alloc ()
 void
 message_list_list_free (message_list_list_ty *mllp, int keep_level)
 {
-  size_t j;
-
   if (keep_level < 2)
-    for (j = 0; j < mllp->nitems; ++j)
+    for (size_t j = 0; j < mllp->nitems; ++j)
       message_list_free (mllp->item[j], keep_level);
   if (mllp->item)
     free (mllp->item);
@@ -684,10 +690,8 @@ message_list_list_append (message_list_list_ty *mllp, message_list_ty *mlp)
 {
   if (mllp->nitems >= mllp->nitems_max)
     {
-      size_t nbytes;
-
       mllp->nitems_max = mllp->nitems_max * 2 + 4;
-      nbytes = mllp->nitems_max * sizeof (message_list_ty *);
+      size_t nbytes = mllp->nitems_max * sizeof (message_list_ty *);
       mllp->item = xrealloc (mllp->item, nbytes);
     }
   mllp->item[mllp->nitems++] = mlp;
@@ -698,9 +702,7 @@ void
 message_list_list_append_list (message_list_list_ty *mllp,
                                message_list_list_ty *mllp2)
 {
-  size_t j;
-
-  for (j = 0; j < mllp2->nitems; ++j)
+  for (size_t j = 0; j < mllp2->nitems; ++j)
     message_list_list_append (mllp, mllp2->item[j]);
 }
 
@@ -709,19 +711,12 @@ message_ty *
 message_list_list_search (message_list_list_ty *mllp,
                           const char *msgctxt, const char *msgid)
 {
-  message_ty *best_mp;
-  int best_weight; /* 0: not found, 1: found without msgstr, 2: translated */
-  size_t j;
-
-  best_mp = NULL;
-  best_weight = 0;
-  for (j = 0; j < mllp->nitems; ++j)
+  message_ty *best_mp = NULL;
+  int best_weight = 0; /* 0: not found, 1: found without msgstr, 2: translated */
+  for (size_t j = 0; j < mllp->nitems; ++j)
     {
-      message_list_ty *mlp;
-      message_ty *mp;
-
-      mlp = mllp->item[j];
-      mp = message_list_search (mlp, msgctxt, msgid);
+      message_list_ty *mlp = mllp->item[j];
+      message_ty *mp = message_list_search (mlp, msgctxt, msgid);
       if (mp)
         {
           int weight = (mp->msgstr_len == 1 && mp->msgstr[0] == '\0' ? 1 : 2);
@@ -741,19 +736,13 @@ message_ty *
 message_list_list_search_fuzzy (message_list_list_ty *mllp,
                                 const char *msgctxt, const char *msgid)
 {
-  size_t j;
-  double best_weight;
-  message_ty *best_mp;
-
-  best_weight = FUZZY_THRESHOLD;
-  best_mp = NULL;
-  for (j = 0; j < mllp->nitems; ++j)
+  double best_weight = FUZZY_THRESHOLD;
+  message_ty *best_mp = NULL;
+  for (size_t j = 0; j < mllp->nitems; ++j)
     {
-      message_list_ty *mlp;
-      message_ty *mp;
-
-      mlp = mllp->item[j];
-      mp = message_list_search_fuzzy_inner (mlp, msgctxt, msgid, &best_weight);
+      message_list_ty *mlp = mllp->item[j];
+      message_ty *mp =
+        message_list_search_fuzzy_inner (mlp, msgctxt, msgid, &best_weight);
       if (mp)
         best_mp = mp;
     }
@@ -765,11 +754,10 @@ message_list_list_search_fuzzy (message_list_list_ty *mllp,
 msgdomain_ty*
 msgdomain_alloc (const char *domain, bool use_hashtable)
 {
-  msgdomain_ty *mdp;
-
-  mdp = XMALLOC (msgdomain_ty);
+  msgdomain_ty *mdp = XMALLOC (msgdomain_ty);
   mdp->domain = domain;
   mdp->messages = message_list_alloc (use_hashtable);
+
   return mdp;
 }
 
@@ -785,9 +773,7 @@ msgdomain_free (msgdomain_ty *mdp)
 msgdomain_list_ty *
 msgdomain_list_alloc (bool use_hashtable)
 {
-  msgdomain_list_ty *mdlp;
-
-  mdlp = XMALLOC (msgdomain_list_ty);
+  msgdomain_list_ty *mdlp = XMALLOC (msgdomain_list_ty);
   /* Put the default domain first, so that when we output it,
      we can omit the 'domain' directive.  */
   mdlp->nitems = 1;
@@ -796,6 +782,7 @@ msgdomain_list_alloc (bool use_hashtable)
   mdlp->item[0] = msgdomain_alloc (MESSAGE_DOMAIN_DEFAULT, use_hashtable);
   mdlp->use_hashtable = use_hashtable;
   mdlp->encoding = NULL;
+
   return mdlp;
 }
 
@@ -803,9 +790,7 @@ msgdomain_list_alloc (bool use_hashtable)
 void
 msgdomain_list_free (msgdomain_list_ty *mdlp)
 {
-  size_t j;
-
-  for (j = 0; j < mdlp->nitems; ++j)
+  for (size_t j = 0; j < mdlp->nitems; ++j)
     msgdomain_free (mdlp->item[j]);
   if (mdlp->item)
     free (mdlp->item);
@@ -818,10 +803,8 @@ msgdomain_list_append (msgdomain_list_ty *mdlp, msgdomain_ty *mdp)
 {
   if (mdlp->nitems >= mdlp->nitems_max)
     {
-      size_t nbytes;
-
       mdlp->nitems_max = mdlp->nitems_max * 2 + 4;
-      nbytes = mdlp->nitems_max * sizeof (msgdomain_ty *);
+      size_t nbytes = mdlp->nitems_max * sizeof (msgdomain_ty *);
       mdlp->item = xrealloc (mdlp->item, nbytes);
     }
   mdlp->item[mdlp->nitems++] = mdp;
@@ -832,9 +815,7 @@ msgdomain_list_append (msgdomain_list_ty *mdlp, msgdomain_ty *mdp)
 void
 msgdomain_list_append_list (msgdomain_list_ty *mdlp, msgdomain_list_ty *mdlp2)
 {
-  size_t j;
-
-  for (j = 0; j < mdlp2->nitems; ++j)
+  for (size_t j = 0; j < mdlp2->nitems; ++j)
     msgdomain_list_append (mdlp, mdlp2->item[j]);
 }
 #endif
@@ -844,9 +825,7 @@ message_list_ty *
 msgdomain_list_sublist (msgdomain_list_ty *mdlp, const char *domain,
                         bool create)
 {
-  size_t j;
-
-  for (j = 0; j < mdlp->nitems; j++)
+  for (size_t j = 0; j < mdlp->nitems; j++)
     if (strcmp (mdlp->item[j]->domain, domain) == 0)
       return mdlp->item[j]->messages;
 
@@ -867,24 +846,20 @@ msgdomain_list_sublist (msgdomain_list_ty *mdlp, const char *domain,
 msgdomain_list_ty *
 msgdomain_list_copy (msgdomain_list_ty *mdlp, int copy_level)
 {
-  msgdomain_list_ty *result;
-  size_t j;
-
-  result = XMALLOC (msgdomain_list_ty);
+  msgdomain_list_ty *result = XMALLOC (msgdomain_list_ty);
   result->nitems = 0;
   result->nitems_max = 0;
   result->item = NULL;
   result->use_hashtable = mdlp->use_hashtable;
   result->encoding = mdlp->encoding;
 
-  for (j = 0; j < mdlp->nitems; j++)
+  for (size_t j = 0; j < mdlp->nitems; j++)
     {
       msgdomain_ty *mdp = mdlp->item[j];
 
       if (copy_level < 2)
         {
           msgdomain_ty *result_mdp = XMALLOC (msgdomain_ty);
-
           result_mdp->domain = mdp->domain;
           result_mdp->messages = message_list_copy (mdp->messages, copy_level);
 
@@ -903,15 +878,10 @@ message_ty *
 msgdomain_list_search (msgdomain_list_ty *mdlp,
                        const char *msgctxt, const char *msgid)
 {
-  size_t j;
-
-  for (j = 0; j < mdlp->nitems; ++j)
+  for (size_t j = 0; j < mdlp->nitems; ++j)
     {
-      msgdomain_ty *mdp;
-      message_ty *mp;
-
-      mdp = mdlp->item[j];
-      mp = message_list_search (mdp->messages, msgctxt, msgid);
+      msgdomain_ty *mdp = mdlp->item[j];
+      message_ty *mp = message_list_search (mdp->messages, msgctxt, msgid);
       if (mp)
         return mp;
     }
@@ -925,20 +895,14 @@ message_ty *
 msgdomain_list_search_fuzzy (msgdomain_list_ty *mdlp,
                              const char *msgctxt, const char *msgid)
 {
-  size_t j;
-  double best_weight;
-  message_ty *best_mp;
-
-  best_weight = FUZZY_THRESHOLD;
-  best_mp = NULL;
-  for (j = 0; j < mdlp->nitems; ++j)
+  double best_weight = FUZZY_THRESHOLD;
+  message_ty *best_mp = NULL;
+  for (size_t j = 0; j < mdlp->nitems; ++j)
     {
-      msgdomain_ty *mdp;
-      message_ty *mp;
-
-      mdp = mdlp->item[j];
-      mp = message_list_search_fuzzy_inner (mdp->messages, msgctxt, msgid,
-                                            &best_weight);
+      msgdomain_ty *mdp = mdlp->item[j];
+      message_ty *mp =
+        message_list_search_fuzzy_inner (mdp->messages, msgctxt, msgid,
+                                         &best_weight);
       if (mp)
         best_mp = mp;
     }
