@@ -1,6 +1,5 @@
 /* Message list test for ASCII character set.
-   Copyright (C) 2001-2025 Free Software Foundation, Inc.
-   Written by Bruno Haible <haible@clisp.cons.org>, 2001.
+   Copyright (C) 2001-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,10 +14,10 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
+/* Written by Bruno Haible.  */
 
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif
+
+#include <config.h>
 
 /* Specification.  */
 #include "msgl-ascii.h"
@@ -42,8 +41,7 @@ bool
 is_ascii_string_desc (string_desc_t string)
 {
   size_t len = sd_length (string);
-  size_t i;
-  for (i = 0; i < len; i++)
+  for (size_t i = 0; i < len; i++)
     if (!c_isascii ((unsigned char) sd_char_at (string, i)))
       return false;
   return true;
@@ -52,10 +50,8 @@ is_ascii_string_desc (string_desc_t string)
 bool
 is_ascii_string_list (const string_list_ty *slp)
 {
-  size_t i;
-
   if (slp != NULL)
-    for (i = 0; i < slp->nitems; i++)
+    for (size_t i = 0; i < slp->nitems; i++)
       if (!is_ascii_string (slp->item[i]))
         return false;
   return true;
@@ -64,12 +60,14 @@ is_ascii_string_list (const string_list_ty *slp)
 bool
 is_ascii_message (const message_ty *mp)
 {
-  const char *p = mp->msgstr;
-  const char *p_end = p + mp->msgstr_len;
+  {
+    const char *p = mp->msgstr;
+    const char *p_end = p + mp->msgstr_len;
 
-  for (; p < p_end; p++)
-    if (!c_isascii ((unsigned char) *p))
-      return false;
+    for (; p < p_end; p++)
+      if (!c_isascii ((unsigned char) *p))
+        return false;
+  }
 
   if (!is_ascii_string_list (mp->comment))
     return false;
@@ -101,9 +99,7 @@ is_ascii_message (const message_ty *mp)
 bool
 is_ascii_message_list (const message_list_ty *mlp)
 {
-  size_t j;
-
-  for (j = 0; j < mlp->nitems; j++)
+  for (size_t j = 0; j < mlp->nitems; j++)
     if (!is_ascii_message (mlp->item[j]))
       return false;
 
@@ -113,9 +109,7 @@ is_ascii_message_list (const message_list_ty *mlp)
 bool
 is_ascii_msgdomain_list (const msgdomain_list_ty *mdlp)
 {
-  size_t k;
-
-  for (k = 0; k < mdlp->nitems; k++)
+  for (size_t k = 0; k < mdlp->nitems; k++)
     if (!is_ascii_message_list (mdlp->item[k]->messages))
       return false;
 
