@@ -1,6 +1,5 @@
 /* Message list concatenation and duplicate handling.
-   Copyright (C) 2001-2003, 2005-2008, 2012, 2015, 2019-2021, 2023 Free Software Foundation, Inc.
-   Written by Bruno Haible <haible@clisp.cons.org>, 2001.
+   Copyright (C) 2001-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,10 +14,10 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
+/* Written by Bruno Haible.  */
 
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif
+
+#include <config.h>
 #include <alloca.h>
 
 /* Specification.  */
@@ -30,16 +29,17 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "error.h"
+#include <error.h>
 #include "xerror.h"
 #include "xvasprintf.h"
 #include "message.h"
-#include "read-catalog.h"
+#include "read-catalog-file.h"
 #include "po-charset.h"
 #include "msgl-ascii.h"
 #include "msgl-ofn.h"
 #include "msgl-equal.h"
 #include "msgl-iconv.h"
+#include "xerror-handler.h"
 #include "xalloc.h"
 #include "xmalloca.h"
 #include "c-strstr.h"
@@ -111,34 +111,27 @@ catenate_msgdomain_list (string_list_ty *file_list,
 {
   const char * const *files = file_list->item;
   size_t nfiles = file_list->nitems;
-  msgdomain_list_ty **mdlps;
-  const char ***canon_charsets;
-  const char ***identifications;
-  msgdomain_list_ty *total_mdlp;
-  const char *canon_to_code;
-  size_t n, j;
 
   /* Read input files.  */
-  mdlps = XNMALLOC (nfiles, msgdomain_list_ty *);
-  for (n = 0; n < nfiles; n++)
+  msgdomain_list_ty **mdlps = XNMALLOC (nfiles, msgdomain_list_ty *);
+  for (size_t n = 0; n < nfiles; n++)
     mdlps[n] = read_catalog_file (files[n], input_syntax);
 
   /* Determine the canonical name of each input file's encoding.  */
-  canon_charsets = XNMALLOC (nfiles, const char **);
-  for (n = 0; n < nfiles; n++)
+  const char ***canon_charsets = XNMALLOC (nfiles, const char **);
+  for (size_t n = 0; n < nfiles; n++)
     {
       msgdomain_list_ty *mdlp = mdlps[n];
-      size_t k;
 
       canon_charsets[n] = XNMALLOC (mdlp->nitems, const char *);
-      for (k = 0; k < mdlp->nitems; k++)
+      for (size_t k = 0; k < mdlp->nitems; k++)
         {
           message_list_ty *mlp = mdlp->item[k]->messages;
           const char *canon_from_code = NULL;
 
           if (mlp->nitems > 0)
             {
-              for (j = 0; j < mlp->nitems; j++)
+              for (size_t j = 0; j < mlp->nitems; j++)
                 if (is_header (mlp->item[j]) && !mlp->item[j]->obsolete)
                   {
                     const char *header = mlp->item[j]->msgstr;
@@ -149,17 +142,14 @@ catenate_msgdomain_list (string_list_ty *file_list,
 
                         if (charsetstr != NULL)
                           {
-                            size_t len;
-                            char *charset;
-                            const char *canon_charset;
-
                             charsetstr += strlen ("charset=");
-                            len = strcspn (charsetstr, " \t\n");
-                            charset = (char *) xmalloca (len + 1);
+                            size_t len = strcspn (charsetstr, " \t\n");
+
+                            char *charset = (char *) xmalloca (len + 1);
                             memcpy (charset, charsetstr, len);
                             charset[len] = '\0';
 
-                            canon_charset = po_charset_canonicalize (charset);
+                            const char *canon_charset = po_charset_canonicalize (charset);
                             if (canon_charset == NULL)
                               {
                                 /* Don't give an error for POT files, because
@@ -217,21 +207,20 @@ catenate_msgdomain_list (string_list_ty *file_list,
     }
 
   /* Determine textual identifications of each file/domain combination.  */
-  identifications = XNMALLOC (nfiles, const char **);
-  for (n = 0; n < nfiles; n++)
+  const char ***identifications = XNMALLOC (nfiles, const char **);
+  for (size_t n = 0; n < nfiles; n++)
     {
       const char *filename = last_component (files[n]);
       msgdomain_list_ty *mdlp = mdlps[n];
-      size_t k;
 
       identifications[n] = XNMALLOC (mdlp->nitems, const char *);
-      for (k = 0; k < mdlp->nitems; k++)
+      for (size_t k = 0; k < mdlp->nitems; k++)
         {
           const char *domain = mdlp->item[k]->domain;
           message_list_ty *mlp = mdlp->item[k]->messages;
           char *project_id = NULL;
 
-          for (j = 0; j < mlp->nitems; j++)
+          for (size_t j = 0; j < mlp->nitems; j++)
             if (is_header (mlp->item[j]) && !mlp->item[j]->obsolete)
               {
                 const char *header = mlp->item[j]->msgstr;
@@ -242,11 +231,9 @@ catenate_msgdomain_list (string_list_ty *file_list,
 
                     if (cp != NULL)
                       {
-                        const char *endp;
-
                         cp += sizeof ("Project-Id-Version:") - 1;
 
-                        endp = strchr (cp, '\n');
+                        const char *endp = strchr (cp, '\n');
                         if (endp == NULL)
                           endp = cp + strlen (cp);
 
@@ -256,6 +243,7 @@ catenate_msgdomain_list (string_list_ty *file_list,
                         if (cp < endp)
                           {
                             size_t len = endp - cp;
+
                             project_id = XNMALLOC (len + 1, char);
                             memcpy (project_id, cp, len);
                             project_id[len] = '\0';
@@ -279,27 +267,22 @@ catenate_msgdomain_list (string_list_ty *file_list,
      If for a message, there is at least one non-fuzzy, non-empty translation,
      use only the non-fuzzy, non-empty translations.  Otherwise use the
      fuzzy or empty translations as well.  */
-  total_mdlp = msgdomain_list_alloc (true);
-  for (n = 0; n < nfiles; n++)
+  msgdomain_list_ty *total_mdlp = msgdomain_list_alloc (true);
+  for (size_t n = 0; n < nfiles; n++)
     {
       msgdomain_list_ty *mdlp = mdlps[n];
-      size_t k;
 
-      for (k = 0; k < mdlp->nitems; k++)
+      for (size_t k = 0; k < mdlp->nitems; k++)
         {
           const char *domain = mdlp->item[k]->domain;
           message_list_ty *mlp = mdlp->item[k]->messages;
-          message_list_ty *total_mlp;
+          message_list_ty *total_mlp = msgdomain_list_sublist (total_mdlp, domain, true);
 
-          total_mlp = msgdomain_list_sublist (total_mdlp, domain, true);
-
-          for (j = 0; j < mlp->nitems; j++)
+          for (size_t j = 0; j < mlp->nitems; j++)
             {
               message_ty *mp = mlp->item[j];
-              message_ty *tmp;
-              size_t i;
 
-              tmp = message_list_search (total_mlp, mp->msgctxt, mp->msgid);
+              message_ty *tmp = message_list_search (total_mlp, mp->msgctxt, mp->msgid);
               if (tmp != NULL)
                 {
                   if ((tmp->msgid_plural != NULL) != (mp->msgid_plural != NULL))
@@ -316,13 +299,11 @@ catenate_msgdomain_list (string_list_ty *file_list,
                   tmp = message_alloc (mp->msgctxt, mp->msgid, mp->msgid_plural,
                                        NULL, 0, &mp->pos);
                   tmp->is_fuzzy = true; /* may be set to false later */
-                  for (i = 0; i < NFORMATS; i++)
+                  for (size_t i = 0; i < NFORMATS; i++)
                     tmp->is_format[i] = undecided; /* may be set to yes/no later */
                   tmp->range.min = - INT_MAX;
                   tmp->range.max = - INT_MAX;
                   tmp->do_wrap = yes; /* may be set to no later */
-                  for (i = 0; i < NSYNTAXCHECKS; i++)
-                    tmp->do_syntax_check[i] = undecided; /* may be set to yes/no later */
                   tmp->obsolete = true; /* may be set to false later */
                   tmp->alternative_count = 0;
                   tmp->alternative = NULL;
@@ -350,12 +331,11 @@ catenate_msgdomain_list (string_list_ty *file_list,
     }
 
   /* Remove messages that are not used and need not be converted.  */
-  for (n = 0; n < nfiles; n++)
+  for (size_t n = 0; n < nfiles; n++)
     {
       msgdomain_list_ty *mdlp = mdlps[n];
-      size_t k;
 
-      for (k = 0; k < mdlp->nitems; k++)
+      for (size_t k = 0; k < mdlp->nitems; k++)
         {
           message_list_ty *mlp = mdlp->item[k]->messages;
 
@@ -369,23 +349,18 @@ catenate_msgdomain_list (string_list_ty *file_list,
             canon_charsets[n][k] = NULL;
         }
     }
-  {
-    size_t k;
+  for (size_t k = 0; k < total_mdlp->nitems; k++)
+    {
+      message_list_ty *mlp = total_mdlp->item[k]->messages;
 
-    for (k = 0; k < total_mdlp->nitems; k++)
-      {
-        message_list_ty *mlp = total_mdlp->item[k]->messages;
-
-        message_list_remove_if_not (mlp, is_message_selected);
-      }
-  }
+      message_list_remove_if_not (mlp, is_message_selected);
+    }
 
   /* Determine the common known a-priori encoding, if any.  */
   if (nfiles > 0)
     {
       bool all_same_encoding = true;
-
-      for (n = 1; n < nfiles; n++)
+      for (size_t n = 1; n < nfiles; n++)
         if (mdlps[n]->encoding != mdlps[0]->encoding)
           {
             all_same_encoding = false;
@@ -399,7 +374,7 @@ catenate_msgdomain_list (string_list_ty *file_list,
   /* Determine whether we need a target encoding that contains the control
      characters needed for escaping file names with spaces.  */
   bool has_filenames_with_spaces = false;
-  for (n = 0; n < nfiles; n++)
+  for (size_t n = 0; n < nfiles; n++)
     {
       has_filenames_with_spaces =
         has_filenames_with_spaces
@@ -407,6 +382,7 @@ catenate_msgdomain_list (string_list_ty *file_list,
     }
 
   /* Determine the target encoding for the remaining messages.  */
+  const char *canon_to_code;
   if (to_code != NULL)
     {
       /* Canonicalize target encoding.  */
@@ -435,12 +411,11 @@ catenate_msgdomain_list (string_list_ty *file_list,
       bool with_UTF8 = false;
       bool all_ASCII_compatible = true;
 
-      for (n = 0; n < nfiles; n++)
+      for (size_t n = 0; n < nfiles; n++)
         {
           msgdomain_list_ty *mdlp = mdlps[n];
-          size_t k;
 
-          for (k = 0; k < mdlp->nitems; k++)
+          for (size_t k = 0; k < mdlp->nitems; k++)
             if (canon_charsets[n][k] != NULL)
               {
                 if (canon_charsets[n][k] == po_charset_ascii)
@@ -519,12 +494,11 @@ Converting the output to %s.\n\
 
   /* Now convert the remaining messages to canon_to_code.  */
   if (canon_to_code != NULL)
-    for (n = 0; n < nfiles; n++)
+    for (size_t n = 0; n < nfiles; n++)
       {
         msgdomain_list_ty *mdlp = mdlps[n];
-        size_t k;
 
-        for (k = 0; k < mdlp->nitems; k++)
+        for (size_t k = 0; k < mdlp->nitems; k++)
           if (canon_charsets[n][k] != NULL)
             /* If the user hasn't given a to_code, don't bother doing a noop
                conversion that would only replace the charset name in the
@@ -532,7 +506,7 @@ Converting the output to %s.\n\
             if (!(to_code == NULL && canon_charsets[n][k] == canon_to_code))
               if (iconv_message_list (mdlp->item[k]->messages,
                                       canon_charsets[n][k], canon_to_code,
-                                      files[n]))
+                                      files[n], textmode_xerror_handler))
                 {
                   multiline_error (xstrdup (""),
                                    xasprintf (_("\
@@ -547,20 +521,18 @@ UTF-8 encoded from the beginning, i.e. already in your source code files.\n"),
       }
 
   /* Fill the resulting messages.  */
-  for (n = 0; n < nfiles; n++)
+  for (size_t n = 0; n < nfiles; n++)
     {
       msgdomain_list_ty *mdlp = mdlps[n];
-      size_t k;
 
-      for (k = 0; k < mdlp->nitems; k++)
+      for (size_t k = 0; k < mdlp->nitems; k++)
         {
           message_list_ty *mlp = mdlp->item[k]->messages;
 
-          for (j = 0; j < mlp->nitems; j++)
+          for (size_t j = 0; j < mlp->nitems; j++)
             {
               message_ty *mp = mlp->item[j];
               message_ty *tmp = mp->tmp;
-              size_t i;
 
               /* No need to discard unneeded weak translations here;
                  they have already been filtered out above.  */
@@ -571,22 +543,20 @@ UTF-8 encoded from the beginning, i.e. already in your source code files.\n"),
                   tmp->msgstr_len = mp->msgstr_len;
                   tmp->pos = mp->pos;
                   if (mp->comment)
-                    for (i = 0; i < mp->comment->nitems; i++)
+                    for (size_t i = 0; i < mp->comment->nitems; i++)
                       message_comment_append (tmp, mp->comment->item[i]);
                   if (mp->comment_dot)
-                    for (i = 0; i < mp->comment_dot->nitems; i++)
+                    for (size_t i = 0; i < mp->comment_dot->nitems; i++)
                       message_comment_dot_append (tmp,
                                                   mp->comment_dot->item[i]);
-                  for (i = 0; i < mp->filepos_count; i++)
+                  for (size_t i = 0; i < mp->filepos_count; i++)
                     message_comment_filepos (tmp, mp->filepos[i].file_name,
                                              mp->filepos[i].line_number);
                   tmp->is_fuzzy = mp->is_fuzzy;
-                  for (i = 0; i < NFORMATS; i++)
+                  for (size_t i = 0; i < NFORMATS; i++)
                     tmp->is_format[i] = mp->is_format[i];
                   tmp->range = mp->range;
                   tmp->do_wrap = mp->do_wrap;
-                  for (i = 0; i < NSYNTAXCHECKS; i++)
-                    tmp->do_syntax_check[i] = mp->do_syntax_check[i];
                   tmp->prev_msgctxt = mp->prev_msgctxt;
                   tmp->prev_msgid = mp->prev_msgid;
                   tmp->prev_msgid_plural = mp->prev_msgid_plural;
@@ -606,16 +576,16 @@ UTF-8 encoded from the beginning, i.e. already in your source code files.\n"),
                       tmp->prev_msgid_plural = mp->prev_msgid_plural;
                     }
                   if (mp->comment && tmp->comment == NULL)
-                    for (i = 0; i < mp->comment->nitems; i++)
+                    for (size_t i = 0; i < mp->comment->nitems; i++)
                       message_comment_append (tmp, mp->comment->item[i]);
                   if (mp->comment_dot && tmp->comment_dot == NULL)
-                    for (i = 0; i < mp->comment_dot->nitems; i++)
+                    for (size_t i = 0; i < mp->comment_dot->nitems; i++)
                       message_comment_dot_append (tmp,
                                                   mp->comment_dot->item[i]);
-                  for (i = 0; i < mp->filepos_count; i++)
+                  for (size_t i = 0; i < mp->filepos_count; i++)
                     message_comment_filepos (tmp, mp->filepos[i].file_name,
                                              mp->filepos[i].line_number);
-                  for (i = 0; i < NFORMATS; i++)
+                  for (size_t i = 0; i < NFORMATS; i++)
                     if (tmp->is_format[i] == undecided)
                       tmp->is_format[i] = mp->is_format[i];
                   if (tmp->range.min == - INT_MAX
@@ -635,9 +605,6 @@ UTF-8 encoded from the beginning, i.e. already in your source code files.\n"),
                     }
                   if (tmp->do_wrap == undecided)
                     tmp->do_wrap = mp->do_wrap;
-                  for (i = 0; i < NSYNTAXCHECKS; i++)
-                    if (tmp->do_syntax_check[i] == undecided)
-                      tmp->do_syntax_check[i] = mp->do_syntax_check[i];
                   tmp->obsolete = false;
                 }
               else
@@ -645,29 +612,30 @@ UTF-8 encoded from the beginning, i.e. already in your source code files.\n"),
                   /* Copy mp, among others, into tmp.  */
                   char *id = xasprintf ("#-#-#-#-#  %s  #-#-#-#-#",
                                         identifications[n][k]);
-                  size_t nbytes;
 
                   if (tmp->alternative_count == 0)
                     tmp->pos = mp->pos;
 
-                  i = tmp->alternative_count;
-                  nbytes = (i + 1) * sizeof (struct altstr);
-                  tmp->alternative = xrealloc (tmp->alternative, nbytes);
-                  tmp->alternative[i].msgstr = mp->msgstr;
-                  tmp->alternative[i].msgstr_len = mp->msgstr_len;
-                  tmp->alternative[i].msgstr_end =
-                    tmp->alternative[i].msgstr + tmp->alternative[i].msgstr_len;
-                  tmp->alternative[i].comment = mp->comment;
-                  tmp->alternative[i].comment_dot = mp->comment_dot;
-                  tmp->alternative[i].id = id;
-                  tmp->alternative_count = i + 1;
+                  {
+                    size_t i = tmp->alternative_count;
+                    size_t nbytes = (i + 1) * sizeof (struct altstr);
+                    tmp->alternative = xrealloc (tmp->alternative, nbytes);
+                    tmp->alternative[i].msgstr = mp->msgstr;
+                    tmp->alternative[i].msgstr_len = mp->msgstr_len;
+                    tmp->alternative[i].msgstr_end =
+                      tmp->alternative[i].msgstr + tmp->alternative[i].msgstr_len;
+                    tmp->alternative[i].comment = mp->comment;
+                    tmp->alternative[i].comment_dot = mp->comment_dot;
+                    tmp->alternative[i].id = id;
+                    tmp->alternative_count = i + 1;
+                  }
 
-                  for (i = 0; i < mp->filepos_count; i++)
+                  for (size_t i = 0; i < mp->filepos_count; i++)
                     message_comment_filepos (tmp, mp->filepos[i].file_name,
                                              mp->filepos[i].line_number);
                   if (!mp->is_fuzzy)
                     tmp->is_fuzzy = false;
-                  for (i = 0; i < NFORMATS; i++)
+                  for (size_t i = 0; i < NFORMATS; i++)
                     if (mp->is_format[i] == yes)
                       tmp->is_format[i] = yes;
                     else if (mp->is_format[i] == no
@@ -690,12 +658,6 @@ UTF-8 encoded from the beginning, i.e. already in your source code files.\n"),
                     }
                   if (mp->do_wrap == no)
                     tmp->do_wrap = no;
-                  for (i = 0; i < NSYNTAXCHECKS; i++)
-                    if (mp->do_syntax_check[i] == yes)
-                      tmp->do_syntax_check[i] = yes;
-                    else if (mp->do_syntax_check[i] == no
-                             && tmp->do_syntax_check[i] == undecided)
-                      tmp->do_syntax_check[i] = no;
                   /* Don't fill tmp->prev_msgid in this case.  */
                   if (!mp->obsolete)
                     tmp->obsolete = false;
@@ -703,158 +665,170 @@ UTF-8 encoded from the beginning, i.e. already in your source code files.\n"),
             }
         }
     }
-  {
-    size_t k;
+  for (size_t k = 0; k < total_mdlp->nitems; k++)
+    {
+      message_list_ty *mlp = total_mdlp->item[k]->messages;
 
-    for (k = 0; k < total_mdlp->nitems; k++)
-      {
-        message_list_ty *mlp = total_mdlp->item[k]->messages;
+      for (size_t j = 0; j < mlp->nitems; j++)
+        {
+          message_ty *tmp = mlp->item[j];
 
-        for (j = 0; j < mlp->nitems; j++)
-          {
-            message_ty *tmp = mlp->item[j];
+          if (tmp->alternative_count > 0)
+            {
+              /* Test whether all alternative translations are equal.  */
+              struct altstr *first = &tmp->alternative[0];
 
-            if (tmp->alternative_count > 0)
-              {
-                /* Test whether all alternative translations are equal.  */
-                struct altstr *first = &tmp->alternative[0];
-                size_t i;
-
-                for (i = 0; i < tmp->alternative_count; i++)
-                  if (!(tmp->alternative[i].msgstr_len == first->msgstr_len
-                        && memcmp (tmp->alternative[i].msgstr, first->msgstr,
-                                   first->msgstr_len) == 0))
+              bool all_equal = true;
+              for (size_t i = 0; i < tmp->alternative_count; i++)
+                if (!(tmp->alternative[i].msgstr_len == first->msgstr_len
+                      && memcmp (tmp->alternative[i].msgstr, first->msgstr,
+                                 first->msgstr_len) == 0))
+                  {
+                    all_equal = false;
                     break;
-
-                if (i == tmp->alternative_count)
-                  {
-                    /* All alternatives are equal.  */
-                    tmp->msgstr = first->msgstr;
-                    tmp->msgstr_len = first->msgstr_len;
                   }
-                else
-                  {
-                    /* Concatenate the alternative msgstrs into a single one,
-                       separated by markers.  */
-                    size_t len;
-                    const char *p;
-                    const char *p_end;
-                    char *new_msgstr;
-                    char *np;
 
-                    len = 0;
-                    for (i = 0; i < tmp->alternative_count; i++)
+              if (all_equal)
+                {
+                  /* All alternatives are equal.  */
+                  tmp->msgstr = first->msgstr;
+                  tmp->msgstr_len = first->msgstr_len;
+                }
+              else
+                {
+                  /* Concatenate the alternative msgstrs into a single one,
+                     separated by markers.  */
+                  char *new_msgstr;
+                  {
+                    size_t len = 0;
+                    for (size_t i = 0; i < tmp->alternative_count; i++)
                       {
                         size_t id_len = strlen (tmp->alternative[i].id);
 
                         len += tmp->alternative[i].msgstr_len;
 
-                        p = tmp->alternative[i].msgstr;
-                        p_end = tmp->alternative[i].msgstr_end;
+                        const char *p = tmp->alternative[i].msgstr;
+                        const char *p_end = tmp->alternative[i].msgstr_end;
                         for (; p < p_end; p += strlen (p) + 1)
                           len += id_len + 2;
                       }
-
                     new_msgstr = XNMALLOC (len, char);
-                    np = new_msgstr;
-                    for (;;)
-                      {
-                        /* Test whether there's one more plural form to
-                           process.  */
-                        for (i = 0; i < tmp->alternative_count; i++)
-                          if (tmp->alternative[i].msgstr
-                              < tmp->alternative[i].msgstr_end)
+                  }
+                  char *np = new_msgstr;
+                  for (;;)
+                    {
+                      /* Test whether there's one more plural form to
+                         process.  */
+                      bool all_plural_forms_processed = true;
+                      for (size_t i = 0; i < tmp->alternative_count; i++)
+                        if (tmp->alternative[i].msgstr
+                            < tmp->alternative[i].msgstr_end)
+                          {
+                            all_plural_forms_processed = false;
                             break;
-                        if (i == tmp->alternative_count)
-                          break;
+                          }
+
+                      if (all_plural_forms_processed)
+                        break;
+
+                      /* Process next plural form.  */
+                      for (size_t i = 0; i < tmp->alternative_count; i++)
+                        if (tmp->alternative[i].msgstr
+                            < tmp->alternative[i].msgstr_end)
+                          {
+                            if (np > new_msgstr && np[-1] != '\0'
+                                && np[-1] != '\n')
+                              *np++ = '\n';
 
-                        /* Process next plural form.  */
-                        for (i = 0; i < tmp->alternative_count; i++)
-                          if (tmp->alternative[i].msgstr
-                              < tmp->alternative[i].msgstr_end)
                             {
-                              if (np > new_msgstr && np[-1] != '\0'
-                                  && np[-1] != '\n')
-                                *np++ = '\n';
-
-                              len = strlen (tmp->alternative[i].id);
+                              size_t len = strlen (tmp->alternative[i].id);
                               memcpy (np, tmp->alternative[i].id, len);
                               np += len;
                               *np++ = '\n';
+                            }
 
-                              len = strlen (tmp->alternative[i].msgstr);
+                            {
+                              size_t len = strlen (tmp->alternative[i].msgstr);
                               memcpy (np, tmp->alternative[i].msgstr, len);
                               np += len;
                               tmp->alternative[i].msgstr += len + 1;
                             }
+                          }
 
-                        /* Plural forms are separated by NUL bytes.  */
-                        *np++ = '\0';
-                      }
-                    tmp->msgstr = new_msgstr;
-                    tmp->msgstr_len = np - new_msgstr;
+                      /* Plural forms are separated by NUL bytes.  */
+                      *np++ = '\0';
+                    }
+                  tmp->msgstr = new_msgstr;
+                  tmp->msgstr_len = np - new_msgstr;
 
-                    tmp->is_fuzzy = true;
-                  }
+                  tmp->is_fuzzy = true;
+                }
 
+              {
                 /* Test whether all alternative comments are equal.  */
-                for (i = 0; i < tmp->alternative_count; i++)
+                bool all_comments_equal = true;
+                for (size_t i = 0; i < tmp->alternative_count; i++)
                   if (tmp->alternative[i].comment == NULL
                       || !string_list_equal (tmp->alternative[i].comment,
                                              first->comment))
-                    break;
+                    {
+                      all_comments_equal = false;
+                      break;
+                    }
 
-                if (i == tmp->alternative_count)
+                if (all_comments_equal)
                   /* All alternatives are equal.  */
                   tmp->comment = first->comment;
                 else
                   /* Concatenate the alternative comments into a single one,
                      separated by markers.  */
-                  for (i = 0; i < tmp->alternative_count; i++)
+                  for (size_t i = 0; i < tmp->alternative_count; i++)
                     {
                       string_list_ty *slp = tmp->alternative[i].comment;
 
                       if (slp != NULL)
                         {
-                          size_t l;
-
                           message_comment_append (tmp, tmp->alternative[i].id);
-                          for (l = 0; l < slp->nitems; l++)
+                          for (size_t l = 0; l < slp->nitems; l++)
                             message_comment_append (tmp, slp->item[l]);
                         }
                     }
+              }
 
+              {
                 /* Test whether all alternative dot comments are equal.  */
-                for (i = 0; i < tmp->alternative_count; i++)
+                bool all_comments_equal = true;
+                for (size_t i = 0; i < tmp->alternative_count; i++)
                   if (tmp->alternative[i].comment_dot == NULL
                       || !string_list_equal (tmp->alternative[i].comment_dot,
                                              first->comment_dot))
-                    break;
+                    {
+                      all_comments_equal = false;
+                      break;
+                    }
 
-                if (i == tmp->alternative_count)
+                if (all_comments_equal)
                   /* All alternatives are equal.  */
                   tmp->comment_dot = first->comment_dot;
                 else
                   /* Concatenate the alternative dot comments into a single one,
                      separated by markers.  */
-                  for (i = 0; i < tmp->alternative_count; i++)
+                  for (size_t i = 0; i < tmp->alternative_count; i++)
                     {
                       string_list_ty *slp = tmp->alternative[i].comment_dot;
 
                       if (slp != NULL)
                         {
-                          size_t l;
-
                           message_comment_dot_append (tmp,
                                                       tmp->alternative[i].id);
-                          for (l = 0; l < slp->nitems; l++)
+                          for (size_t l = 0; l < slp->nitems; l++)
                             message_comment_dot_append (tmp, slp->item[l]);
                         }
                     }
               }
-          }
-      }
-  }
+            }
+        }
+    }
 
   return total_mdlp;
 }
