@@ -1,5 +1,5 @@
 /* Source file positions.
-   Copyright (C) 1995-1998, 2000-2001, 2015-2016 Free Software Foundation, Inc.
+   Copyright (C) 1995-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,7 +12,9 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
+
+/* Written by Peter Miller, Ulrich Drepper, and Bruno Haible.  */
 
 #ifndef _POS_H
 #define _POS_H
@@ -20,13 +22,20 @@
 /* Get size_t.  */
 #include <stddef.h>
 
+/* Get bool.  */
+#include <stdbool.h>
+
 /* Position of a message within a source file.
    Used for error reporting purposes.  */
 typedef struct lex_pos_ty lex_pos_ty;
 struct lex_pos_ty
 {
-  char *file_name;
+  const char *file_name;
   size_t line_number;
 };
 
+/* Determines whether the file name in the position has spaces.
+   Such spaces need special protection in PO files and .properties files.  */
+extern bool pos_filename_has_spaces (const struct lex_pos_ty *pos);
+
 #endif /* _POS_H */
