@@ -1,6 +1,5 @@
 /* xgettext Java backend.
-   Copyright (C) 2003-2025 Free Software Foundation, Inc.
-   Written by Bruno Haible <bruno@clisp.org>, 2003.
+   Copyright (C) 2003-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,9 +14,9 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif
+/* Written by Bruno Haible.  */
+
+#include <config.h>
 
 /* Specification.  */
 #include "x-java.h"
@@ -89,19 +88,17 @@ x_java_keyword (const char *name)
     default_keywords = false;
   else
     {
-      const char *end;
-      struct callshape shape;
-      const char *colon;
-
       if (keywords.table == NULL)
         hash_init (&keywords, 100);
 
+      const char *end;
+      struct callshape shape;
       split_keywordspec (name, &end, &shape);
 
       /* The characters between name and end should form a valid Java
          identifier sequence with dots.
          A colon means an invalid parse in split_keywordspec().  */
-      colon = strchr (name, ':');
+      const char *colon = strchr (name, ':');
       if (colon == NULL || colon >= end)
         insert_keyword_callshape (&keywords, name, end - name, &shape);
     }
@@ -268,11 +265,11 @@ static int phase2_pushback_length;
 static int
 phase2_getc ()
 {
-  int c;
-
   if (phase2_pushback_length)
     return phase2_pushback[--phase2_pushback_length];
 
+  int c;
+
   c = phase1_getc ();
   if (c == EOF)
     return P2_EOF;
@@ -282,10 +279,6 @@ phase2_getc ()
       if (c == 'u')
         {
           unsigned int u_count = 1;
-          unsigned char buf[4];
-          unsigned int n;
-          int i;
-
           for (;;)
             {
               c = phase1_getc ();
@@ -295,8 +288,9 @@ phase2_getc ()
             }
           phase1_ungetc (c);
 
-          n = 0;
-          for (i = 0; i < 4; i++)
+          unsigned char buf[4];
+          unsigned int n = 0;
+          for (int i = 0; i < 4; i++)
             {
               c = phase1_getc ();
 
@@ -483,11 +477,9 @@ static int last_non_comment_line;
 static int
 phase4_getc ()
 {
-  int c0;
   int c;
-  bool last_was_star;
 
-  c0 = phase3_getc ();
+  int c0 = phase3_getc ();
   if (RED (c0) != '/')
     return c0;
   c = phase3_getc ();
@@ -498,45 +490,47 @@ phase4_getc ()
       return c0;
 
     case '*':
-      /* C style comment.  */
-      comment_start ();
-      last_was_star = false;
-      for (;;)
-        {
-          c = phase3_getc ();
-          if (c == P2_EOF)
+      {
+        /* C style comment.  */
+        comment_start ();
+        bool last_was_star = false;
+        for (;;)
+          {
+            c = phase3_getc ();
+            if (c == P2_EOF)
+              break;
+            /* We skip all leading white space, but not EOLs.  */
+            if (!(comment_at_start () && (RED (c) == ' ' || RED (c) == '\t')))
+              comment_add (c);
+            switch (RED (c))
+              {
+              case '\n':
+                comment_line_end (1);
+                comment_start ();
+                last_was_star = false;
+                continue;
+
+              case '*':
+                last_was_star = true;
+                continue;
+
+              case '/':
+                if (last_was_star)
+                  {
+                    comment_line_end (2);
+                    break;
+                  }
+                FALLTHROUGH;
+
+              default:
+                last_was_star = false;
+                continue;
+              }
             break;
-          /* We skip all leading white space, but not EOLs.  */
-          if (!(comment_at_start () && (RED (c) == ' ' || RED (c) == '\t')))
-            comment_add (c);
-          switch (RED (c))
-            {
-            case '\n':
-              comment_line_end (1);
-              comment_start ();
-              last_was_star = false;
-              continue;
-
-            case '*':
-              last_was_star = true;
-              continue;
-
-            case '/':
-              if (last_was_star)
-                {
-                  comment_line_end (2);
-                  break;
-                }
-              FALLTHROUGH;
-
-            default:
-              last_was_star = false;
-              continue;
-            }
-          break;
-        }
-      last_comment_line = line_number;
-      return ' ';
+          }
+        last_comment_line = line_number;
+        return ' ';
+      }
 
     case '/':
       /* C++ style comment.  */
@@ -687,10 +681,10 @@ do_getc_escaped ()
 static void
 accumulate_escaped (struct mixed_string_buffer *literal, int delimiter)
 {
-  int c;
-
   for (;;)
     {
+      int c;
+
       /* Use phase 3, because phase 4 elides comments.  */
       c = phase3_getc ();
       if (c == P2_EOF || RED (c) == delimiter)
@@ -727,9 +721,8 @@ strip_indent (mixed_string_ty *ms)
   {
     size_t curr_line_indentation = 0;
     bool curr_line_blank = true;
-    size_t i;
 
-    for (i = 0; i < nsegments; i++)
+    for (size_t i = 0; i < nsegments; i++)
       {
         struct mixed_string_segment *segment = ms->segments[i];
 
@@ -739,9 +732,8 @@ strip_indent (mixed_string_ty *ms)
           {
             /* Consider Unicode whitespace characters.  */
             size_t seglength = segment->length;
-            size_t j;
 
-            for (j = 0; j < seglength; )
+            for (size_t j = 0; j < seglength; )
               {
                 ucs4_t uc;
                 int bytes =
@@ -777,9 +769,8 @@ strip_indent (mixed_string_ty *ms)
             /* When the encoding is not UTF-8, consider only ASCII whitespace
                characters.  */
             size_t seglength = segment->length;
-            size_t j;
 
-            for (j = 0; j < seglength; j++)
+            for (size_t j = 0; j < seglength; j++)
               {
                 char c = segment->contents[j];
                 if (c == '\n')
@@ -823,9 +814,8 @@ strip_indent (mixed_string_ty *ms)
     size_t start_of_trailing_whitespace_i = 0;
     size_t start_of_trailing_whitespace_j = 0;
     size_t whitespace_to_remove = minimum_indentation;
-    size_t i;
 
-    for (i = 0; i < nsegments; i++)
+    for (size_t i = 0; i < nsegments; i++)
       {
         struct mixed_string_segment *segment = ms->segments[i];
         /* Perform a sliding copy from segment->contents[from_j] to
@@ -855,9 +845,8 @@ strip_indent (mixed_string_ty *ms)
                            whitespace characters.  Remove all this whitespace.  */
                         if (start_of_curr_line_i < i)
                           {
-                            size_t k;
                             ms->segments[start_of_curr_line_i]->length = start_of_curr_line_j;
-                            for (k = start_of_curr_line_i + 1; k < i; k++)
+                            for (size_t k = start_of_curr_line_i + 1; k < i; k++)
                               ms->segments[k]->length = 0;
                             to_j = 0;
                           }
@@ -870,9 +859,8 @@ strip_indent (mixed_string_ty *ms)
                            current line.  */
                         if (start_of_trailing_whitespace_i < i)
                           {
-                            size_t k;
                             ms->segments[start_of_trailing_whitespace_i]->length = start_of_trailing_whitespace_j;
-                            for (k = start_of_trailing_whitespace_i + 1; k < i; k++)
+                            for (size_t k = start_of_trailing_whitespace_i + 1; k < i; k++)
                               ms->segments[k]->length = 0;
                             to_j = 0;
                           }
@@ -903,9 +891,8 @@ strip_indent (mixed_string_ty *ms)
                            characters from the current line.  */
                         if (start_of_curr_line_i < i)
                           {
-                            size_t k;
                             ms->segments[start_of_curr_line_i]->length = start_of_curr_line_j;
-                            for (k = start_of_curr_line_i + 1; k < i; k++)
+                            for (size_t k = start_of_curr_line_i + 1; k < i; k++)
                               ms->segments[k]->length = 0;
                             to_j = 0;
                           }
@@ -942,9 +929,8 @@ strip_indent (mixed_string_ty *ms)
                            whitespace characters.  Remove all this whitespace.  */
                         if (start_of_curr_line_i < i)
                           {
-                            size_t k;
                             ms->segments[start_of_curr_line_i]->length = start_of_curr_line_j;
-                            for (k = start_of_curr_line_i + 1; k < i; k++)
+                            for (size_t k = start_of_curr_line_i + 1; k < i; k++)
                               ms->segments[k]->length = 0;
                             to_j = 0;
                           }
@@ -957,9 +943,8 @@ strip_indent (mixed_string_ty *ms)
                            current line.  */
                         if (start_of_trailing_whitespace_i < i)
                           {
-                            size_t k;
                             ms->segments[start_of_trailing_whitespace_i]->length = start_of_trailing_whitespace_j;
-                            for (k = start_of_trailing_whitespace_i + 1; k < i; k++)
+                            for (size_t k = start_of_trailing_whitespace_i + 1; k < i; k++)
                               ms->segments[k]->length = 0;
                             to_j = 0;
                           }
@@ -989,9 +974,8 @@ strip_indent (mixed_string_ty *ms)
                            characters from the current line.  */
                         if (start_of_curr_line_i < i)
                           {
-                            size_t k;
                             ms->segments[start_of_curr_line_i]->length = start_of_curr_line_j;
-                            for (k = start_of_curr_line_i + 1; k < i; k++)
+                            for (size_t k = start_of_curr_line_i + 1; k < i; k++)
                               ms->segments[k]->length = 0;
                             to_j = 0;
                           }
@@ -1018,9 +1002,8 @@ strip_indent (mixed_string_ty *ms)
                    whitespace characters.  Remove all this whitespace.  */
                 if (start_of_curr_line_i < i)
                   {
-                    size_t k;
                     ms->segments[start_of_curr_line_i]->length = start_of_curr_line_j;
-                    for (k = start_of_curr_line_i + 1; k < i; k++)
+                    for (size_t k = start_of_curr_line_i + 1; k < i; k++)
                       ms->segments[k]->length = 0;
                     to_j = 0;
                   }
@@ -1033,9 +1016,8 @@ strip_indent (mixed_string_ty *ms)
                    current line.  */
                 if (start_of_trailing_whitespace_i < i)
                   {
-                    size_t k;
                     ms->segments[start_of_trailing_whitespace_i]->length = start_of_trailing_whitespace_j;
-                    for (k = start_of_trailing_whitespace_i + 1; k < i; k++)
+                    for (size_t k = start_of_trailing_whitespace_i + 1; k < i; k++)
                       ms->segments[k]->length = 0;
                     to_j = 0;
                   }
@@ -1057,8 +1039,6 @@ static int phase5_pushback_length;
 static void
 phase5_get (token_ty *tp)
 {
-  int c;
-
   if (phase5_pushback_length)
     {
       *tp = phase5_pushback[--phase5_pushback_length];
@@ -1068,6 +1048,8 @@ phase5_get (token_ty *tp)
 
   for (;;)
     {
+      int c;
+
       tp->line_number = line_number;
       c = phase4_getc ();
 
@@ -1203,9 +1185,6 @@ phase5_get (token_ty *tp)
                   {
                     /* Text block.  Specification:
                        <https://docs.oracle.com/javase/specs/jls/se13/preview/text-blocks.html>  */
-                    struct mixed_string_buffer block;
-                    unsigned int consecutive_unescaped_doublequotes;
-                    mixed_string_ty *block_content;
 
                     /* Parse the part up to and including the first newline.  */
                     for (;;)
@@ -1234,9 +1213,10 @@ phase5_get (token_ty *tp)
                       }
 
                     /* Parse the part after the first newline.  */
+                    struct mixed_string_buffer block;
                     mixed_string_buffer_init (&block, lc_string,
                                               logical_file_name, line_number);
-                    consecutive_unescaped_doublequotes = 0;
+                    unsigned int consecutive_unescaped_doublequotes = 0;
                     for (;;)
                       {
                         int ic = phase3_getc ();
@@ -1265,7 +1245,7 @@ phase5_get (token_ty *tp)
                             mixed_string_buffer_append (&block, ic);
                           }
                       }
-                    block_content = mixed_string_buffer_result (&block);
+                    mixed_string_ty *block_content = mixed_string_buffer_result (&block);
 
                     /* Remove the common indentation from the content.  */
                     strip_indent (block_content);
@@ -1282,7 +1262,6 @@ phase5_get (token_ty *tp)
           /* String literal.  */
           {
             struct mixed_string_buffer literal;
-
             mixed_string_buffer_init (&literal, lc_string,
                                       logical_file_name, line_number);
             accumulate_escaped (&literal, '"');
@@ -1296,7 +1275,6 @@ phase5_get (token_ty *tp)
           /* Character literal.  */
           {
             struct mixed_string_buffer literal;
-
             mixed_string_buffer_init (&literal, lc_outside,
                                       logical_file_name, line_number);
             accumulate_escaped (&literal, '\'');
@@ -1586,18 +1564,18 @@ phase6_get (token_ty *tp)
       for (;;)
         {
           token_ty token2;
-
           phase5_get (&token2);
+
           if (token2.type == token_type_plus)
             {
               token_ty token3;
-
               phase5_get (&token3);
+
               if (token3.type == token_type_string_literal)
                 {
                   token_ty token_after;
-
                   phase5_get (&token_after);
+
                   if (token_after.type != token_type_dot)
                     {
                       sum = mixed_string_concat_free1 (sum, token3.mixed_string);
@@ -1707,8 +1685,8 @@ extract_parenthesized (message_list_ty *mlp, token_type_ty terminator,
   for (;;)
     {
       token_ty token;
-
       x_java_lex (&token);
+
       switch (token.type)
         {
         case token_type_symbol:
@@ -1720,19 +1698,17 @@ extract_parenthesized (message_list_ty *mlp, token_type_ty terminator,
                symbolJ.....symbolN with J > I.  */
             char *sum = token.string;
             size_t sum_len = strlen (sum);
-            const char *dottedname;
-            flag_context_list_ty *context_list;
 
             for (;;)
               {
                 token_ty token2;
-
                 x_java_lex (&token2);
+
                 if (token2.type == token_type_dot)
                   {
                     token_ty token3;
-
                     x_java_lex (&token3);
+
                     if (token3.type == token_type_symbol)
                       {
                         char *addend = token3.string;
@@ -1763,10 +1739,9 @@ extract_parenthesized (message_list_ty *mlp, token_type_ty terminator,
                they should not.  */
             if (strcmp (sum, "return") != 0)
               {
-                for (dottedname = sum;;)
+                for (const char *dottedname = sum;;)
                   {
                     void *keyword_value;
-
                     if (hash_find_entry (&keywords, dottedname, strlen (dottedname),
                                          &keyword_value)
                         == 0)
@@ -1785,7 +1760,8 @@ extract_parenthesized (message_list_ty *mlp, token_type_ty terminator,
                     dottedname++;
                   }
 
-                for (dottedname = sum;;)
+                flag_context_list_ty *context_list;
+                for (const char *dottedname = sum;;)
                   {
                     context_list =
                       flag_context_list_table_lookup (
@@ -1803,7 +1779,7 @@ extract_parenthesized (message_list_ty *mlp, token_type_ty terminator,
               }
 
             free (sum);
-            continue;
+            break;
           }
 
         case token_type_lparen:
@@ -1843,7 +1819,7 @@ extract_parenthesized (message_list_ty *mlp, token_type_ty terminator,
           }
           next_context_iter = null_context_list_iterator;
           state = 0;
-          continue;
+          break;
 
         case token_type_rparen:
           if (terminator == token_type_rparen)
@@ -1858,7 +1834,7 @@ extract_parenthesized (message_list_ty *mlp, token_type_ty terminator,
                       _("')' found where '}' was expected"));
           next_context_iter = null_context_list_iterator;
           state = 0;
-          continue;
+          break;
 
         case token_type_lbrace:
           if (++brace_nesting_depth > MAX_NESTING_DEPTH)
@@ -1877,7 +1853,7 @@ extract_parenthesized (message_list_ty *mlp, token_type_ty terminator,
           brace_nesting_depth--;
           next_context_iter = null_context_list_iterator;
           state = 0;
-          continue;
+          break;
 
         case token_type_rbrace:
           if (terminator == token_type_rbrace)
@@ -1892,7 +1868,7 @@ extract_parenthesized (message_list_ty *mlp, token_type_ty terminator,
                       _("'}' found where ')' was expected"));
           next_context_iter = null_context_list_iterator;
           state = 0;
-          continue;
+          break;
 
         case token_type_comma:
           arg++;
@@ -1901,7 +1877,7 @@ extract_parenthesized (message_list_ty *mlp, token_type_ty terminator,
           inner_region = new_sub_region (outer_region, curr_context);
           next_context_iter = passthrough_context_list_iterator;
           state = 0;
-          continue;
+          break;
 
         case token_type_conditional:
           /* In an expression A ? B : C, each of A, B, C is a distinct
@@ -1914,7 +1890,7 @@ extract_parenthesized (message_list_ty *mlp, token_type_ty terminator,
           inner_region = new_sub_region (outer_region, curr_context);
           next_context_iter = passthrough_context_list_iterator;
           state = 0;
-          continue;
+          break;
 
         case token_type_colon:
           /* In an expression A ? B : C, each of A, B, C is a distinct
@@ -1923,7 +1899,7 @@ extract_parenthesized (message_list_ty *mlp, token_type_ty terminator,
           inner_region = new_sub_region (outer_region, curr_context);
           next_context_iter = passthrough_context_list_iterator;
           state = 0;
-          continue;
+          break;
 
         case token_type_assign:
           /* In an expression A = B, A and B are distinct sub-regions.
@@ -1933,7 +1909,7 @@ extract_parenthesized (message_list_ty *mlp, token_type_ty terminator,
           inner_region = new_sub_region (outer_region, curr_context);
           next_context_iter = passthrough_context_list_iterator;
           state = 0;
-          continue;
+          break;
 
         case token_type_plus:
         case token_type_operator:
@@ -1949,12 +1925,11 @@ extract_parenthesized (message_list_ty *mlp, token_type_ty terminator,
           inner_region->inherit_from_parent_region = false;
           next_context_iter = passthrough_context_list_iterator;
           state = 0;
-          continue;
+          break;
 
         case token_type_string_literal:
           {
             lex_pos_ty pos;
-
             pos.file_name = logical_file_name;
             pos.line_number = token.line_number;
 
@@ -1975,12 +1950,21 @@ extract_parenthesized (message_list_ty *mlp, token_type_ty terminator,
           drop_reference (token.comment);
           next_context_iter = null_context_list_iterator;
           state = 0;
-          continue;
+          break;
 
         case token_type_semicolon:
           arglist_parser_done (argparser, arg);
           unref_region (inner_region);
-          return false;
+          if (terminator == token_type_rbrace)
+            {
+              argparser = arglist_parser_alloc (mlp, NULL);
+              inner_region = new_sub_region (outer_region, curr_context);
+              next_context_iter = null_context_list_iterator;
+              state = 0;
+              break;
+            }
+          else
+            return false;
 
         case token_type_eof:
           arglist_parser_done (argparser, arg);
@@ -1992,7 +1976,7 @@ extract_parenthesized (message_list_ty *mlp, token_type_ty terminator,
         case token_type_other:
           next_context_iter = null_context_list_iterator;
           state = 0;
-          continue;
+          break;
 
         default:
           abort ();
