@@ -1,7 +1,5 @@
 /* xgettext PHP backend.
-   Copyright (C) 2001-2025 Free Software Foundation, Inc.
-
-   This file was written by Bruno Haible <bruno@clisp.org>, 2002.
+   Copyright (C) 2001-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,9 +14,9 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif
+/* Written by Bruno Haible.  */
+
+#include <config.h>
 
 /* Specification.  */
 #include "x-php.h"
@@ -84,18 +82,16 @@ x_php_keyword (const char *name)
     default_keywords = false;
   else
     {
-      const char *end;
-      struct callshape shape;
-      const char *colon;
-
       if (keywords.table == NULL)
         hash_init (&keywords, 100);
 
+      const char *end;
+      struct callshape shape;
       split_keywordspec (name, &end, &shape);
 
       /* The characters between name and end should form a valid C identifier.
          A colon means an invalid parse in split_keywordspec().  */
-      colon = strchr (name, ':');
+      const char *colon = strchr (name, ':');
       if (colon == NULL || colon >= end)
         insert_keyword_callshape (&keywords, name, end - name, &shape);
     }
@@ -155,6 +151,8 @@ enum token_type_ty
   token_type_dot,               /* . */
   token_type_operator1,         /* * / % ++ -- */
   token_type_operator2,         /* + - ! ~ @ */
+  token_type_lbrace,            /* { */
+  token_type_rbrace,            /* } */
   token_type_string_literal,    /* "abc" */
   token_type_symbol,            /* symbol, number */
   token_type_other              /* misc. operator */
@@ -215,6 +213,7 @@ struct php_extractor
   /* Current nesting depths.  */
   int paren_nesting_depth;
   int bracket_nesting_depth;
+  int brace_nesting_depth;
 };
 
 static inline void
@@ -239,10 +238,15 @@ php_extractor_init_rest (struct php_extractor *xp)
 
   xp->paren_nesting_depth = 0;
   xp->bracket_nesting_depth = 0;
+  xp->brace_nesting_depth = 0;
 }
 
 /* Forward declarations.  */
-static void extract_php_input (struct php_extractor *xp);
+static bool extract_balanced (struct php_extractor *xp,
+                              token_type_ty delim,
+                              flag_region_ty *outer_region,
+                              flag_context_list_iterator_ty context_iter,
+                              struct arglist_parser *argparser);
 
 
 /* ======================== Reading of characters.  ======================== */
@@ -529,11 +533,11 @@ skip_html (struct php_extractor *xp)
 static int
 phase2_getc (struct php_extractor *xp)
 {
-  int c;
-
   if (xp->phase2_pushback_length)
     return xp->phase2_pushback[--(xp->phase2_pushback_length)];
 
+  int c;
+
   c = phase1_getc (xp);
   switch (c)
     {
@@ -661,7 +665,6 @@ comment_line_end (struct php_extractor *xp, size_t chars_to_remove)
 static int
 phase3_getc (struct php_extractor *xp)
 {
-  int lineno;
   int c;
 
   if (xp->phase3_pushback_length)
@@ -672,10 +675,9 @@ phase3_getc (struct php_extractor *xp)
   if (c == '#')
     {
       /* sh comment.  */
-      bool last_was_qmark = false;
-
       comment_start (xp);
-      lineno = xp->line_number;
+      int lineno = xp->line_number;
+      bool last_was_qmark = false;
       for (;;)
         {
           c = phase1_getc (xp);
@@ -711,11 +713,9 @@ phase3_getc (struct php_extractor *xp)
         case '*':
           {
             /* C comment.  */
-            bool last_was_star;
-
             comment_start (xp);
-            lineno = xp->line_number;
-            last_was_star = false;
+            int lineno = xp->line_number;
+            bool last_was_star = false;
             for (;;)
               {
                 c = phase1_getc (xp);
@@ -759,10 +759,9 @@ phase3_getc (struct php_extractor *xp)
         case '/':
           {
             /* C++ comment.  */
-            bool last_was_qmark = false;
-
             comment_start (xp);
-            lineno = xp->line_number;
+            int lineno = xp->line_number;
+            bool last_was_qmark = false;
             for (;;)
               {
                 c = phase1_getc (xp);
@@ -791,8 +790,7 @@ phase3_getc (struct php_extractor *xp)
     return c;
 }
 
-#ifdef unused
-static void
+MAYBE_UNUSED static void
 phase3_ungetc (struct php_extractor *xp, int c)
 {
   if (c != EOF)
@@ -802,7 +800,6 @@ phase3_ungetc (struct php_extractor *xp, int c)
       xp->phase3_pushback[xp->phase3_pushback_length++] = c;
     }
 }
-#endif
 
 
 /* ========================== Reading of tokens.  ========================== */
@@ -881,8 +878,6 @@ process_dquote_or_heredoc (struct php_extractor *xp, bool heredoc)
           }
         if (c == '\\')
           {
-            int n, j;
-
             c = phase1_getc (xp);
             switch (c)
               {
@@ -892,60 +887,65 @@ process_dquote_or_heredoc (struct php_extractor *xp, bool heredoc)
 
               case '0': case '1': case '2': case '3':
               case '4': case '5': case '6': case '7':
-                n = 0;
-                for (j = 0; j < 3; ++j)
-                  {
-                    n = n * 8 + c - '0';
-                    c = phase1_getc (xp);
-                    switch (c)
-                      {
-                      default:
-                        break;
+                {
+                  int n = 0;
+                  for (int j = 0; j < 3; ++j)
+                    {
+                      n = n * 8 + c - '0';
+                      c = phase1_getc (xp);
+                      switch (c)
+                        {
+                        default:
+                          break;
 
-                      case '0': case '1': case '2': case '3':
-                      case '4': case '5': case '6': case '7':
-                        continue;
-                      }
-                    break;
-                  }
-                phase1_ungetc (xp, c);
-                c = n;
+                        case '0': case '1': case '2': case '3':
+                        case '4': case '5': case '6': case '7':
+                          continue;
+                        }
+                      break;
+                    }
+                  phase1_ungetc (xp, c);
+                  c = n;
+                }
                 break;
 
               case 'x':
-                n = 0;
-                for (j = 0; j < 2; ++j)
-                  {
-                    c = phase1_getc (xp);
-                    switch (c)
-                      {
-                      case '0': case '1': case '2': case '3': case '4':
-                      case '5': case '6': case '7': case '8': case '9':
-                        n = n * 16 + c - '0';
+                {
+                  int j;
+                  int n = 0;
+                  for (j = 0; j < 2; ++j)
+                    {
+                      c = phase1_getc (xp);
+                      switch (c)
+                        {
+                        case '0': case '1': case '2': case '3': case '4':
+                        case '5': case '6': case '7': case '8': case '9':
+                          n = n * 16 + c - '0';
+                          break;
+                        case 'A': case 'B': case 'C': case 'D': case 'E':
+                        case 'F':
+                          n = n * 16 + 10 + c - 'A';
+                          break;
+                        case 'a': case 'b': case 'c': case 'd': case 'e':
+                        case 'f':
+                          n = n * 16 + 10 + c - 'a';
+                          break;
+                        default:
+                          phase1_ungetc (xp, c);
+                          c = 0;
+                          break;
+                        }
+                      if (c == 0)
                         break;
-                      case 'A': case 'B': case 'C': case 'D': case 'E':
-                      case 'F':
-                        n = n * 16 + 10 + c - 'A';
-                        break;
-                      case 'a': case 'b': case 'c': case 'd': case 'e':
-                      case 'f':
-                        n = n * 16 + 10 + c - 'a';
-                        break;
-                      default:
-                        phase1_ungetc (xp, c);
-                        c = 0;
-                        break;
-                      }
-                    if (c == 0)
-                      break;
-                  }
-                if (j == 0)
-                  {
-                    phase1_ungetc (xp, 'x');
-                    c = '\\';
-                  }
-                else
-                  c = n;
+                    }
+                  if (j == 0)
+                    {
+                      phase1_ungetc (xp, 'x');
+                      c = '\\';
+                    }
+                  else
+                    c = n;
+                }
                 break;
 
               case 'n':
@@ -981,89 +981,22 @@ process_dquote_or_heredoc (struct php_extractor *xp, bool heredoc)
 
  string_with_embedded_expressions:
   is_constant = false;
-  {
-    size_t nesting_stack_alloc = 10;
-    char *nesting_stack = xmalloc (nesting_stack_alloc);
-    size_t nesting_stack_depth = 0;
-    /* We just read a '{', so expect a matching '}'.  */
-    nesting_stack[nesting_stack_depth++] = '}';
-
-    /* Find the extent of the expression.  */
-    struct string_buffer buffer;
-    sb_init (&buffer);
-    for (;;)
-      {
-        int c;
-
-        c = phase1_getc (xp);
-        if (!heredoc && c == EOF)
-          break;
-        if (c == (heredoc ? EOF : '"'))
-          {
-            if (nesting_stack_depth > 0)
-              if_error (IF_SEVERITY_WARNING,
-                        logical_file_name, xp->line_number, (size_t)(-1), false,
-                        heredoc
-                        ? _("unterminated expression in heredoc, expected a '%c'")
-                        : _("unterminated expression in string literal, expected a '%c'"),
-                        nesting_stack[nesting_stack_depth - 1]);
-            break;
-          }
-        if (heredoc && c == '\n')
-          xp->line_number++;
-        if (c == '{' || c == '[' || c == '(')
-          {
-            if (nesting_stack_depth >= nesting_stack_alloc)
-              {
-                nesting_stack_alloc = 2 * nesting_stack_alloc;
-                nesting_stack =
-                  xrealloc (nesting_stack, nesting_stack_alloc);
-              }
-            nesting_stack[nesting_stack_depth++] =
-              (c == '{' ? '}' : c == '[' ? ']' : ')');
-          }
-        else if (c == '}' || c == ']' || c == ')')
-          {
-            if (nesting_stack_depth > 0
-                && c == nesting_stack[nesting_stack_depth - 1])
-              {
-                if (--nesting_stack_depth == 0)
-                  break;
-              }
-            else
-              if_error (IF_SEVERITY_WARNING,
-                        logical_file_name, xp->line_number, (size_t)(-1), false,
-                        heredoc
-                        ? _("unterminated expression in heredoc contains unbalanced '%c'")
-                        : _("unterminated expression in string literal contains unbalanced '%c'"),
-                        c);
-          }
-        sb_xappend1 (&buffer, c);
-      }
-
-    /* Recursively extract messages from the expression.  */
-    string_desc_t substring = sb_contents (&buffer);
-
-    struct php_extractor *rxp = XMALLOC (struct php_extractor);
-    rxp->mlp = xp->mlp;
-    sf_istream_init_from_string_desc (&rxp->input, substring);
-    rxp->line_number = xp->line_number;
-    php_extractor_init_rest (rxp);
-
-    extract_php_input (rxp);
-
-    free (rxp);
-    sb_free (&buffer);
-    free (nesting_stack);
-  }
+  if (++(xp->brace_nesting_depth) > MAX_NESTING_DEPTH)
+    if_error (IF_SEVERITY_FATAL_ERROR,
+              logical_file_name, xp->line_number, (size_t)(-1), false,
+              _("too many open braces"));
+  if (extract_balanced (xp, token_type_rbrace,
+                        null_context_region (),
+                        null_context_list_iterator,
+                        arglist_parser_alloc (xp->mlp, NULL)))
+    return NULL;
+  xp->brace_nesting_depth--;
   goto string_continued;
 }
 
 static void
 phase4_get (struct php_extractor *xp, token_ty *tp)
 {
-  int c;
-
   if (xp->phase4_pushback_length)
     {
       *tp = xp->phase4_pushback[--(xp->phase4_pushback_length)];
@@ -1073,6 +1006,8 @@ phase4_get (struct php_extractor *xp, token_ty *tp)
 
   for (;;)
     {
+      int c;
+
       tp->line_number = xp->line_number;
       c = phase3_getc (xp);
       switch (c)
@@ -1291,6 +1226,14 @@ phase4_get (struct php_extractor *xp, token_ty *tp)
           tp->type = token_type_operator2;
           return;
 
+        case '{':
+          tp->type = token_type_lbrace;
+          return;
+
+        case '}':
+          tp->type = token_type_rbrace;
+          return;
+
         case '<':
           {
             int c2 = phase1_getc (xp);
@@ -1452,11 +1395,10 @@ phase4_get (struct php_extractor *xp, token_ty *tp)
                     if (end_label_indent > 0)
                       {
                         /* Scan through the doc string, copying *q = *p.  */
-                        const char *p;
                         char *q = doc;
                         int curr_line_indent = 0;
 
-                        for (p = doc; p < doc + doc_len; p++)
+                        for (const char *p = doc; p < doc + doc_len; p++)
                           {
                             /* Invariant: doc <= q <= p <= doc + doc_len.  */
                             char d = *p;
@@ -1644,18 +1586,18 @@ x_php_lex (struct php_extractor *xp, token_ty *tp)
       for (;;)
         {
           token_ty token2;
-
           phase4_get (xp, &token2);
+
           if (token2.type == token_type_dot)
             {
               token_ty token3;
-
               phase4_get (xp, &token3);
+
               if (token3.type == token_type_string_literal)
                 {
                   token_ty token_after;
-
                   phase4_get (xp, &token_after);
+
                   if (token_after.type != token_type_operator1)
                     {
                       char *addend = token3.string;
@@ -1707,8 +1649,8 @@ static flag_context_list_table_ty *flag_context_list_table;
 
 /* Extract messages until the next balanced closing parenthesis or bracket.
    Extracted messages are added to XP->MLP.
-   DELIM can be either token_type_rparen or token_type_rbracket, or
-   token_type_eof to accept both.
+   DELIM can be either token_type_rparen or token_type_rbracket or
+   token_type_rbrace, or token_type_eof to accept any of them.
    Return true upon eof, false upon closing parenthesis or bracket.  */
 static bool
 extract_balanced (struct php_extractor *xp,
@@ -1737,14 +1679,13 @@ extract_balanced (struct php_extractor *xp,
   for (;;)
     {
       token_ty token;
-
       x_php_lex (xp, &token);
+
       switch (token.type)
         {
         case token_type_symbol:
           {
             void *keyword_value;
-
             if (hash_find_entry (&keywords, token.string, strlen (token.string),
                                  &keyword_value)
                 == 0)
@@ -1761,7 +1702,7 @@ extract_balanced (struct php_extractor *xp,
                 flag_context_list_table,
                 token.string, strlen (token.string)));
           free (token.string);
-          continue;
+          break;
 
         case token_type_lparen:
           if (++(xp->paren_nesting_depth) > MAX_NESTING_DEPTH)
@@ -1780,7 +1721,7 @@ extract_balanced (struct php_extractor *xp,
           xp->paren_nesting_depth--;
           next_context_iter = null_context_list_iterator;
           state = 0;
-          continue;
+          break;
 
         case token_type_rparen:
           if (delim == token_type_rparen || delim == token_type_eof)
@@ -1791,7 +1732,7 @@ extract_balanced (struct php_extractor *xp,
             }
           next_context_iter = null_context_list_iterator;
           state = 0;
-          continue;
+          break;
 
         case token_type_comma:
           arg++;
@@ -1802,7 +1743,7 @@ extract_balanced (struct php_extractor *xp,
                                  &context_iter));
           next_context_iter = passthrough_context_list_iterator;
           state = 0;
-          continue;
+          break;
 
         case token_type_lbracket:
           if (++(xp->bracket_nesting_depth) > MAX_NESTING_DEPTH)
@@ -1821,7 +1762,7 @@ extract_balanced (struct php_extractor *xp,
           xp->bracket_nesting_depth--;
           next_context_iter = null_context_list_iterator;
           state = 0;
-          continue;
+          break;
 
         case token_type_rbracket:
           if (delim == token_type_rbracket || delim == token_type_eof)
@@ -1832,7 +1773,37 @@ extract_balanced (struct php_extractor *xp,
             }
           next_context_iter = null_context_list_iterator;
           state = 0;
-          continue;
+          break;
+
+        case token_type_lbrace:
+          if (++(xp->brace_nesting_depth) > MAX_NESTING_DEPTH)
+            if_error (IF_SEVERITY_FATAL_ERROR,
+                      logical_file_name, xp->line_number, (size_t)(-1), false,
+                      _("too many open braces"));
+          if (extract_balanced (xp, token_type_rbrace,
+                                null_context_region (),
+                                null_context_list_iterator,
+                                arglist_parser_alloc (xp->mlp, NULL)))
+            {
+              arglist_parser_done (argparser, arg);
+              unref_region (inner_region);
+              return true;
+            }
+          xp->brace_nesting_depth--;
+          next_context_iter = null_context_list_iterator;
+          state = 0;
+          break;
+
+        case token_type_rbrace:
+          if (delim == token_type_rbrace || delim == token_type_eof)
+            {
+              arglist_parser_done (argparser, arg);
+              unref_region (inner_region);
+              return false;
+            }
+          next_context_iter = null_context_list_iterator;
+          state = 0;
+          break;
 
         case token_type_string_literal:
           {
@@ -1858,7 +1829,7 @@ extract_balanced (struct php_extractor *xp,
           }
           next_context_iter = null_context_list_iterator;
           state = 0;
-          continue;
+          break;
 
         case token_type_dot:
         case token_type_operator1:
@@ -1866,7 +1837,7 @@ extract_balanced (struct php_extractor *xp,
         case token_type_other:
           next_context_iter = null_context_list_iterator;
           state = 0;
-          continue;
+          break;
 
         case token_type_eof:
           arglist_parser_done (argparser, arg);
@@ -1880,18 +1851,6 @@ extract_balanced (struct php_extractor *xp,
 }
 
 
-static void
-extract_php_input (struct php_extractor *xp)
-{
-  /* Eat tokens until eof is seen.  When extract_balanced returns
-     due to an unbalanced closing parenthesis, just restart it.  */
-  while (!extract_balanced (xp, token_type_eof,
-                            null_context_region (), null_context_list_iterator,
-                            arglist_parser_alloc (xp->mlp, NULL)))
-    ;
-}
-
-
 void
 extract_php (FILE *f,
              const char *real_filename, const char *logical_filename,
@@ -1914,7 +1873,12 @@ extract_php (FILE *f,
   /* Initial mode is HTML mode, not PHP mode.  */
   skip_html (xp);
 
-  extract_php_input (xp);
+  /* Eat tokens until eof is seen.  When extract_balanced returns
+     due to an unbalanced closing parenthesis, just restart it.  */
+  while (!extract_balanced (xp, token_type_eof,
+                            null_context_region (), null_context_list_iterator,
+                            arglist_parser_alloc (xp->mlp, NULL)))
+    ;
 
   /* Close scanner.  */
   free (xp);
