@@ -1,5 +1,5 @@
 /* xgettext Rust backend.
-   Copyright (C) 2001-2025 Free Software Foundation, Inc.
+   Copyright (C) 2001-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -14,11 +14,9 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
-/* Written by Bruno Haible <bruno@clisp.org>, 2025.  */
+/* Written by Bruno Haible.  */
 
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif
+#include <config.h>
 
 /* Specification.  */
 #include "x-rust.h"
@@ -89,21 +87,19 @@ x_rust_keyword (const char *name)
     default_keywords = false;
   else
     {
-      const char *end;
-      struct callshape shape;
-      const char *colon;
-
       if (function_keywords.table == NULL)
         hash_init (&function_keywords, 100);
       if (macro_keywords.table == NULL)
         hash_init (&macro_keywords, 100);
 
+      const char *end;
+      struct callshape shape;
       split_keywordspec (name, &end, &shape);
 
       /* The characters between name and end should form a valid Rust
          identifier, possibly with a trailing '!'.
          A colon means an invalid parse in split_keywordspec().  */
-      colon = strchr (name, ':');
+      const char *colon = strchr (name, ':');
       if (colon == NULL || colon >= end)
         {
           if (end > name && end[-1] == '!')
@@ -214,6 +210,7 @@ static TSSymbol ts_symbol_raw_string_literal;
 static TSSymbol ts_symbol_string_content;
 static TSSymbol ts_symbol_escape_sequence;
 static TSSymbol ts_symbol_identifier;
+static TSSymbol ts_symbol_scoped_identifier;
 static TSSymbol ts_symbol_call_expression;
 static TSSymbol ts_symbol_macro_invocation;
 static TSSymbol ts_symbol_arguments;
@@ -352,8 +349,7 @@ string_literal_value (TSNode node)
   sb_init (&buffer);
   uint32_t count = ts_node_named_child_count (node);
   bool skip_leading_whitespace = false;
-  uint32_t i;
-  for (i = 0; i < count; i++)
+  for (uint32_t i = 0; i < count; i++)
     {
       TSNode subnode = ts_node_named_child (node, i);
       if (ts_node_symbol (subnode) == ts_symbol_string_content)
@@ -413,8 +409,7 @@ string_literal_value (TSNode node)
           else if (escape_start[1] == 'x')
             {
               unsigned int value = 0;
-              const char *p;
-              for (p = escape_start + 2; p < escape_end; p++)
+              for (const char *p = escape_start + 2; p < escape_end; p++)
                 {
                   /* Only 2 hexadecimal digits are accepted.
                      No overflow is possible.  */
@@ -443,8 +438,7 @@ string_literal_value (TSNode node)
                    && escape_start[2] == '{' && escape_end[-1] == '}')
             {
               unsigned int value = 0;
-              const char *p;
-              for (p = escape_start + 3; p < escape_end - 1; p++)
+              for (const char *p = escape_start + 3; p < escape_end - 1; p++)
                 {
                   char c = *p;
                   if (c >= '0' && c <= '9')
@@ -517,7 +511,8 @@ static void extract_from_node (TSNode node,
                                message_list_ty *mlp);
 
 /* Extracts messages from the function call consisting of
-     - CALLEE_NODE: a tree node of type 'identifier',
+     - CALLEE_NODE: a tree node of type 'identifier' or
+       a tree node of type 'scoped_identifier' with at least 1 named child,
      - ARGS_NODE: a tree node of type 'arguments'.
    Extracted messages are added to MLP.  */
 static void
@@ -528,9 +523,17 @@ extract_from_function_call (TSNode callee_node,
 {
   uint32_t args_count = ts_node_child_count (args_node);
 
+  TSNode callee_last_identifier_node;
+  if (ts_node_symbol (callee_node) == ts_symbol_identifier)
+    callee_last_identifier_node = callee_node;
+  else
+    /* Here ts_node_named_child_count (callee_node) >= 1.  */
+    callee_last_identifier_node =
+      ts_node_named_child (callee_node, ts_node_named_child_count (callee_node) - 1);
+
   string_desc_t callee_name =
-    sd_new_addr (ts_node_end_byte (callee_node) - ts_node_start_byte (callee_node),
-                 contents + ts_node_start_byte (callee_node));
+    sd_new_addr (ts_node_end_byte (callee_last_identifier_node) - ts_node_start_byte (callee_last_identifier_node),
+                 contents + ts_node_start_byte (callee_last_identifier_node));
 
   /* Context iterator.  */
   flag_context_list_iterator_ty next_context_iter =
@@ -556,10 +559,9 @@ extract_from_function_call (TSNode callee_node,
 
       /* Current argument number.  */
       uint32_t arg;
-      uint32_t i;
 
       arg = 0;
-      for (i = 0; i < args_count; i++)
+      for (uint32_t i = 0; i < args_count; i++)
         {
           TSNode arg_node = ts_node_child (args_node, i);
           handle_comments (arg_node);
@@ -625,10 +627,9 @@ extract_from_function_call (TSNode callee_node,
 
   /* Current argument number.  */
   uint32_t arg;
-  uint32_t i;
 
   arg = 0;
-  for (i = 0; i < args_count; i++)
+  for (uint32_t i = 0; i < args_count; i++)
     {
       TSNode arg_node = ts_node_child (args_node, i);
       handle_comments (arg_node);
@@ -676,8 +677,8 @@ extract_from_function_call_like (TSNode *callee_node, bool callee_is_macro,
 
   /* Context iterator.  */
   flag_context_list_iterator_ty next_context_iter;
-  void *keyword_value;
 
+  void *keyword_value;
   if (callee_node != NULL)
     {
       string_desc_t callee_name =
@@ -717,8 +718,7 @@ extract_from_function_call_like (TSNode *callee_node, bool callee_is_macro,
       #if DEBUG_RUST
       {
         fprintf (stderr, "children:\n");
-        uint32_t i;
-        for (i = 0; i < args_count; i++)
+        for (uint32_t i = 0; i < args_count; i++)
           fprintf (stderr, "%u -> [%s]|%s|\n", i, ts_node_type (ts_node_child (args_node, i)), ts_node_string (ts_node_child (args_node, i)));
       }
       #endif
@@ -734,12 +734,11 @@ extract_from_function_call_like (TSNode *callee_node, bool callee_is_macro,
           /* Current argument number.  */
           uint32_t arg;
           flag_region_ty *arg_region;
-          uint32_t i;
           uint32_t prev2_token_in_same_arg;
           uint32_t prev1_token_in_same_arg;
 
           arg = 0;
-          for (i = 0; i < args_count; i++)
+          for (uint32_t i = 0; i < args_count; i++)
             {
               TSNode arg_node = ts_node_child (args_node, i);
               handle_comments (arg_node);
@@ -853,12 +852,11 @@ extract_from_function_call_like (TSNode *callee_node, bool callee_is_macro,
   /* Current argument number.  */
   uint32_t arg;
   flag_region_ty *arg_region;
-  uint32_t i;
   uint32_t prev2_token_in_same_arg;
   uint32_t prev1_token_in_same_arg;
 
   arg = 0;
-  for (i = 0; i < args_count; i++)
+  for (uint32_t i = 0; i < args_count; i++)
     {
       TSNode arg_node = ts_node_child (args_node, i);
       handle_comments (arg_node);
@@ -962,7 +960,9 @@ extract_from_node (TSNode node,
       if (! ts_node_eq (ts_node_child_by_field_id (node, ts_field_function),
                         callee_node))
         abort ();
-      if (ts_node_symbol (callee_node) == ts_symbol_identifier)
+      if (ts_node_symbol (callee_node) == ts_symbol_identifier
+          || (ts_node_symbol (callee_node) == ts_symbol_scoped_identifier
+              && ts_node_named_child_count (callee_node) >= 1))
         {
           TSNode args_node = ts_node_child_by_field_id (node, ts_field_arguments);
           /* This is the field called 'arguments'.  */
@@ -971,8 +971,7 @@ extract_from_node (TSNode node,
               /* Handle the potential comments between 'function' and 'arguments'.  */
               {
                 uint32_t count = ts_node_child_count (node);
-                uint32_t i;
-                for (i = 0; i < count; i++)
+                for (uint32_t i = 0; i < count; i++)
                   {
                     TSNode subnode = ts_node_child (node, i);
                     if (ts_node_eq (subnode, args_node))
@@ -1002,23 +1001,19 @@ extract_from_node (TSNode node,
              It is not always = ts_node_named_child (node, 1),
              namely when there are comments before it.  */
           uint32_t count = ts_node_child_count (node);
-          uint32_t args_index;
-          for (args_index = 0; args_index < count; args_index++)
+          for (uint32_t args_index = 0; args_index < count; args_index++)
             {
               TSNode args_node = ts_node_child (node, args_index);
               if (ts_node_symbol (args_node) == ts_symbol_token_tree)
                 {
                   /* Handle the potential comments between 'macro' and the args_node.  */
-                  {
-                    uint32_t i;
-                    for (i = 0; i < count; i++)
-                      {
-                        TSNode subnode = ts_node_child (node, i);
-                        if (ts_node_eq (subnode, args_node))
-                          break;
-                        handle_comments (subnode);
-                      }
-                  }
+                  for (uint32_t i = 0; i < count; i++)
+                    {
+                      TSNode subnode = ts_node_child (node, i);
+                      if (ts_node_eq (subnode, args_node))
+                        break;
+                      handle_comments (subnode);
+                    }
                   extract_from_function_call_like (&callee_node, true,
                                                    args_node,
                                                    outer_region,
@@ -1045,8 +1040,7 @@ extract_from_node (TSNode node,
               fprintf (stderr, "gettext arguments: %s\n", ts_node_string (argsnode));
               fprintf (stderr, "gettext children:\n");
               uint32_t count = ts_node_named_child_count (node);
-              uint32_t i;
-              for (i = 0; i < count; i++)
+              for (uint32_t i = 0; i < count; i++)
                 fprintf (stderr, "%u -> %s\n", i, ts_node_string (ts_node_named_child (node, i)));
             }
         }
@@ -1064,8 +1058,7 @@ extract_from_node (TSNode node,
             {
               fprintf (stderr, "children:\n");
               uint32_t count = ts_node_child_count (node);
-              uint32_t i;
-              for (i = 0; i < count; i++)
+              for (uint32_t i = 0; i < count; i++)
                 fprintf (stderr, "%u -> [%s]|%s|\n", i, ts_node_type (ts_node_child (node, i)), ts_node_string (ts_node_child (node, i)));
             }
         }
@@ -1077,8 +1070,7 @@ extract_from_node (TSNode node,
         || ts_node_symbol (node) == ts_symbol_block_comment))
     {
       uint32_t count = ts_node_child_count (node);
-      uint32_t i;
-      for (i = 0; i < count; i++)
+      for (uint32_t i = 0; i < count; i++)
         {
           TSNode subnode = ts_node_child (node, i);
           handle_comments (subnode);
@@ -1121,6 +1113,7 @@ extract_rust (FILE *f,
       ts_symbol_string_content     = ts_language_symbol ("string_content", true);
       ts_symbol_escape_sequence    = ts_language_symbol ("escape_sequence", true);
       ts_symbol_identifier         = ts_language_symbol ("identifier", true);
+      ts_symbol_scoped_identifier  = ts_language_symbol ("scoped_identifier", true);
       ts_symbol_call_expression    = ts_language_symbol ("call_expression", true);
       ts_symbol_macro_invocation   = ts_language_symbol ("macro_invocation", true);
       ts_symbol_arguments          = ts_language_symbol ("arguments", true);
@@ -1135,9 +1128,8 @@ extract_rust (FILE *f,
     }
 
   /* Read the file into memory.  */
-  char *contents_data;
   size_t contents_length;
-  contents_data = read_file (real_filename, 0, &contents_length);
+  char *contents_data = read_file (real_filename, 0, &contents_length);
   if (contents_data == NULL)
     error (EXIT_FAILURE, errno, _("error while reading \"%s\""),
            real_filename);
