@@ -1,7 +1,5 @@
 /* xgettext Vala backend.
-   Copyright (C) 2013-2024 Free Software Foundation, Inc.
-
-   This file was written by Daiki Ueno <ueno@gnu.org>, 2013.
+   Copyright (C) 2013-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,9 +14,9 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif
+/* Written by Peter Miller, Bruno Haible, and Daiki Ueno.  */
+
+#include <config.h>
 
 /* Specification.  */
 #include "x-vala.h"
@@ -30,6 +28,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#define SB_NO_APPENDF
 #include <error.h>
 #include "attribute.h"
 #include "message.h"
@@ -87,18 +86,16 @@ add_keyword (const char *name, hash_table *keywords)
     default_keywords = false;
   else
     {
-      const char *end;
-      struct callshape shape;
-      const char *colon;
-
       if (keywords->table == NULL)
         hash_init (keywords, 100);
 
+      const char *end;
+      struct callshape shape;
       split_keywordspec (name, &end, &shape);
 
       /* The characters between name and end should form a valid C identifier.
          A colon means an invalid parse in split_keywordspec().  */
-      colon = strchr (name, ':');
+      const char *colon = strchr (name, ':');
       if (colon == NULL || colon >= end)
         insert_keyword_callshape (keywords, name, end - name, &shape);
     }
@@ -308,7 +305,6 @@ static int
 phase2_getc ()
 {
   int c;
-  bool last_was_star;
 
   c = phase1_getc ();
   if (c != '/')
@@ -323,41 +319,43 @@ phase2_getc ()
     case '*':
       /* C comment.  */
       comment_start ();
-      last_was_star = false;
-      for (;;)
-        {
-          c = phase1_getc ();
-          if (c == EOF)
+      {
+        bool last_was_star = false;
+        for (;;)
+          {
+            c = phase1_getc ();
+            if (c == EOF)
+              break;
+            /* We skip all leading white space, but not EOLs.  */
+            if (!(buflen == 0 && (c == ' ' || c == '\t')))
+              comment_add (c);
+            switch (c)
+              {
+              case '\n':
+                comment_line_end (1);
+                comment_start ();
+                last_was_star = false;
+                continue;
+
+              case '*':
+                last_was_star = true;
+                continue;
+
+              case '/':
+                if (last_was_star)
+                  {
+                    comment_line_end (2);
+                    break;
+                  }
+                FALLTHROUGH;
+
+              default:
+                last_was_star = false;
+                continue;
+              }
             break;
-          /* We skip all leading white space, but not EOLs.  */
-          if (!(buflen == 0 && (c == ' ' || c == '\t')))
-            comment_add (c);
-          switch (c)
-            {
-            case '\n':
-              comment_line_end (1);
-              comment_start ();
-              last_was_star = false;
-              continue;
-
-            case '*':
-              last_was_star = true;
-              continue;
-
-            case '/':
-              if (last_was_star)
-                {
-                  comment_line_end (2);
-                  break;
-                }
-              FALLTHROUGH;
-
-            default:
-              last_was_star = false;
-              continue;
-            }
-          break;
-        }
+          }
+      }
       last_comment_line = line_number;
       return ' ';
 
@@ -466,7 +464,7 @@ free_token (token_ty *tp)
 static int
 get_string_element ()
 {
-  int c, j;
+  int c;
 
   /* Use phase 1, because phase 2 elides comments.  */
   c = phase1_getc ();
@@ -545,12 +543,8 @@ get_string_element ()
           break;
         }
       {
-        int n;
-        bool overflow;
-
-        n = 0;
-        overflow = false;
-
+        int n = 0;
+        bool overflow = false;
         for (;;)
           {
             switch (c)
@@ -591,10 +585,8 @@ get_string_element ()
 
     case '0':
       {
-        int n;
-
-        n = 0;
-        for (j = 0; j < 3; ++j)
+        int n = 0;
+        for (int j = 0; j < 3; ++j)
           {
             n = n * 8 + c - '0';
             c = phase1_getc ();
@@ -616,9 +608,9 @@ get_string_element ()
     case 'u':
       {
         unsigned char buf[8];
-        int n;
+        int j;
 
-        n = 0;
+        int n = 0;
         for (j = 0; j < 4; j++)
           {
             int c1 = phase1_getc ();
@@ -719,8 +711,6 @@ phase3_get (token_ty *tp)
 
   for (;;)
     {
-      bool template;
-      bool verbatim;
       int c;
 
       tp->line_number = line_number;
@@ -746,8 +736,8 @@ phase3_get (token_ty *tp)
         }
 
       last_non_comment_line = tp->line_number;
-      template = false;
-      verbatim = false;
+      bool template = false;
+      bool verbatim = false;
 
       switch (c)
         {
@@ -924,7 +914,6 @@ phase3_get (token_ty *tp)
             struct mixed_string_buffer msb;
             {
               int c2 = phase1_getc ();
-
               if (c2 == '"')
                 {
                   int c3 = phase1_getc ();
@@ -1274,13 +1263,13 @@ phase4_get (token_ty *tp)
       for (;;)
         {
           token_ty token2;
-
           phase3_get (&token2);
+
           if (token2.type == token_type_plus)
             {
               token_ty token3;
-
               phase3_get (&token3);
+
               if (token3.type == token_type_string_literal)
                 {
                   sum = mixed_string_concat_free1 (sum, token3.mixed_string);
@@ -1386,7 +1375,6 @@ extract_balanced (message_list_ty *mlp, token_type_ty delim,
   for (;;)
     {
       token_ty token;
-
       x_vala_lex (&token);
 
       switch (token.type)
@@ -1394,7 +1382,6 @@ extract_balanced (message_list_ty *mlp, token_type_ty delim,
         case token_type_symbol:
           {
             void *keyword_value;
-
             if (hash_find_entry (&keywords, token.string, strlen (token.string),
                                  &keyword_value)
                 == 0)
@@ -1411,7 +1398,7 @@ extract_balanced (message_list_ty *mlp, token_type_ty delim,
                 flag_context_list_table,
                 token.string, strlen (token.string)));
           free (token.string);
-          continue;
+          break;
 
         case token_type_lparen:
           if (++nesting_depth > MAX_NESTING_DEPTH)
@@ -1464,7 +1451,7 @@ extract_balanced (message_list_ty *mlp, token_type_ty delim,
 
           next_context_iter = null_context_list_iterator;
           state = 0;
-          continue;
+          break;
 
         case token_type_comma:
           arg++;
@@ -1473,7 +1460,7 @@ extract_balanced (message_list_ty *mlp, token_type_ty delim,
           inner_region = new_sub_region (outer_region, curr_context);
           next_context_iter = passthrough_context_list_iterator;
           state = 0;
-          continue;
+          break;
 
         case token_type_question:
           /* In an expression A ? B : C, each of A, B, C is a distinct
@@ -1486,7 +1473,7 @@ extract_balanced (message_list_ty *mlp, token_type_ty delim,
           inner_region = new_sub_region (outer_region, curr_context);
           next_context_iter = passthrough_context_list_iterator;
           state = 0;
-          continue;
+          break;
 
         case token_type_colon:
           /* In an expression A ? B : C, each of A, B, C is a distinct
@@ -1495,7 +1482,7 @@ extract_balanced (message_list_ty *mlp, token_type_ty delim,
           inner_region = new_sub_region (outer_region, curr_context);
           next_context_iter = passthrough_context_list_iterator;
           state = 0;
-          continue;
+          break;
 
         case token_type_assign:
           /* In an expression A = B, A and B are distinct sub-regions.
@@ -1505,7 +1492,7 @@ extract_balanced (message_list_ty *mlp, token_type_ty delim,
           inner_region = new_sub_region (outer_region, curr_context);
           next_context_iter = passthrough_context_list_iterator;
           state = 0;
-          continue;
+          break;
 
         case token_type_plus:
         case token_type_arithmetic_operator:
@@ -1524,7 +1511,7 @@ extract_balanced (message_list_ty *mlp, token_type_ty delim,
           inner_region->inherit_from_parent_region = false;
           next_context_iter = passthrough_context_list_iterator;
           state = 0;
-          continue;
+          break;
 
         case token_type_semicolon:
           arglist_parser_done (argparser, arg);
@@ -1539,7 +1526,6 @@ extract_balanced (message_list_ty *mlp, token_type_ty delim,
         case token_type_string_literal:
           {
             lex_pos_ty pos;
-
             pos.file_name = logical_file_name;
             pos.line_number = token.line_number;
 
@@ -1575,12 +1561,12 @@ extract_balanced (message_list_ty *mlp, token_type_ty delim,
           drop_reference (token.comment);
           next_context_iter = null_context_list_iterator;
           state = 0;
-          continue;
+          break;
 
         case token_type_return:
           next_context_iter = passthrough_context_list_iterator;
           state = 0;
-          continue;
+          break;
 
         case token_type_character_constant:
         case token_type_lbrace:
@@ -1591,7 +1577,7 @@ extract_balanced (message_list_ty *mlp, token_type_ty delim,
         case token_type_other:
           next_context_iter = null_context_list_iterator;
           state = 0;
-          continue;
+          break;
 
         default:
           abort ();
